      subroutine checkint4(numdist,nested,ifint4,ifint6,ipt,jpt)
 
c     Non-integrated residuals may enter in continuous and (simple/ordered/
c     multinomial) probit models only.  If they are correlated with residuals
c     in other model types, we need to compute the conditional distribution.
c     This is not possible for probit residuals, because a range of residual
c     values can lead to probit outcomes.  Put differently, no closed form
c     solution exists.
c
c     This subroutine checks that non-integrated probit residuals are not
c     correlated with residuals in other model types, except as non-integrated
c     residuals in continuous models.
c
c     This routine was triggered by Riccardo Borgoni's heckpois model
c     (Trouble\heckpois); also see my e-mail to him of March 2, 2003.
c
c     Constantijn Panis, 2 March 2003
 
      implicit none
      integer numdist,nested(numdist),ifint4(numdist),ifint6(numdist),
     *   ipt,jpt,i,intvecmax
 
c     If a residual enters integrated in a continuous model or enters at all
c     in any other model (which for currently model types always implies that
c     it enters as integrated residual), trigger an error message.
c     If a residual is integrated-out anywhere, nested(.) is nonzero.
      do 100 i=1,numdist
         if(ifint4(i).ne.-1)goto 100
         if(ifint6(i).eq.1)goto 800
         if(nested(i).gt.0)goto 810
         if(ifint6(i).eq.-1)then
c           Non-integrated probit and continuous residuals are correlated.
c           Not a problem except when continuous outcomes are evaluated inside
c           the integral and probit outcomes do not have any integrated
c           residual, so that their draw=0 and they are evaluated separately
c           from the continuous residuals.  E.g., Select\select1b.
            if(intvecmax(ifint6,numdist).eq.1.and.
     *         intvecmax(ifint4,numdist).ne.1)goto 820
         endif
 
  100 continue
 
      goto 999
 
  800 if(ipt.gt.0)write(8,801)
      if(jpt.gt.0)write(6,801)
  801 format(/,'ERROR:  your model contains a non-integrated probit ',
     *   'residual which is',/,'correlated with an integrated ',
     *   'residual in a continuous model.  No closed',/,
     *   'form solution of the likelihood exists.  Solution: either',/,
     *   'do not integrate-out the continuous residual or do ',
     *   'integrate-out the probit',/,'residual (but be aware that ',
     *   'probits need at least one non-integrated',/,'residual which ',
     *   'aML will add if it is not explicitly specified).')
      stop
  810 if(ipt.gt.0)write(8,811)
      if(jpt.gt.0)write(6,811)
  811 format(/,'ERROR:  your model contains a non-integrated probit ',
     *   'residual which is',/,'correlated with a residual in another ',
     *   'model type.  No closed form',/,'solution of the likelihood ',
     *   'exists.  Consider integrating-out the probit',/,'residual ',
     *   '(but be aware that probits need at least one ',
     *   'non-integrated',/,'residual which aML will add if it is not ',
     *   'explicitly specified).')
      stop
  820 if(ipt.gt.0)write(8,821)
      if(jpt.gt.0)write(6,821)
  821 format(/,'ERROR:  your model contains continuous outcomes with ',
     *   'integrated',/,'residuals that are correlated with probit ',
     *   'outcomes without integrated',/,'residuals.  This is ',
     *   'probably conceptually fine, but not allowed for',/,
     *   'internal technical reasons.  (Probit residuals need to be ',
     *   'conditioned on',/,'continuous outcomes, but probit outcomes ',
     *   'are evaluated outside the',/,'numerical integral that ',
     *   'evaluates continuous outcomes.)  Equivalently,',/,'please',//,
     *   '(1) change the integrated residuals in the continuous ',
     *   'models into non-',/,'integrated residuals; or',//,
     *   '(2) include a dummy integrated residual in the probit ',
     *   'models.  For',/,'example, a newly defined integrated ',
     *   'residual with one integration point',/,'and a standard ',
     *   'deviation that is not estimated (fixed).  (A residual',/,
     *   'with one integration point is approximated by zero and ',
     *   'has no effect.)')
      stop
  999 return
      end
