      subroutine codelamtyp(task,numdist,lamtyp)
 
c     Subroutine to summarize all eight load matrix types of a
c     particular distribution (corresponding to hazard, logit,
c     probit, continuous, binomial, negative binomial, Poisson, and
c     multinomial logit models) into a scalar, and to untransform also.
c
c     lamtyp may be -1 (not used), 0 (generic), 1 (identity),
c     2 (symmetric), 3 (block-diagonal), or 4 (sparse).  In
c     reality, 2 and 3 are not used for lamtyp.  The total number
c     of distinct codes is six, hence transformation into a base-6
c     scalar.  The highest scalar will be 6^8 = 1,679,616 < 2*10^9 = maxint().
c
c     This routine overwrites the old lamtyp(numdist,8) with new
c     values in the first numdist elements of lamtyp.
c
c     Constantijn Panis, 2 March 2004
c     Last change: fixed bug: lamtyp(i+(j-1)*numdist) rather than
c     lamtyp(j+(i-1)*numdist).  Caused troubles when numdist>8.
 
      implicit none
      integer task,numdist,lamtyp(numdist*8),base,i,j,temp
      parameter(base=6)
 
      if(task.eq.1)then
         do 100 i=1,numdist
            temp=0
            do 110 j=1,8
               temp=base*temp+lamtyp(i+(j-1)*numdist)+1
  110       continue
            lamtyp(i)=temp
  100    continue
      else
         do 200 i=numdist,1,-1
            temp=lamtyp(i)
            do 210 j=8,1,-1
               lamtyp(i+(j-1)*numdist) = mod(temp,base)-1
               temp=temp/base
  210       continue
  200    continue
      endif
 
  999 return
      end
