      subroutine define_dist(maxnumdist,maxref,numdist,sumdim,sumdim2,
     *   distdim,distyp,distn,intpnts,intpntr,distpos,
     *   distrestrpos,distrestr,beginrestr,fulltype,nchol,
     *   resname,ifull,nfull,resref,line,il,ipt,jpt)
 
c     Subroutine to define a distribution of a random variable.
c
c     The syntax for normal distributions is:
c     DEFINE NORMAL DISTRIBUTION;  DIM = n;
c        [NUMBER OF INTEGRATION POINTS = n [n...n];]
c        NAME = [name]; [REF = ...;]
c           :
c        NAME = [name]; [REF = ...;]
c        [RESTRICTIONS [sigma(n)=sigma(n) rho(n,n)=rho(n,n)];]
c        [SEARCH=CHOLESKY;]
c
c     Searching on the Cholesky decomposition pre-empts restrictions on
c     standard deviations and correlations.
c
c     Constantijn Panis, 12 November 2002
c     Last change: restrict number of integration points to <=171
 
      implicit none
      integer maxnumdist,maxref,numdist,sumdim,sumdim2,
     *   distdim(maxnumdist),distn(maxnumdist),intpnts(sumdim),
     *   intpntr(sumdim),distyp(maxnumdist),distpos(maxnumdist),
     *   distrestrpos(maxnumdist),beginrestr,
     *   distrestr(0:sumdim2),nfull,fulltype(nfull),nchol,
     *   ifull,resref(maxref),beginref,numname,nref,dim,i,n,np,
     *   il,i1,i2,ipt,jpt,match,status,intvecsum,intvecmax
      logical cholesky,smtxt,dupchar12,cmpstri
      character line*128,resname(sumdim)*12
 
      numdist=numdist+1
      if(numdist.gt.maxnumdist)call bug(ipt,jpt)
c     distyp=1 indicates a normal distribution:
      distyp(numdist)=1
c     Zero means that there are no restrictions (yet):
      distrestrpos(numdist)=0
      cholesky=.false.
      distpos(numdist)=ifull+1
      numname=intvecsum(distdim,numdist-1)
c     Locate begin point of distribution reference numbers:
      call find_br(-1,numdist,maxref,resref,beginref,ipt,jpt)
 
c     The first statement must specify the dimension:
      if(.not.smtxt(4,line,il,'DIM='))goto 800
      call getint(4,dim,line,il,status)
      if(status.gt.0)goto 800
      if(.not.smtxt(4,line,il,';'))goto 800
      distdim(numdist)=dim
c     If there are no restrictions, there are dim*(dim+1)/2 parameters:
      distn(numdist)=(dim*(dim+1))/2
 
c     Other statements may appear in any order, except that reference
c     numbers must immediately follow corresponding name statements.
  100 if(smtxt(4,line,il,'NAME ='))goto 110
      if(smtxt(4,line,il,'NUMber of INTegration POINTS ='))goto 130
      if(smtxt(4,line,il,'RESTRICTION'))goto 150
      if(smtxt(4,line,il,'SEARCH=CHOLESKY;'))then
         if(distrestrpos(numdist).ne.0)goto 800
         cholesky=.true.
         goto 100
      endif
 
c     Nothing matches.  Either end of definition or syntax error (which
c     will be caught by subroutine readctl.)
c     Cholesky-search won't make sense for dim=1 (actually, not even for dim=2)
      if(dim.eq.1)cholesky=.false.
      if(cholesky)nchol=nchol+(dim*(dim+1))/2
c     Check that names or reference numbers have been assigned:
      if(numname.ne.intvecsum(distdim,numdist))goto 800
c     Finish reference numbers with a -1, even if none are defined:
      resref(beginref)=-1
c     Impose domain restrictions on standard deviations and correlations:
      if(ifull+distn(numdist).gt.nfull)goto 810
      call impose_restr(dim,fulltype(ifull+1),cholesky,
     *   distrestrpos(numdist),distrestr)
c     Keep track of total number of parameters defined thus far:
      ifull=ifull+distn(numdist)
      goto 999
 
c     NAME = [name]; [REF = ...;]
  110 numname=numname+1
      if(smtxt(4,line,il,';'))then
         resname(numname)='(noname)'
      else
         call getword(4,line,il,i1,i2,status)
         if(status.gt.0)goto 800
         resname(numname)=line(i1:i2)
c        Check that name is unique:
         if(dupchar12(resname(numname),resname,numname-1))goto 860
         if(.not.smtxt(4,line,il,';'))goto 800
      endif
      nref=0
      if(smtxt(4,line,il,'REFerence='))then
         call readrefs(maxref,beginref,resref,nref,line,il,ipt,jpt)
      else
         if(resname(numname).eq.'(unnamed)')goto 800
         resref(beginref)=0
         beginref=beginref+1
      endif
      goto 100
 
c     NUMBER OF INTEGRATION POINTS = n [n...n];
c     Number of points are stored in intpnts(.).  This array contains one
c     element for every residual, regardless of whether the corresponding
c     distribution is integrated-out.  However, in amlfunc, etc, the array
c     elements should refer to integrated residuals only.  Prepscratch
c     compresses both intpnts and intpntr.
  130 n=intvecsum(distdim,numdist-1)+1
c     Check duplicate assignment:
      if(intpnts(n).gt.0)goto 800
      call getints(4,intpnts(n),dim,np,line,il,status)
      if(status.gt.0)goto 800
      if(intvecmax(intpnts(n),np).gt.171)goto 820
      if(np.eq.1.and.dim.gt.1)then
c        Number of points is equal for all dimensions:
         call putint(intpnts(n),intpnts(n+1),dim-1)
      elseif(np.ne.dim)then
         goto 800
      endif
c     Points and weights are computed in prepscratch
      goto 100
 
c     RESTRICTIONS
c     If two or more parameters are equated, only the first must
c     appear in the list of starting values.  Integer array distrestrpos
c     points at the first element of distrestr containing restrictions.
c     If there are no restrictions, distrestrpos=0.
c     See subroutines init_corr/read_restrict for restrictions codes.
  150 if(cmpstri(line(il:il),'S'))il=il+1
      if(cholesky)goto 800
c     Multiple RESTRICTION statements may appear:
      if(distrestrpos(numdist).eq.0)then
c        Initialize restrictions to be none:
         distrestrpos(numdist)=beginrestr
         if(beginrestr+dim*dim-1.gt.sumdim2)call bug(ipt,jpt)
         call init_corr(distrestr(beginrestr),dim,dim+1)
         beginrestr=beginrestr+dim*dim
      endif
      call read_restrict(dim,distrestr(distrestrpos(numdist)),dim+1,
     *   distn(numdist),line,il,ipt,jpt)
      goto 100
 
 
c     Syntax error.  If anything is changed here, also change scanctl.
  800 if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,801)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         write(6,801)
      endif
  801 format('ERROR encountered in the definition of a normal ',
     *   'distribution.  The syntax is:',//,
     *   3x,'DEFINE NORMAL DISTRIBUTION;  DIM = n;',/,
     *   6x,'[NUMBER OF INTEGRATION POINTS = n [n...n];]',/,
     *   6x,'NAME = [name]; [REF = ...;]',/,9x,':',/,
     *   6x,'NAME = [name]; [REF = ...;]',/,
     *   6x,'[RESTRICTIONS [sigma(n)=sigma(n) rho(n,n)=rho(n,n)];]',/,
     *   6x,'[SEARCH=CHOLESKY;]',//,
     *   'Note: each dimension must either be named or assigned ',
     *   'reference numbers.',/,'Note: searching on the ',
     *   'Cholesky decomposition can only be done if',/,6x,'there are ',
     *   'no restrictions on the distribution''s parameters.')
      stop
  810 if(ipt.gt.0)write(8,811)
      if(jpt.gt.0)write(6,811)
  811 format(/,'ERROR:  you provided too few starting values.')
      if(distn(numdist).lt.(dim*(dim+1))/2)then
         if(ipt.gt.0)call writeaia(8,72,1,'Hint:  the definition of '//
     *      'a normal distribution and its restriction(s) imply that '//
     *      'this distribution is defined by ',distn(numdist),
     *      ' standard deviations and correlations; did you '//
     *      'initialize them all?')
         if(jpt.gt.0)call writeaia(6,72,1,'Hint:  the definition of '//
     *      'a normal distribution and its restriction(s) imply that '//
     *      'this distribution is defined by ',distn(numdist),
     *      ' standard deviations and correlations; did you '//
     *      'initialize them all?')
      elseif(dim.gt.2)then
         if(ipt.gt.0)call writeaia(8,72,1,'Hint:  did you initialize '//
     *      'all ',distn(numdist),' standard deviations and '//
     *      'correlations of one of your distributions?')
         if(jpt.gt.0)call writeaia(6,72,1,'Hint:  did you initialize '//
     *      'all ',distn(numdist),' standard deviations and '//
     *      'correlations of one of your distributions?')
      endif
      stop
  820 if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,821)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         write(6,821)
      endif
  821 format(/,'ERROR:  The maximum number of integration points may ',
     *   'not exceed 171',/,'because of floating point limitations.')
      stop
  860 if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,861)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         write(6,861)
      endif
  861 format('ERROR in NORMAL DISTRIBUTION definition.',/,
     *   'You assigned duplicate residual names.')
      stop
  999 return
      end
