      subroutine define_fm(nfull,maxnumdist,maxref,numdist,numpar,
     *   sumdim,intpnts,distdim,distyp,distpos,distrestrpos,pardim,
     *   parpos,fulltype,resname,parname,resref,line,il,ipt,jpt)
 
c     Subroutine to define an asymmetric finite mixture distribution.
c     The syntax is:
c
c     DEFINE FINITE MIXTURE DISTRIBUTION;  [DIM = 1;]
c        FORM = ASYMMETRIC;
c        POINTS = vector;  WEIGHTS = vector;
c        NAME = [name]; [REF = ...;]
c
c     Constantijn Panis, 1 February 2000
c     Last change: reference numbers
 
      implicit none
      integer nfull,maxnumdist,maxref,numdist,numpar,sumdim,
     *   intpnts(sumdim),distdim(maxnumdist),
     *   distyp(maxnumdist),distpos(maxnumdist),
     *   distrestrpos(maxnumdist),pardim(numpar),parpos(numpar),
     *   fulltype(*),resref(maxref),beginref,numname,nref,dim,
     *   il,i1,i2,n,n1,n2,ipt,jpt,status,intvecsum
      logical smtxt,dupchar12
      character line*128,resname(*)*12,parname(numpar)*12
 
      n1=0
      n2=0
 
      numdist=numdist+1
      if(numdist.gt.maxnumdist)call bug(ipt,jpt)
      distdim(numdist)=1
c     distyp=10000*n1+10*n2+4  where n1 is vector of points and n2 of weights.
      distyp(numdist)=4
      distrestrpos(numdist)=0
c     distpos=10000*n1+n2  where n1 is position of points and n2 of weights.
      distpos(numdist)=0
      numname=intvecsum(distdim,numdist-1)
c     Locate begin point of distribution reference numbers:
      call find_br(-1,numdist,maxref,resref,beginref,ipt,jpt)
 
  100 if(smtxt(4,line,il,'DIMension='))goto 110
      if(smtxt(4,line,il,'NAME ='))goto 120
      if(smtxt(4,line,il,'POINTS='))goto 130
      if(smtxt(4,line,il,'WEIGHTS='))goto 140
      if(smtxt(4,line,il,'FORM='))goto 150
 
c     Nothing matches.  Either end of definition or syntax error.
c     Check that names or reference numbers have been assigned:
      if(numname.ne.intvecsum(distdim,numdist))goto 800
c     Finish reference numbers with a -1, even if none are defined:
      resref(beginref)=-1
c     Finite mixture does not introduce new parameters, so do not
c     increment ifull.
c     Check both points and weights have been specified:
c     distyp=10000*n1+10*n2+4  where n1 is vector of points and n2 of weights.
      if(distyp(numdist)/10000.eq.0.or.
     *   mod(distyp(numdist),10000)/10.eq.0)goto 800
c     Check that the vector with points has dimension one higher
c     than the vector used for weights:
      if(pardim(distyp(numdist)/10000).ne.
     *   pardim(mod(distyp(numdist),10000)/10)+1)goto 840
      goto 999
 
c     DIM = 1;
  110 call getint(4,dim,line,il,status)
      if(status.gt.0)goto 800
      if(.not.smtxt(4,line,il,';'))goto 800
      if(dim.ne.1)goto 800
      goto 100
 
c     NAME = [name]; [REF = ...;]
  120 numname=numname+1
      if(smtxt(4,line,il,';'))then
         resname(numname)='(noname)'
      else
         call getword(4,line,il,i1,i2,status)
         if(status.gt.0)goto 800
         resname(numname)=line(i1:i2)
c        Check that name is unique:
         if(dupchar12(resname(numname),resname,numname-1))goto 820
         if(.not.smtxt(4,line,il,';'))goto 800
      endif
      nref=0
      if(smtxt(4,line,il,'REFerence='))then
         call readrefs(maxref,beginref,resref,nref,line,il,ipt,jpt)
      else
         if(resname(numname).eq.'(unnamed)')goto 800
         resref(beginref)=0
         beginref=beginref+1
      endif
      goto 100
 
c     POINTS = vector;
  130 call getword(4,line,il,i1,i2,status)
      if(status.gt.0)goto 800
      if(.not.smtxt(4,line,il,';'))goto 800
      call char12match(line(i1:i2),parname,numpar,n1)
      if(n1.eq.0)goto 850
      if(pardim(n1).gt.nfull)goto 850
c     Make sure the vector is strictly increasing
      if(fulltype(parpos(n1)).ne.41)goto 870
      distyp(numdist)=distyp(numdist)+10000*n1
      n=intvecsum(distdim,numdist-1)+1
      intpnts(n)=pardim(n1)
c     Note that intpntr() remains zero.
c     distpos=10000*n1+n2  where n1 is position of points and n2 of weights.
      distpos(numdist)=distpos(numdist)+10000*parpos(n1)
      goto 100
 
c     WEIGHTS = vector;
  140 call getword(4,line,il,i1,i2,status)
      if(status.gt.0)goto 800
      if(.not.smtxt(4,line,il,';'))goto 800
      call char12match(line(i1:i2),parname,numpar,n2)
      if(n2.eq.0)goto 850
      if(pardim(n2).gt.nfull)goto 850
c     Make sure the vector is strictly increasing
      if(fulltype(parpos(n2)).ne.41.and.pardim(n2).gt.1)goto 880
      distyp(numdist)=distyp(numdist)+10*n2
c     distpos=10000*n1+n2  where n1 is position of points and n2 of weights.
      distpos(numdist)=distpos(numdist)+parpos(n2)
      goto 100
 
c     FORM=ASYMMETRIC;
  150 if(.not.smtxt(4,line,il,'ASYMmetric;'))goto 860
      goto 100
 
 
c     Syntax error.  If anything is changed here, also change in scanctl.
  800 if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,801)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         write(6,801)
      endif
  801 format(/,'ERROR encountered in the definition of a finite ',
     *   'mixture',/,'definition.  The syntax is:',//,
     *   'DEFINE FINITE MIXTURE DISTRIBUTION;  ',
     *   'DIM = 1;',/,3x,'FORM = ASYMMETRIC;',/,
     *   3x,'POINTS = vector;  WEIGHTS = vector;',/,
     *   3x,'NAME = [name]; [REF = ...;]')
      stop
  820 if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,821)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         write(6,821)
      endif
  821 format(/,'ERROR in FINITE MIXTURE DISTRIBUTION definition.',/,
     *   'You assigned duplicate residual names.')
      stop
c     distyp=10000*n1+10*n2+4  where n1 is vector of points and n2 of weights.
  840 n1=pardim(distyp(numdist)/10000)
      n2=pardim(mod(distyp(numdist),10000)/10)
      if(ipt.gt.0)then
         call caretline(8,line,il)
         call writea4ia(8,72,1,'ERROR in the definition of a '//
     *      'finite mixture distribution.  The vectors with mixture '//
     *      'points and weights have dimensions ',n1,' and ',n2,
     *      ', respectively.  However, since weights are restricted '//
     *      'to add up to one, there may only be ',n1,'-1=',n1-1,
     *      ' weights.')
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         call writea4ia(6,72,1,'ERROR in the definition of a '//
     *      'finite mixture distribution.  The vectors with mixture '//
     *      'points and weights have dimensions ',n1,' and ',n2,
     *      ', respectively.  However, since weights are restricted '//
     *      'to add up to one, there may only be ',n1,'-1=',n1-1,
     *      ' weights.')
      endif
      stop
  850 if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,851)line(i1:i2)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         write(6,851)line(i1:i2)
      endif
  851 format('ERROR:  vector ''',a,''' not defined.')
      stop
  860 if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,861)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         write(6,861)
      endif
  861 format('ERROR:  only asymmetric finite mixtures are currently ',
     *   'supported.')
      stop
  870 if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,871)'support points',line(i1:i2)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         write(6,871)'support points',line(i1:i2)
      endif
  871 format('ERROR:  the vector used for ',a,' in a finite ',
     *  'mixture',/,'distribution (',a,') must be strictly increasing.',
     *   /,'Specify option "INCREASING;" in the vector definition.')
      stop
  880 if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,871)'weights',line(i1:i2)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         write(6,871)'weights',line(i1:i2)
      endif
      stop
  999 return
      end
