      subroutine define_par(nfull,ifull,numpar,maxnumpar,
     *   partype,maxref,pardim,parpos,parref,fulltype,fullbeta,
     *   parname,line,il,ipt,jpt)
 
c     Subroutine to define a parameter, vector, or matrix.
c     The syntax is:
c
c     DEFINE PARAMETER [name];  [REF = [n...n];]
c        [RANGE = {(0,Inf) | (-1,1) | (0,1)};]
c
c     DEFINE VECTOR [name];  [REF = [n...n];]
c        DIM = n;
c        [INCREASING = {YES|NO};]
c        [RANGE = {(0,Inf) | (-1,1) | (0,1)};]
c        [INITIAL = {GHQ WEIGHTS | GHQ POINTS(STD=x)};]
c
c     DEFINE MATRIX name; DIM=(n1,n2);
c
c     The types are distinguished by
c        partype=1 (parameter)
c        partype=2 (vector)
c        partype=3 (matrix)
c
c     pardim contains the dimension.  It is 1 for scalar parameters,
c     dim for a vector and -(dim1*nfull+dim2) for matrices, so that
c        dim1 = abs(pardim)/nfull
c        dim2 = mod(abs(pardim),nfull)
c     (Note that dim=0 is never allowed.)
c     Note that the negative pardim for matrices is set to a positive
c     value if an element of the inverse matrix is ever used.  This
c     is done in subroutine read_par.  This feature flags whether
c     the inverse needs to be computed in subroutine prepinv.
c
c     Constantijn Panis, 24 January 2000
c     Last change: vectors are increasing by default
 
      implicit none
      integer nfull,ifull,numpar,maxnumpar,partype,maxref,
     *   pardim(maxnumpar),parpos(maxnumpar),parref(maxref),
     *   fulltype(nfull),maxghdim,initial,dim,beginref,range,
     *   flag,il,i1,i2,i,nref,ipt,jpt,status,trm
      parameter(maxghdim=10)
      double precision fullbeta(nfull),z(maxghdim),wt(maxghdim),
     *   std,prob,invnrm
      character line*128,parname(maxnumpar)*12
      logical increasing,smtxt,dupchar12,cmpstri
 
      numpar=numpar+1
      if(numpar.gt.maxnumpar)call bug(ipt,jpt)
      parpos(numpar)=ifull+1
c     Locate begin point of reference numbers:
      call find_br(0,numpar,maxref,parref,beginref,ipt,jpt)
 
      call getword(4,line,il,i1,i2,status)
      if(status.gt.0)goto 800
      if(line(i1:i2).eq.';')then
c        Unnamed (but referenced) parameters and vectors are OK.
         if(partype.eq.3)goto 800
         parname(numpar)='(unnamed)'
      else
         parname(numpar)=line(i1:i2)
         if(.not.smtxt(4,line,il,';'))goto 800
         if(dupchar12(parname(numpar),parname,numpar-1))goto 810
         if(cmpstri(parname(numpar)(1:3),'INV'))goto 820
      endif
 
      dim=0
      nref=0
      pardim(numpar)=1
      increasing=.true.
      range=0
      initial=0
  100 if(smtxt(4,line,il,'REFerence='))goto 110
      if(smtxt(4,line,il,'RANGE='))goto 120
      if(smtxt(4,line,il,'DIMension='))goto 130
      if(smtxt(4,line,il,'INCREASING'))goto 140
      if(smtxt(4,line,il,'INITIAL='))goto 150
      goto 200
 
  110 if(nref.ne.0)goto 800
      if(partype.eq.3)goto 800
      call readrefs(maxref,beginref,parref,nref,line,il,ipt,jpt)
      goto 100
 
c     RANGE = {(0,Inf) | (-1,1) | (0,1)};
  120 if(range.ne.0)goto 800
      if(partype.eq.3)goto 800
      if(smtxt(4,line,il,';'))then
         range=0
      elseif(smtxt(4,line,il,'(0,INF);'))then
         range=1
      elseif(smtxt(4,line,il,'(-1,1);'))then
         range=2
      elseif(smtxt(4,line,il,'(0,1);'))then
         range=3
      elseif(smtxt(4,line,il,'(0,+INF);'))then
         range=1
      else
         goto 850
      endif
      goto 100
 
c     DIM = n;
  130 if(partype.eq.1)then
         goto 800
      elseif(partype.eq.2)then
         call getint(4,dim,line,il,status)
         if(status.gt.0)goto 800
         if(dim.le.0)goto 800
         pardim(numpar)=dim
      elseif(partype.eq.3)then
         if(.not.smtxt(4,line,il,'('))goto 800
         call getint(4,dim,line,il,status)
         if(status.gt.0)goto 800
         if(dim.le.0)goto 800
         pardim(numpar)=nfull*dim
         call getint(4,dim,line,il,status)
         if(status.gt.0)goto 800
         if(dim.le.0)goto 800
         if(.not.smtxt(4,line,il,')'))goto 800
         pardim(numpar)=pardim(numpar)+dim
c        Set to negative number, to be changed to positive if an
c        element of the inverse matrix is ever used (see read_par).
         pardim(numpar)=-pardim(numpar)
      endif
      if(.not.smtxt(4,line,il,';'))goto 800
      goto 100
 
c     INCREASING ={yes|no};
  140 if(partype.ne.2)goto 800
      call semiyesno(flag,4,line,il,ipt,jpt)
      increasing=(flag.eq.1)
      goto 100
 
c     INITIAL = {GHQ WEIGHTS | GHQ POINTS(STD=x)};
  150 if(partype.ne.2)goto 800
      if(smtxt(4,line,il,';'))then
         initial=0
      elseif(smtxt(4,line,il,'GHQ WEIGHTS;'))then
         initial=1
      elseif(smtxt(4,line,il,'GHQ POINTS(STD='))then
         initial=2
         call getdp(4,std,line,il,status)
         if(status.gt.0)goto 800
         if(std.le.0.d0)goto 800
         if(.not.smtxt(4,line,il,');'))goto 800
      else
         goto 800
      endif
      goto 100
 
c     Parameters must either be named or assigned reference numbers.
  200 if(parname(numpar).eq.'(unnamed)'.and.nref.eq.0)goto 800
c     Vectors and matrices must have a DIM= statement
      if(partype.ne.1.and.dim.eq.0)goto 800
c     By default, vectors are increasing.  This may not be combined with
c     RANGE=(-1,1) or RANGE=(0,1):
      if(partype.eq.2.and.increasing.and.(range.eq.2.or.range.eq.3))
     *   goto 860
c     Only vectors may be initialized automatically
      if(initial.ne.0.and.dim.eq.0)goto 800
c     No domain restrictions or automatic initialization for matrices:
      if(partype.eq.3)goto 400
      dim=pardim(numpar)
c     Impose domain restrictions:
      if(partype.eq.1.or.partype.eq.2)then
         call putint(range,fulltype(ifull+1),dim)
         if(increasing.and.dim.gt.1)then
            fulltype(ifull+1)=41
            call putint(4,fulltype(ifull+2),dim-1)
         endif
      endif
 
c     If parameter values are automatically initialized, do so here:
      if(initial.eq.1)then
c        Gauss-Hermite weights, transformed into ordered probit thresholds
         if(dim/2+1.gt.maxghdim)goto 870
         call ghquad(2,dim+1,z,wt)
c        Convert weights into cumulative weights, then into thresholds:
         prob=0.d0
         do 300 i=1,(dim+1)/2
            prob=prob+wt(i)
            if(2*i.eq.dim+1)then
               fullbeta(ifull+i)=0.d0
            else
               fullbeta(ifull+i)=invnrm(prob)
               fullbeta(ifull+dim-i+1)=-fullbeta(ifull+i)
            endif
  300    continue
      elseif(initial.eq.2)then
c        Gauss-Hermite points
         if((dim+1)/2.gt.maxghdim)goto 870
         call ghquad(2,dim,z,wt)
c        Copy points into starting values vector, scaled by std:
         do 310 i=1,(dim+1)/2
            fullbeta(ifull+i)=std*z(i)
            fullbeta(ifull+dim-i+1)=-std*z(i)
  310    continue
      endif
 
c     Update total number of parameters in model:
  400 if(partype.eq.1)then
         ifull=ifull+1
      elseif(partype.eq.2)then
         ifull=ifull+dim
      elseif(partype.eq.3)then
c        dim1 = abs(pardim)/nfull
c        dim2 = mod(abs(pardim),nfull)
         ifull=ifull + (abs(pardim(numpar))/nfull)*
     *      mod(abs(pardim(numpar)),nfull)
c        Note that an additional dim1*dim2 is allocated to fullbeta
c        to store the inverse matrix.  The required space (nfullplus)
c        was determined in scanctl.
      endif
      goto 999
 
c     Syntax error.  If anything is changed here, also change scanctl.
  800 if(ipt.gt.0)then
         call caretline(8,line,il)
         if(partype.eq.1)write(8,801)
         if(partype.eq.2)write(8,802)
         if(partype.eq.3)write(8,803)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         if(partype.eq.1)write(6,801)
         if(partype.eq.2)write(6,802)
         if(partype.eq.3)write(6,803)
      endif
  801 format('ERROR encountered in a PARAMETER definition.  The ',
     *   'syntax is:',//,
     *   6x,'DEFINE PARAMETER [name];  [REF = [n...n];]',/,
     *   9x,'[RANGE = {(0,Inf) | (-1,1) | (0,1)};]')
  802 format('ERROR encountered in a VECTOR definition.  The ',
     *   'syntax is:',//,
     *   6x,'DEFINE VECTOR [name];  [REF = [n...n];]',/,
     *   9x,'DIM = n;',/,
     *   9x,'[INCREASING = {yes|no};]',/,
     *   9x,'[RANGE = {(0,Inf) | (-1,1) | (0,1)};]',/,
     *   9x,'[INITIAL = {GHQ WEIGHTS | GHQ POINTS(STD=x)};]')
  803 format('ERROR encountered in a MATRIX definition.  The ',
     *   'syntax is:',//,
     *   6x,'DEFINE MATRIX matrixname;  DIM = (n1,n2);')
      stop
  810 if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,811)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         write(6,811)
      endif
  811 format('ERROR: you assigned duplicate names to parameter, ',
     *   'vector and/or',/,'matrix definitions.')
      stop
  820 if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,821)parname(numpar)(1:trm(parname(numpar)))
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         write(6,821)parname(numpar)(1:trm(parname(numpar)))
      endif
  821 format('ERROR:  illegal name, ''',a,'''.',/,
     *   'Parameter, vector, and matrix names may not start with ',
     *   '''INV''',/,'to prevent conflicts with references to ',
     *   'inverse matrix elements.')
      stop
  850 if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,851)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         write(6,851)
      endif
  851 format('ERROR in PARAMETER or VECTOR definition.  The ',
     *   'range restrictions',/,
     *   'currently supported are:  RANGE = (0,Inf);',/,
     *   26x,'RANGE = (-1,1);',/,26x,'RANGE = (0,1);')
      stop
  860 if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,861)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         write(6,861)
      endif
  861 format('ERROR encountered in a VECTOR definition.  RANGE ',
     *   'restrictions (-1,1)',/,'and (0,1) may only be specified ',
     *   'in combination with INCREASING=NO.')
      stop
  870 if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,871)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         write(6,871)
      endif
  871 format('ERROR:  unable to automatically generate starting ',
     *   'values for a vector.',/,'Please specify starting ',
     *   'values explicitly.')
      stop
  999 return
      end
