      subroutine evalexpr(nexpr,maxxnode,xdim,xnodepntr,nold,levelmax,
     *   maxlevel,nbdim,nb,branch,xpntr,nxold,namepntr,xnode,x,outcome,
     *   oldname,struc,iobs,id,ipt,jpt,status)
 
c     Subroutine to evaluate one or more expressions.
c
c     Note: operands are recorded in xnode and scr by two real
c     numbers.  The first is opernext; the second indicates the
c     operand (see getoper):
c
c     11 = uniform()
c     20 = (
c     21 = exp(
c     22 = log(
c     23 = abs(
c     24 = sqrt(
c     25 = dummy(
c     26 = spline(
c     28 = min(
c     29 = max(
c     30 = int(
c     39 = )
c     40 = ^
c     41 = *
c     42 = /
c     43 = +
c     44 = -
c     50 = ==
c     51 = <
c     52 = <=
c     53 = >
c     54 = >=
c     55 = !=
c     60 = and
c     61 = or
c     62 = not
c     70 = ,
c     71 = node=  or  nodes=
c     80 = begin of an array (formed by spline)
c     81 = end of an array
c
c     The following error status info is returned (see findvar):
c     status [out] 0 = no error
c                  1 = variable in expression not found
c                  2 = variable found but not at the mandatory -maxlevel
c                  3 = variable found, at level=maxlevel+1
c                      (Note: useful for hazard model error diagnosis)
c                  4 = variable found, at level>maxlevel+1
c
c     Constantijn Panis, 8 October 2002
c     Last change: int() and uniform() added
 
      implicit none
      integer dim,nexpr,maxxnode,xdim,xnodepntr,nold,level,varpos
      parameter(dim=100)
      integer nbdim,levelmax,maxlevel,struc,iobs,id,
     *   ipt,jpt,nb(nbdim,2),branch(levelmax),xpntr(2:levelmax),
     *   nxold(levelmax),namepntr(levelmax),numnodes,status,
     *   iexpr,sp,xp,xpbegin,exprdim,first,last,match,i,n,xpos,trm
      real x(xdim),xnode(maxxnode),outcome(nexpr),scr(dim),varnext,
     *   opernext,enditem,endexpr,endexprs,node1,nodelow,nodehigh,
     *   temp1,temp2,a,b,c,rand
      character oldname(nold)*8,name*8,string*60,idstring*60
 
      status=0
      iexpr=0
      xp=xnodepntr
 
c     Surrounding the expression by parentheses, copy xnode into
c     scr, and replace variables by scalars.
  100 if(iexpr.ge.nexpr)call bug(ipt,jpt)
 
c--------------------------------------------------------------------
c     Copy expression from xnode into scr.  Take a short-cut if the
c     expression is just a variable or a real number.
      if(xnode(xp).eq.varnext().and.
     *  (xnode(xp+2).eq.endexpr().or.xnode(xp+2).eq.endexprs()))then
c        Take a shortcut if the expression is just a variable (incl uniform())
         varpos=int(xnode(xp+1))
         if(varpos.eq.-1)then
            scr(1)=rand(0)
         elseif(varpos.le.nold)then
c           Only the name of the variable, oldname(varpos), is known;
c           figure out what level and position, then find value.
            call findvar(levelmax,maxlevel,level,oldname(varpos),
     *         oldname,namepntr,nxold,varpos,oldname,-1,struc,
     *         ipt,jpt,status)
            if(status.gt.0)goto 999
            n=xpos(level,nbdim,nb,branch,xpntr,nxold,varpos)
            scr(1)=x(n)
         else
            level=varpos/(nold+1)
            varpos=mod(varpos,nold+1)
            if(level.gt.maxlevel)goto 840
            n=xpos(level,nbdim,nb,branch,xpntr,nxold,varpos)
            scr(1)=x(n)
         endif
         xp=xp+2
         goto 500
      elseif(xnode(xp+1).eq.endexpr().or.xnode(xp+1).eq.endexprs())then
c        Take a shortcut if the expression is just a number:
         scr(1)=xnode(xp)
         xp=xp+1
         goto 500
      endif
 
c     Otherwise open parenthesis, copy xnode into scr (while evaluating
c     variables) and close the parenthesis.
      xpbegin=xp
      scr(1)=opernext()
      scr(2)=20
      sp=2
 
c     New expression at endexpr; stop evaluation at endexprs
  110 if(sp+1.gt.dim)goto 810
      if(xnode(xp).eq.endexpr().or.xnode(xp).eq.endexprs())then
c        End-of-expression; close parentheses
         sp=sp+1
         scr(sp)=opernext()
         sp=sp+1
         scr(sp)=39
         exprdim=sp
         first=1
         goto 200
      elseif(xnode(xp).eq.varnext())then
c        Following is a variable or uniform().  May already know data structure
c        and level and varpos, or may not.
         xp=xp+1
         sp=sp+1
         varpos=int(xnode(xp))
         if(varpos.eq.-1)then
            scr(sp)=rand(0)
         elseif(varpos.le.nold)then
c           Only the name of the variable, oldname(xnode), is known;
c           figure out what level and position, then find value.
            call findvar(levelmax,maxlevel,level,
     *         oldname(varpos),oldname,namepntr,nxold,
     *         varpos,oldname,-1,struc,ipt,jpt,status)
            if(status.gt.0)goto 999
            n=xpos(level,nbdim,nb,branch,xpntr,nxold,varpos)
            scr(sp)=x(n)
         else
            level=int(xnode(xp))/(nold+1)
            varpos=mod(int(xnode(xp)),nold+1)
            if(level.gt.maxlevel)goto 840
            n=xpos(level,nbdim,nb,branch,xpntr,nxold,varpos)
            scr(sp)=x(n)
         endif
      else
         sp=sp+1
         scr(sp)=xnode(xp)
      endif
      xp=xp+1
      goto 110
c--------------------------------------------------------------------
 
c     scr contains operands and scalars only.  For example,
c     .5*(log(x+y)-z^2) translates into:
c
c     +---+----+----+---+----+---+----+---+---+----+---+---+----+---+----+---+--
c     | X | 20 | .5 | X | 20 | X | 22 | a | X | 43 | b | X | 39 | X | 44 | c | X
c     +---+----+----+---+----+---+----+---+---+----+---+---+----+---+----+---+--
c       1    2    3   4    5   6    7   8   9   10   11  12  13   14  15   16  1
c
c     where x, y, and z evaluate to a, b, and c, respectively.
c     At this stage, exprdim=21.
c
c     The expression will evolve as follows.  Assume a=1, b=2, c=3:
c           (.5*(log(1+2)-3^2))
c           (.5*(log(3)-3^2))
c           (.5*(1.0986-3^2))
c           (.5*(1.0986-9))
c           (.5*(-7.9014))
c           (.5*-7.9014)
c           (-3.9507)
c           -3.9507
c
c     The procedure is to repeatedly find the first closing parenthesis
c     and evaluate.  A closing parenthesis is indicated by opernext+39;
c     integer variable last denotes the position of that X (opernext);
c     integer variable first denotes the position of the X preceding it.
c     That X may be followed by a 20=(; 21=exp(; 22=log(; 23=abs; etc.
c     For example, the first subexpression log(1+2) is:
c
c     +---+---+---+---+---+---+----+---+---+----+---+---+----+---+---+
c     | . | . | . | . | . | X | 22 | 1 | X | 43 | 2 | X | 39 | . | . |
c     +---+---+---+---+---+---+----+---+---+----+---+---+----+---+---+
c       1   2   3   4   5   6    7   8   9   10   11  12  13
c
c     where first=6 last=12 exprdim=21.  The routine first searches
c     for X43=+ and X44=- which only serve to preserve or invert signs,
c     but does not find any.  Then it searches for X40=^; then for X41=*;
c     then for X42=/; then for X43=+ bingo, integer variable sp=9.
c     Add scr(sp-1) and scr(sp+2) and place in scr(sp-1).  Move
c     scr(sp+3) through scr(exprdim) to scr(sp) through scr(exprdim-3);
c     adjust last=last-3 and exprdim=exprdim-3, but leave sp unchanged:
c
c     +---+---+---+---+---+---+----+---+---+----+---+---+----+---+---+
c     | . | . | . | . | . | X | 22 | 3 | X | 39 | . | . |  . | . | . |
c     +---+---+---+---+---+---+----+---+---+----+---+---+----+---+---+
c       1   2   3   4   5   6    7   8   9   10   11  12  13
c
c     Then check whether last=first+3, indicating that everything in
c     the parentheses has been evaluated.  If anything remains, keep
c     going with the evaluation until last=first+3.  Then jump to the
c     evaluation of the parentheses themselves, here the log(.).
c     Place log(3) into scr(first), adjust exprdim=exprdim-4 and find
c     the next closing parenthesis.  Continue until there are no more
c     parentheses.
c
c     Splines are different.  A spline with n nodes evaluates to an
c     array with n+1 elements.  These elements are delimited by X80
c     and X81.  For example, spline(x, 10 40) with x=25 evaluates to
c
c        spline(25, 10 40)
c        { 10 15 0 }
c
c     +---+---+---+----+----+----+----+---+----+---+---+---+
c     | . | . | X | 26 | 25 | 10 | 40 | X | 39 | . | . | . |
c     +---+---+---+----+----+----+----+---+----+---+---+---+
c       1   2   3    4    5    6    7   8    9  10  11  12
c
c     +---+---+---+----+----+----+---+---+----+---+---+---+
c     | . | . | X | 80 | 10 | 15 | 0 | X | 81 | . | . | . |
c     +---+---+---+----+----+----+---+---+----+---+---+---+
c       1   2   3    4    5    6   7   8    9  10  11  12
c
c     Note that the number of elements does not change.
c     Splines may be interacted with some, not all, operands.
c     The following is OK, all other things are not:
c         -spline(...)
c         a*spline(...)
c         spline(...)*a
c         spline(...)/a
c         spline(...) + spline(...) with same number of nodes
c
c     Another peculiarity of splines which happens to work out OK is
c     that no parentheses are required around subexpressions.  E.g.,
c        spline(age-14, 1/2 25+x)
c     contains three subexpressions.  They are evaluated until (say
c     age=20 and x=5):
c        spline(6 .5 30)
c     which is where the spline is unraveled.
 
  200 continue
      do 201 last=3,exprdim-1
         if(scr(last+1).eq.39.and.scr(last).eq.opernext())goto 202
  201 continue
c     Expression evaluated
      goto 500
c     Find the corresponding opening parentheses
  202 do 203 first=last-2,1,-1
         if(scr(first).eq.opernext().and.
     *      20.le.scr(first+1).and.scr(first+1).lt.39)goto 220
  203 continue
      call bug(ipt,jpt)
c     Parentheses open at first and close at last.
c     Check for empty expression:
  220 if(last.eq.first+2)goto 830
c     Check for unbalanced parentheses:
      if(first.eq.1.and.last.ne.exprdim-1)call bug(ipt,jpt)
      if(first.ne.1.and.last.eq.exprdim-1)call bug(ipt,jpt)
c     Ready for resolution of parentheses?
      if(last.eq.first+3)goto 380
      if(scr(first+2).eq.opernext().and.scr(first+3).eq.80.and.
     *   scr(last-2).eq.opernext().and.scr(last-1).eq.81)goto 380
 
c--------------------------------------------------------------------
c     First resolve +=43 and -=44 signs without first operand, as in
c     +a*-b = a*(-b).  Note that there are no opening or closing
c     parentheses between first and last, i.e., no need to account
c     for (...)+x or (...)-x, except for spline arrays: +{...}
c     and -{...}.  There may, however, be enditems.
      sp=first+2
  230 if(sp.ge.last-2)goto 240
      if(scr(sp).ne.opernext())then
         sp=sp+1
      elseif(scr(sp+1).eq.43)then
         if(scr(sp+2).eq.opernext().and.
     *      scr(sp+3).ne.80.and.scr(sp+3).ne.44)goto 830
         if(sp.eq.first+2.or.scr(sp-1).eq.enditem().or.
     *      (scr(sp-2).eq.opernext().and.scr(sp-1).ne.81))then
            call copy4(scr(sp+2),scr(sp),exprdim-sp-1)
            exprdim=exprdim-2
            last=last-2
         else
            sp=sp+2
         endif
      elseif(scr(sp+1).eq.44)then
         if(scr(sp+2).eq.opernext().and.
     *      scr(sp+3).ne.80.and.scr(sp+3).ne.43)goto 830
         if(sp.eq.first+2.or.scr(sp-1).eq.enditem().or.
     *      (scr(sp-2).eq.opernext().and.scr(sp-1).ne.81))then
            if(scr(sp+2).eq.opernext().and.scr(sp+3).eq.80)then
c              Entire array is negated, -{...}
               call realmatch(opernext(),scr(sp+4),last-sp-5,match)
               if(match.eq.0.or.scr(sp+match+4).ne.81)call bug(ipt,jpt)
               call opposite4(scr(sp+4),scr(sp+4),match-1)
            else
               scr(sp+2)=-scr(sp+2)
            endif
            call copy4(scr(sp+2),scr(sp),exprdim-sp-1)
            exprdim=exprdim-2
            last=last-2
         else
            sp=sp+2
         endif
      else
         sp=sp+2
      endif
      goto 230
 
c     Resolve ^=40, if any
  240 if(last.eq.first+3)goto 380
      sp=first+2
  250 if(sp.ge.last-1)goto 260
      if(scr(sp).ne.opernext())then
         sp=sp+1
      elseif(scr(sp+1).eq.40)then
         a=scr(sp-1)
         b=scr(sp+2)
         if(scr(sp-2).eq.opernext().or.b.eq.opernext())goto 830
         if(a.lt.0.)then
            if(int(b).ne.b)goto 800
            scr(sp-1)=a**int(b)
         elseif(a.eq.0.)then
            if(b.eq.0.)goto 800
            scr(sp-1)=0.
         else
            scr(sp-1)=a**b
         endif
         call copy4(scr(sp+3),scr(sp),exprdim-sp-2)
         exprdim=exprdim-3
         last=last-3
      else
         sp=sp+2
      endif
      goto 250
 
c     Resolve *=41 and /=42, if any
  260 if(last.eq.first+3)goto 380
      sp=first+2
  270 if(sp.ge.last-1)goto 280
      if(scr(sp).ne.opernext())then
         sp=sp+1
      elseif(scr(sp+1).eq.41)then
c        Watch out for array multiplications
         if(scr(sp-2).ne.opernext())then
            if(scr(sp+2).ne.opernext())then
               scr(sp-1)=scr(sp-1)*scr(sp+2)
               call copy4(scr(sp+3),scr(sp),exprdim-sp-2)
            elseif(scr(sp+3).eq.80)then
c              a * {...}
               i=sp+4
  271          if(scr(i).ne.opernext())then
                  scr(i)=scr(sp-1)*scr(i)
                  i=i+1
                  goto 271
               else
                  if(scr(i+1).ne.81)call bug(ipt,jpt)
               endif
               call copy4(scr(sp+2),scr(sp-1),exprdim-sp-1)
               sp=i-1
            else
               goto 830
            endif
         elseif(scr(sp-1).eq.81)then
c           {...} * a
            if(scr(sp+2).ne.opernext())then
               i=sp-3
  272          if(scr(i-1).ne.opernext())then
                  scr(i)=scr(i)*scr(sp+2)
                  i=i-1
                  goto 272
               else
                  if(scr(i).ne.80)call bug(ipt,jpt)
               endif
               call copy4(scr(sp+3),scr(sp),exprdim-sp-2)
            else
               goto 830
            endif
         else
            goto 830
         endif
         exprdim=exprdim-3
         last=last-3
      elseif(scr(sp+1).eq.42)then
         if(scr(sp+2).eq.0.)goto 800
c        Watch out for array divisions
         if(scr(sp-2).ne.opernext())then
            if(scr(sp+2).ne.opernext())then
               scr(sp-1)=scr(sp-1)/scr(sp+2)
               call copy4(scr(sp+3),scr(sp),exprdim-sp-2)
            else
               goto 830
            endif
         elseif(scr(sp-1).eq.81)then
c           {...} / a
            if(scr(sp+2).ne.opernext())then
               i=sp-3
  273          if(scr(i-1).ne.opernext())then
                  scr(i)=scr(i)/scr(sp+2)
                  i=i-1
                  goto 273
               else
                  if(scr(i).ne.80)call bug(ipt,jpt)
               endif
               call copy4(scr(sp+3),scr(sp),exprdim-sp-2)
            else
               goto 830
            endif
         else
            goto 830
         endif
         exprdim=exprdim-3
         last=last-3
      else
         sp=sp+2
      endif
      goto 270
 
c     Resolve +=43 and -=44, if any
  280 if(last.eq.first+3)goto 380
      sp=first+2
  290 if(sp.ge.last-1)goto 300
      if(scr(sp).ne.opernext())then
         sp=sp+1
      elseif(scr(sp+1).eq.43)then
c        Sum of array and scalar not allowed.
c        Sum of two arrays with equal number of elements is OK.
         if(scr(sp-2).ne.opernext().and.scr(sp+2).ne.opernext())then
            scr(sp-1)=scr(sp-1)+scr(sp+2)
         elseif(scr(sp-1).ne.81.or.scr(sp+3).ne.80)then
c           Sum of array and scalar not allowed.
            goto 830
         else
c           Sum of two arrays only allowed if equal number of elements
            call realmatch(opernext(),scr(sp+4),last-sp-5,match)
            if(match.eq.0.or.scr(sp+4+match).ne.81)call bug(ipt,jpt)
            if(scr(sp-match-3).ne.opernext().or.scr(sp-match-2).ne.80)
     *         goto 830
            call vadd4(scr(sp-match-1),scr(sp+4),scr(sp-match-1),
     *         match-1)
         endif
         call copy4(scr(sp+3),scr(sp),exprdim-sp-2)
         exprdim=exprdim-3
         last=last-3
      elseif(scr(sp+1).eq.44)then
c        Difference of array and scalar not allowed.
c        Difference of two arrays with equal number of elements is OK.
         if(scr(sp-2).ne.opernext().and.scr(sp+2).ne.opernext())then
            scr(sp-1)=scr(sp-1)-scr(sp+2)
         elseif(scr(sp-1).ne.81.or.scr(sp+3).ne.80)then
c           Difference of array and scalar not allowed.
            goto 830
         else
c           Difference of two arrays only allowed if equal number of elements
            call realmatch(opernext(),scr(sp+4),last-sp-5,match)
            if(match.eq.0.or.scr(sp+4+match).ne.81)call bug(ipt,jpt)
            if(scr(sp-match-3).ne.opernext().or.scr(sp-match-2).ne.80)
     *         goto 830
            call vsubtract4(scr(sp-match-1),scr(sp+4),scr(sp-match-1),
     *         match-1)
         endif
         call copy4(scr(sp+3),scr(sp),exprdim-sp-2)
         exprdim=exprdim-3
         last=last-3
      else
         sp=sp+2
      endif
      goto 290
 
c     Resolve <~50, <=~51, >~52, >=~53, ==~54, !=~55, if any.
c     Note: account for constructions like (a<b<=c) and (a<=b<c).
c     Do this in two stages, replacing a<b by b or something larger
c     than c, so that the second part is always checked and is
c     possibly forced to fail.
  300 if(last.eq.first+3)goto 380
      sp=first+2
  310 if(sp.ge.last-1)goto 320
      if(scr(sp).ne.opernext())then
         sp=sp+1
         goto 310
      elseif(scr(sp+1).lt.50.or.scr(sp+1).gt.55)then
         sp=sp+2
         goto 310
      endif
c     Conditions involving arrays or operands are not allowed.
      if(scr(sp-2).eq.opernext().or.scr(sp+2).eq.opernext())goto 830
      a=scr(sp-1)
      b=scr(sp+2)
      if(scr(sp+1).eq.51)then
         if(sp+4.lt.last.and.scr(sp+3).eq.opernext().and.
     *      (scr(sp+4).eq.51.or.scr(sp+4).eq.52))then
            c=scr(sp+5)
            if(a.lt.b)then
               scr(sp-1)=b
            else
               scr(sp-1)=c+1.
            endif
         elseif(a.lt.b)then
            scr(sp-1)=1.
         else
            scr(sp-1)=0.
         endif
      elseif(scr(sp+1).eq.52)then
         if(sp+4.lt.last.and.scr(sp+3).eq.opernext().and.
     *      (scr(sp+4).eq.51.or.scr(sp+4).eq.52))then
            c=scr(sp+5)
            if(a.le.b)then
               scr(sp-1)=b
            else
               scr(sp-1)=c+1.
            endif
         elseif(a.le.b)then
            scr(sp-1)=1.
         else
            scr(sp-1)=0.
         endif
      elseif(scr(sp+1).eq.53)then
         if(sp+4.lt.last.and.scr(sp+3).eq.opernext().and.
     *      (scr(sp+4).eq.53.or.scr(sp+4).eq.54))then
            c=scr(sp+5)
            if(a.gt.b)then
               scr(sp-1)=b
            else
               scr(sp-1)=c-1.
            endif
         elseif(a.gt.b)then
            scr(sp-1)=1.
         else
            scr(sp-1)=0.
         endif
      elseif(scr(sp+1).eq.54)then
         if(sp+4.lt.last.and.scr(sp+3).eq.opernext().and.
     *      (scr(sp+4).eq.53.or.scr(sp+4).eq.54))then
            c=scr(sp+5)
            if(a.ge.b)then
               scr(sp-1)=b
            else
               scr(sp-1)=c-1.
            endif
         elseif(a.ge.b)then
            scr(sp-1)=1.
         else
            scr(sp-1)=0.
         endif
      elseif(scr(sp+1).eq.50)then
         if(a.eq.b)then
            scr(sp-1)=1.
         else
            scr(sp-1)=0.
         endif
      elseif(scr(sp+1).eq.55)then
         if(a.ne.b)then
            scr(sp-1)=1.
         else
            scr(sp-1)=0.
         endif
      endif
      call copy4(scr(sp+3),scr(sp),exprdim-sp-2)
      exprdim=exprdim-3
      last=last-3
      goto 310
 
c     Resolve NOT=62, if any
  320 if(last.eq.first+3)goto 380
      sp=first+1
  330 if(sp.ge.last-1)goto 340
      if(scr(sp).ne.opernext())then
         sp=sp+1
      elseif(scr(sp+1).eq.62)then
         a=scr(sp+2)
c        NOT followed by non-dummy, array or operand not allowed.
         if(a.ne.0..and.a.ne.1.)goto 800
         scr(sp) = 1.-a
         call copy4(scr(sp+3),scr(sp+1),exprdim-sp-1)
         exprdim=exprdim-2
         last=last-2
      else
         sp=sp+2
      endif
      goto 330
 
c     Resolve AND=60, if any
  340 if(last.eq.first+3)goto 380
      sp=first+2
  350 if(sp.ge.last-1)goto 360
      if(scr(sp).ne.opernext())then
         sp=sp+1
      elseif(scr(sp+1).eq.60)then
         a=scr(sp-1)
         b=scr(sp+2)
         if((a.ne.0..and.a.ne.1.).or.(b.ne.0..and.b.ne.1.))goto 800
         if(a.eq.1..and.b.eq.1.)then
            scr(sp-1)=1.
         else
            scr(sp-1)=0.
         endif
         call copy4(scr(sp+3),scr(sp),exprdim-sp-2)
         exprdim=exprdim-3
         last=last-3
      else
         sp=sp+2
      endif
      goto 350
 
c     Resolve OR=61, if any
  360 if(last.eq.first+3)goto 380
      sp=first+2
  370 if(sp.ge.last-1)goto 380
      if(scr(sp).ne.opernext())then
         sp=sp+1
      elseif(scr(sp+1).eq.61)then
         a=scr(sp-1)
         b=scr(sp+2)
         if((a.ne.0..and.a.ne.1.).or.(b.ne.0..and.b.ne.1.))goto 800
         if(a.eq.1..or.b.eq.1.)then
            scr(sp-1)=1.
         else
            scr(sp-1)=0.
         endif
         call copy4(scr(sp+3),scr(sp),exprdim-sp-2)
         exprdim=exprdim-3
         last=last-3
      else
         sp=sp+2
      endif
      goto 370
 
 
c     All operands have been resolved; resolve parentheses themselves
  380 a=scr(first+2)
      if(scr(first+1).eq.26)then
c        Spline; next elements must be all be reals, separated by
c        enditem.  The first is the variable to be transformed; the
c        next n are the nodes.  Convert variable and nodes into an
c        array, {...}.
         call realmatch(opernext(),scr(first+2),last-first-2,match)
         if(match.ne.0)then
c           Nested splines not allowed:
            if(scr(first+2+match).eq.80)goto 830
            call bug(ipt,jpt)
         endif
         numnodes=(last-first-3)/2
         if(numnodes.eq.0)then
c           Special case: spline without nodes
            scr(first)=a
         elseif(scr(first+4).lt.0.)then
c           First node negative:
c              beta'T(x) = beta'T*(x-node1) - beta'T*(0-node1)
c           where T*(x) is the spline transformation of x with
c           nodes relative to node1.
c           Mark the beginning of an array:
            scr(first+1)=80
            node1=scr(first+4)
            nodehigh=0.
            temp1=min(a-node1,nodehigh)
            temp2=min(-node1,nodehigh)
            scr(first+2)=temp1-temp2
            nodelow=nodehigh
            do 400 i=2,numnodes
               if(scr(first+2*i+1).ne.enditem())call bug(ipt,jpt)
               nodehigh=scr(first+2*i+2)-node1
               if(nodehigh.lt.nodelow)goto 820
               temp1=max(0.,min(a-node1-nodelow,nodehigh-nodelow))
               temp2=max(0.,min(-node1-nodelow,nodehigh-nodelow))
               scr(first+i+1)=temp1-temp2
               nodelow=nodehigh
  400       continue
            temp1=max(0.,a-node1-nodelow)
            temp2=max(0.,-node1-nodelow)
            scr(first+numnodes+2)=temp1-temp2
c           Delimit end of array
            scr(first+numnodes+3)=opernext()
            scr(first+numnodes+4)=81
c           Compression removes the enditems
            call copy4(scr(last+2),scr(first+numnodes+5),exprdim-last-1)
            exprdim=exprdim-numnodes
            goto 200
         else
c           Mark the beginning of an array:
            scr(first+1)=80
            nodehigh=scr(first+4)
            scr(first+2)=min(a,nodehigh)
            nodelow=nodehigh
            do 410 i=2,numnodes
               if(scr(first+2*i+1).ne.enditem())call bug(ipt,jpt)
               nodehigh=scr(first+2*i+2)
               if(nodehigh.lt.nodelow)goto 820
               scr(first+i+1)=max(0.,min(a-nodelow,nodehigh-nodelow))
               nodelow=nodehigh
  410       continue
            scr(first+numnodes+2)=max(0.,a-nodelow)
c           Delimit end of array
            scr(first+numnodes+3)=opernext()
            scr(first+numnodes+4)=81
c           Compression removes the enditems
            call copy4(scr(last+2),scr(first+numnodes+5),exprdim-last-1)
            exprdim=exprdim-numnodes
            goto 200
         endif
      elseif(scr(first+1).eq.28)then
c        MIN(...)
         if(last.eq.first+2)goto 830
         call realmatch(opernext(),scr(first+2),last-first-2,match)
         if(match.ne.0)then
c           arrays not allowed as arguments
            if(scr(first+2+match).eq.80)goto 830
            call bug(ipt,jpt)
         endif
         scr(first)=scr(first+2)
         do 420 i=first+4,last-1,2
            if(scr(i-1).ne.enditem())call bug(ipt,jpt)
            scr(first)=min(scr(first),scr(i))
  420    continue
      elseif(scr(first+1).eq.29)then
c        MAX(...)
         if(last.eq.first+2)goto 830
         call realmatch(opernext(),scr(first+2),last-first-2,match)
         if(match.ne.0)then
c           arrays not allowed as arguments
            if(scr(first+2+match).eq.80)goto 830
            call bug(ipt,jpt)
         endif
         scr(first)=scr(first+2)
         do 430 i=first+4,last-1,2
            if(scr(i-1).ne.enditem())call bug(ipt,jpt)
            scr(first)=max(scr(first),scr(i))
  430    continue
      elseif(last.ne.first+3)then
c        Array (or invalid expression or bug)
         if(last.lt.first+3)call bug(ipt,jpt)
         if(scr(first+2).eq.opernext().and.scr(first+3).eq.80.and.
     *      scr(last-2).eq.opernext().and.scr(last-1).eq.81)then
c           Array; copy { and all numnodes+1 elements and } to beginning
            if(scr(first+1).ne.20)goto 830
            call copy4(scr(first+2),scr(first),last-first-2)
            call copy4(scr(last+2),scr(last-2),exprdim-last-1)
            if(first.eq.1.and.last.eq.exprdim-1)goto 500
            exprdim=exprdim-4
            goto 200
         else
            call realmatch(opernext(),scr(first+2),last-first-2,match)
            if(match.ne.0)call bug(ipt,jpt)
            goto 830
         endif
      elseif(scr(first+1).eq.20)then
         scr(first)=a
      elseif(scr(first+1).eq.21)then
         scr(first)=exp(a)
      elseif(scr(first+1).eq.22)then
         if(a.le.0.)goto 800
         scr(first)=log(a)
      elseif(scr(first+1).eq.23)then
         scr(first)=abs(a)
      elseif(scr(first+1).eq.24)then
         if(a.lt.0.)goto 800
         scr(first)=sqrt(a)
      elseif(scr(first+1).eq.25)then
         if(a.ne.0..and.a.ne.1.)goto 800
         scr(first)=a
      elseif(scr(first+1).eq.30)then
         scr(first)=int(a)
      else
         call bug(ipt,jpt)
      endif
      if(first.eq.1.and.last.eq.exprdim-1)goto 500
      call copy4(scr(last+2),scr(first+1),exprdim-last-1)
      exprdim=exprdim-(last-first+1)
      goto 200
 
c     Expression evaluated:
  500 iexpr=iexpr+1
      if(scr(1).ne.opernext())then
         outcome(iexpr)=scr(1)
      else
         call copy4(scr(3),outcome(iexpr),numnodes+1)
         if(scr(2).ne.80.or.scr(numnodes+4).ne.opernext().or.
     *      scr(numnodes+5).ne.81)call bug(ipt,jpt)
         iexpr=iexpr+numnodes
      endif
      if(xnode(xp).eq.endexprs())then
         if(iexpr.ne.nexpr)call bug(ipt,jpt)
         goto 999
      else
c        Go to the next expression
         xp=xp+1
         goto 100
      endif
 
  800 string=idstring(struc,id,iobs,maxlevel,branch)
      if(ipt.gt.0)then
         call wrapstring(8,0,72,0,1,'ERROR:  illegal argument in '//
     *      'expression ('//string(1:trm(string))//'):')
         call writeexpr(110,8,maxxnode,xdim,levelmax,nbdim,
     *      -xpbegin,nb,branch,xpntr,1,nold,nxold,namepntr,-1,
     *      outcome,xnode,x,oldname,struc)
      endif
      if(jpt.gt.0)then
         call wrapstring(6,0,72,0,1,'ERROR:  illegal argument in '//
     *      'expression ('//string(1:trm(string))//'):')
         call writeexpr(110,6,maxxnode,xdim,levelmax,nbdim,
     *      -xpbegin,nb,branch,xpntr,1,nold,nxold,namepntr,-1,
     *      outcome,xnode,x,oldname,struc)
      endif
      stop
  810 if(ipt.gt.0)then
         write(8,811)
         call writeexpr(100,8,maxxnode,xdim,levelmax,nbdim,
     *      -xpbegin,nb,branch,xpntr,1,nold,nxold,namepntr,-1,
     *      outcome,xnode,x,oldname,struc)
      endif
      if(jpt.gt.0)then
         write(6,811)
         call writeexpr(100,6,maxxnode,xdim,levelmax,nbdim,
     *      -xpbegin,nb,branch,xpntr,1,nold,nxold,namepntr,-1,
     *      outcome,xnode,x,oldname,struc)
      endif
  811 format(/,'ERROR:  expression too long.')
      stop
  820 string=idstring(struc,id,iobs,maxlevel,branch)
      if(ipt.gt.0)then
         call wrapstring(8,0,72,0,1,'ERROR: spline nodes are '//
     *      'unordered ('//string(1:trm(string))//'):')
         call writeexpr(110,8,maxxnode,xdim,levelmax,nbdim,
     *      -xpbegin,nb,branch,xpntr,1,nold,nxold,namepntr,-1,
     *      outcome,xnode,x,oldname,struc)
      endif
      if(jpt.gt.0)then
         call wrapstring(6,0,72,0,1,'ERROR: spline nodes are '//
     *      'unordered ('//string(1:trm(string))//'):')
         call writeexpr(110,6,maxxnode,xdim,levelmax,nbdim,
     *      -xpbegin,nb,branch,xpntr,1,nold,nxold,namepntr,-1,
     *      outcome,xnode,x,oldname,struc)
      endif
      stop
  830 if(ipt.gt.0)then
         write(8,831)
         call writeexpr(100,8,maxxnode,xdim,levelmax,nbdim,
     *      -xpbegin,nb,branch,xpntr,1,nold,nxold,namepntr,-1,
     *      outcome,xnode,x,oldname,struc)
      endif
      if(jpt.gt.0)then
         write(6,831)
         call writeexpr(100,6,maxxnode,xdim,levelmax,nbdim,
     *      -xpbegin,nb,branch,xpntr,1,nold,nxold,namepntr,-1,
     *      outcome,xnode,x,oldname,struc)
      endif
  831 format(/,'ERROR:  invalid expression:')
      stop
  840 string=idstring(struc,id,iobs,maxlevel,branch)
      name=oldname(namepntr(level)+varpos-1)
      if(ipt.gt.0)then
         call writea2ia(8,72,1,'ERROR:  level ',level,' variable '//
     *      name(1:trm(name))//' enters a model ('//
     *      string(1:trm(string))//').  Given the level of the '//
     *      'outcome of this model, the lowest level variables '//
     *      'that may enter are at level ',maxlevel,':')
         call writeexpr(100,8,maxxnode,xdim,levelmax,nbdim,-xpbegin,nb,
     *      branch,xpntr,1,nold,nxold,namepntr,-1,outcome,xnode,
     *      x,oldname,struc)
      endif
      if(jpt.gt.0)then
         call writea2ia(6,72,1,'ERROR:  level ',level,' variable '//
     *      name(1:trm(name))//' enters a model ('//
     *      string(1:trm(string))//').  Given the level of the '//
     *      'outcome of this model, the lowest level variables '//
     *      'that may enter are at level ',maxlevel,':')
         call writeexpr(100,6,maxxnode,xdim,levelmax,nbdim,-xpbegin,nb,
     *      branch,xpntr,1,nold,nxold,namepntr,-1,outcome,xnode,
     *      x,oldname,struc)
      endif
      stop
  999 return
      end
 
