      subroutine figure_dspl(levelmax,mdllvl,nbdim,nb,branch,numspl,
     *   maxref,nds,maxmspl,splref,nxold,namepntr,maxxnode,nold,vdim,
     *   oldvdim,numnodes,spltype,nngam,meno,maxmeno,moel,
     *   nmapmoel,mapmoel,originstats,xnode,splineorigin,xpntr,xdim,x,
     *   endmark,detail,check99999,splinedone,splname,oldname,struc,
     *   iobs,id,ipt,jpt)
 
c     Subroutine to figure out which duration spline enters into a hazard
c     model and where the gammas and nodes are located.
c
c     Model input information:
c     moel(1) = 5  this is a duration spline
c     moel(2) = xnode position of expression
c     moel(3) = 1  spline definition directly specified
c               2  spline definition indirectly referenced
c     moel(4) = spline definition number if moel(3)=1
c               expression position in xnode if moel(3)=2
c
c     Model output information:
c        mapmoel(1) = 5  this is a duration spline
c        mapmoel(2) =    spline definition number
c
c     Constantijn Panis,  2 February 2000
c     Last change: zero referenced interaction fixed (aML 2.54)
 
      implicit none
      integer levelmax,mdllvl,nbdim,nb(nbdim,2),branch(levelmax),
     *   numspl,maxref,nds,maxmspl,splref(maxref),
     *   namepntr(levelmax),nxold(levelmax),maxxnode,nold,vdim,oldvdim,
     *   numnodes(numspl),spltype(numspl),nngam,meno,
     *   maxmeno,moel(maxmeno),nmapmoel,mapmoel(*),xpntr(2:levelmax),
     *   xdim,ref,splinedone,struc,iobs,id,status,ipt,jpt,trm
      double precision originstats(5,numspl)
      real xnode(maxxnode),splineorigin(maxmspl),x(xdim),refvar,
     *   origin,endmark
      logical detail,check99999
      character splname(*)*12,oldname(nold)*8,string*60,idstring*60
 
c     First determine origin, which is an expression:
      call evalexpr(1,maxxnode,xdim,moel(meno+1),nold,levelmax,
     *   mdllvl,nbdim,nb,branch,xpntr,nxold,namepntr,xnode,x,origin,
     *   oldname,struc,iobs,id,ipt,jpt,status)
      if(status.gt.0)stop
 
c     Which spline enters, if any?  Save meno for various error messages.
      if(moel(meno+2).eq.1)then
c        Directly referenced:
         ref=moel(meno+3)
      elseif(moel(meno+2).eq.2)then
c        Indirect reference.
         call evalexpr(1,maxxnode,xdim,moel(meno+3),nold,levelmax,
     *      mdllvl,nbdim,nb,branch,xpntr,nxold,namepntr,xnode,x,
     *      refvar,oldname,struc,iobs,id,ipt,jpt,status)
         if(status.gt.0)stop
         if(refvar.eq.0.)then
            if(check99999.and.abs(origin).ne.99999.)goto 810
c           This term and everything interacted with it drops out.
            call dropblock(vdim,oldvdim,nmapmoel,mapmoel,
     *         meno,maxmeno,moel,ipt,jpt)
            goto 999
         elseif(int(refvar).ne.refvar)then
            goto 830
         endif
         call ref2def(int(refvar),maxref,splref,ref,status)
         if(status.gt.0)goto 800
      else
         call bug(ipt,jpt)
      endif
      meno=meno+4
 
c     Check that the origin is not 99999.
      if(check99999.and.abs(origin).eq.99999.)goto 810
 
      if(spltype(ref).ge.3.and.-origin.ge.endmark)then
c        This spline effectively does not enter (and everything interacted
c        with it neither).
         call dropblock(vdim,oldvdim,nmapmoel,mapmoel,
     *      meno,maxmeno,moel,ipt,jpt)
         goto 999
      endif
 
c     Does this spline apply to the beginning of the episode?
      if(origin.ge.0..or.spltype(ref).le.2)then
         if(splinedone.eq.-1)call bug(ipt,jpt)
         splinedone=1
      endif
 
c     Record which spline enters:
      mapmoel(nmapmoel+1)=5
      mapmoel(nmapmoel+2)=ref
      nmapmoel=nmapmoel+2
 
c     Record the origin:
      nds=nds+1
      if(nds.gt.maxmspl)call maxexceed('maxmspl',nds,maxmspl,ipt,jpt)
      splineorigin(nds)=origin
c     Keep track of summary statistics for origin:
      if(detail)call figstats(originstats(1,ref),dble(origin))
c     Keep track of total number of intercepts and slopes in this episode:
      nngam=nngam+numnodes(ref)+1
      if(spltype(ref).eq.2.or.spltype(ref).eq.4)nngam=nngam+1
 
      goto 999
 
  800 call norefmatch(1,struc,id,iobs,mdllvl,branch,ipt,jpt,maxxnode,
     *   xdim,levelmax,nbdim,nb,moel(meno+3),xpntr,nold,nxold,
     *   namepntr,refvar,xnode,x,oldname,'duration spline','spline')
  810 string=idstring(struc,id,iobs,mdllvl,branch)
      if(moel(meno+2).eq.1)then
c        Directly referenced spline:
         if(ipt.gt.0)then
            call writeaia(8,72,1,'ERROR: spline '''//
     *         splname(ref)(1:trm(splname(ref)))//''' enters a '//
     *         'hazard model.  The variable to be transformed is ',
     *         int(origin),' ('//string(1:trm(string))//
     *         '), which is most likely an error.')
            write(8,811)
         endif
         if(jpt.gt.0)then
            call writeaia(6,72,1,'ERROR: spline '''//
     *         splname(ref)(1:trm(splname(ref)))//''' enters a '//
     *         'hazard model.  The variable to be transformed is ',
     *         int(origin),' ('//string(1:trm(string))//
     *         '), which is most likely an error.')
            write(6,811)
         endif
  811    format('Note:  specify "option check99999=no;" in the ',
     *      'control file to',/,'switch off this data integrity check.')
      else
c        Indirectly referenced spline:
         if(ipt.gt.0)then
            call wrapstring(8,0,72,0,1,'ERROR in a duration spline '//
     *         'in a hazard model specification ('//
     *         string(1:trm(string))//').  Its indirect reference is ')
            call writeexpr(111,8,maxxnode,xdim,levelmax,nbdim,
     *         -moel(meno+3),nb,branch,xpntr,1,nold,nxold,namepntr,
     *         -1,refvar,xnode,x,oldname,struc)
            if(abs(origin).ne.99999.)then
               call writeada(8,72,1,'The corresponding origin must '//
     *            'then be 99999, but instead it is ',-8,dble(origin),
     *            '.')
            else
               call writeaia(8,72,1,'The corresponding origin is ',
     *            int(origin),', which should only occur if the '//
     *            'spline does not enter the equation (i.e., in '//
     *            'combination with a zero-valued indirect reference).')
            endif
            write(8,811)
         endif
         if(jpt.gt.0)then
            call wrapstring(6,0,72,0,1,'ERROR in a duration spline '//
     *         'in a hazard model specification ('//
     *         string(1:trm(string))//').  Its indirect reference is ')
            call writeexpr(111,6,maxxnode,xdim,levelmax,nbdim,
     *         -moel(meno+3),nb,branch,xpntr,1,nold,nxold,namepntr,
     *         -1,refvar,xnode,x,oldname,struc)
            if(abs(origin).ne.99999.)then
               call writeada(6,72,1,'The corresponding origin must '//
     *            'then be 99999, but instead it is ',-8,dble(origin),
     *            '.')
            else
               call writeaia(6,72,1,'The corresponding origin is ',
     *            int(origin),', which should only occur if the '//
     *            'spline does not enter the equation (i.e., in '//
     *            'combination with a zero-valued indirect reference).')
            endif
            write(6,811)
         endif
      endif
      stop
  830 call norefmatch(2,struc,id,iobs,mdllvl,branch,ipt,jpt,maxxnode,
     *   xdim,levelmax,nbdim,nb,moel(meno+3),xpntr,nold,nxold,
     *   namepntr,refvar,xnode,x,oldname,'duration spline','spline')
  999 return
      end
 
