      subroutine figure_rset(levelmax,mdllvl,outlevel,nfull,nold,nsb,
     *   nbdim,nb,meno,maxmeno,maxxnode,maxref,vdimmax,numrset,moel,
     *   rsetbegin,rsetnum,rsetref,namepntr,nxold,vdim,oldvdim,xpntr,
     *   nb2,fullxpntr,branch,nmapmoel,mapmoel,rsetstats,xdim,x,v,
     *   xnode,oldname,detail,iobs,id,struc,modeltype,ipt,jpt)
 
c     Subroutine to place regressor set variables into a scratch array.
c     There are two fundamentally different scenarios. The first is when
c     outlevel=mdllvl. This is relatively straightforward:  every
c     variable in the input data set takes up one position in the output
c     (scratch) data vector. The second is when outlevel=mdllvl+1. This
c     implies that variables at mdllvl or higher must be replicated to
c     mimic the output level. This occurs only in hazard models with
c     time-varying RHS variables (not interacted with baseline survivor
c     function or integrated residuals).
c
c     Upon input, regressor set reference information is stored in moel
c     (see read_rset):
c     moel(1)  = 3  This is a regressor set
c     moel(2)  = 1 reference is direct
c                2 reference is indirect
c     moel(3)  = definition rank number if moel(2)=1
c                expression position in xnode if moel(2)=2
c
c     Upon output, the following is stored:
c     mapmoel(1) = 3  This is a regressor set
c     mapmoel(2) = position in fullbeta where regressor parameters start
c     mapmoel(3) = number of regressors
c
c     Constantijn Panis,  2 February 2000
c     Last change: zero referenced interaction fixed (aML 2.54)
 
      implicit none
      integer levelmax,mdllvl,outlevel,nfull,nold,nbdim,nb(nbdim,2),
     *   branch(levelmax),branchnum,meno,maxxnode,maxref,vdimmax,
     *   numrset,
     *   maxmeno,moel(maxmeno),rsetbegin(numrset),rsetnum(numrset),
     *   rsetref(maxref),namepntr(levelmax),nxold(levelmax),nsb,vdim,
     *   oldvdim,xpntr(2:levelmax),nb2,fullxpntr(2:levelmax,nb2),xdim,
     *   nmapmoel,mapmoel(*),ref,first,xnodepntr,status,i,iobs,struc,
     *   id,modeltype,ipt,jpt,ib2
      double precision rsetstats(5,nfull)
      real x(xdim),v(vdimmax),xnode(maxxnode),refvar
      character oldname(nold)*8
      logical detail
 
      if(moel(meno+1).eq.1)then
c        Direct reference
         ref=moel(meno+2)
      elseif(moel(meno+1).eq.2)then
c        Indirect reference
         call evalexpr(1,maxxnode,xdim,moel(meno+2),nold,levelmax,
     *      mdllvl,nbdim,nb,branch,xpntr,nxold,namepntr,xnode,x,refvar,
     *      oldname,struc,iobs,id,ipt,jpt,status)
         if(status.gt.0)stop
         if(refvar.eq.0.)then
c           This term and everything interacted with it drops out.
            call dropblock(vdim,oldvdim,nmapmoel,mapmoel,
     *         meno,maxmeno,moel,ipt,jpt)
            goto 999
         elseif(int(refvar).ne.refvar)then
            goto 810
         endif
         call ref2def(int(refvar),maxref,rsetref,ref,status)
         if(status.gt.0)goto 800
      else
         call bug(ipt,jpt)
      endif
      meno=meno+3
c     The ref-th set is used.
 
      xnodepntr=mod(rsetbegin(ref),maxxnode+1)
      first=rsetbegin(ref)/(maxxnode+1)-1
 
      if(outlevel.eq.mdllvl)then
         if(vdim+rsetnum(ref).gt.vdimmax)call
     *      maxexceed('vdimmax',vdim+rsetnum(ref),vdimmax,ipt,jpt)
         call evalexpr(rsetnum(ref),maxxnode,xdim,xnodepntr,nold,
     *      levelmax,outlevel,nbdim,nb,branch,xpntr,nxold,namepntr,
     *      xnode,x,v(vdim+1),oldname,struc,iobs,id,ipt,jpt,status)
         if(status.gt.0)goto 820
         if(detail)then
            do 100 i=1,rsetnum(ref)
               call figstats(rsetstats(1,first+i),dble(v(vdim+i)))
  100       continue
         endif
         vdim=vdim+rsetnum(ref)
      elseif(outlevel.eq.mdllvl+1)then
c        For hazard models only; need to loop over subbranches.  If
c        these subbranches are at level 2, xpntr varies.  In such cases
c        use fullxpntr, which is xpntr for all levels 2.
         if(vdim+rsetnum(ref)*nsb.gt.vdimmax)call
     *      maxexceed('vdimmax',vdim+rsetnum(ref)*nsb,vdimmax,ipt,jpt)
         ib2=branch(2)
c        Warning: this routine will place all time-varying variables
c        in V, even if the upper event took place before the highest
c        timemark.  This can be slightly wasteful.  Do not use all
c        nsb branches in mapmodel, but only the first nu.
         branchnum=0
  200    call nextbranch(mdllvl,mdllvl+1,branch,nbdim,nb,branchnum)
         if(branchnum.eq.0)goto 300
         if(mdllvl.eq.1)ib2=branch(2)
         call evalexpr(rsetnum(ref),maxxnode,xdim,xnodepntr,nold,
     *      levelmax,outlevel,nbdim,nb,branch,fullxpntr(2,ib2),
     *      nxold,namepntr,xnode,x,v(vdim+1),oldname,struc,
     *      iobs,id,ipt,jpt,status)
         if(status.gt.0)stop
         if(detail)then
            do 210 i=1,rsetnum(ref)
               call figstats(rsetstats(1,first+i),dble(v(vdim+i)))
  210       continue
         endif
         vdim=vdim+rsetnum(ref)
         goto 200
      else
         call bug(ipt,jpt)
      endif
 
c     Register information about this regressor set: its beginning
c     position in the parameter vector and the number of variables:
  300 mapmoel(nmapmoel+1) = 3
      mapmoel(nmapmoel+2) = first+1
      mapmoel(nmapmoel+3) = rsetnum(ref)
      nmapmoel=nmapmoel+3
      goto 999
 
  800 call norefmatch(1,struc,id,iobs,mdllvl,branch,ipt,jpt,maxxnode,
     *   xdim,levelmax,nbdim,nb,moel(meno+2),xpntr,nold,nxold,
     *   namepntr,refvar,xnode,x,oldname,'regressor set',
     *   'regressor set')
  810 call norefmatch(2,struc,id,iobs,mdllvl,branch,ipt,jpt,maxxnode,
     *   xdim,levelmax,nbdim,nb,moel(meno+2),xpntr,nold,nxold,
     *   namepntr,refvar,xnode,x,oldname,'regressor set',
     *   'regressor set')
c     status   [out] 0 = no error
c                    1 = variable not found
c                    2 = variable found but not at the mandatory -maxlevel
c                    3 = variable found, at level=maxlevel+1
c                        (Note: useful for hazard model error diagnosis)
c                    4 = variable found, at level>maxlevel+1
  820 if(modeltype.ne.1.or.status.ne.3)stop
c     Attempt to use time-varying variables in a hazard model
      if(ipt.gt.0)call writeaia(8,72,1,'NOTE:  level ',outlevel+1,
     *   ' variables are allowed in hazard models but require that '//
     *   'you specify TIMEMARKS in the model statement to '//
     *   'delimit the intervals over which the variables are constant.')
      if(jpt.gt.0)call writeaia(6,72,1,'NOTE:  level ',outlevel+1,
     *   ' variables are allowed in hazard models but require that '//
     *   'you specify TIMEMARKS in the model statement to '//
     *   'delimit the intervals over which the variables are constant.')
      stop
  999 return
      end
