      subroutine figure_rspl(levelmax,mdllvl,outlevel,nsb,nbdim,nb,
     *   branch,meno,maxmeno,maxref,vdimmax,moel,splref,numspl,beginspl,
     *   nodepos,numnodes,spltype,namepntr,nxold,maxxnode,nold,
     *   modeltype,vdim,oldvdim,xpntr,nb2,fullxpntr,xdim,nmapmoel,
     *   mapmoel,originstats,x,v,xnode,detail,check99999,splname,
     *   oldname,iobs,id,struc,ipt,jpt)
 
c     Regressor splines are just like regressor sets, but stem from
c     a different model element definition. Place transformed spline
c     variables into a scratch array as if it were a regressor set.
c
c     Regressor spline reference information is stored in moel:
c     moel(1) = 8  this is a regressor spline
c     moel(2) =    position in xnode where origin expression starts
c     moel(3) = 1  spline definition directly specified
c               2  spline definition indirectly referenced
c     moel(4) = spline definition number if moel(4)=1
c               expression position in xnode if moel(4)=2
c
c     Output information on the regressor set (splines becomes sets):
c     mapmoel(1) = 3 this is a regressor set (!)
c     mapmoel(2) = begin position in fullpar of the betas
c     mapmoel(3) = number of betas involved
c
c     Note that the intercept of a spline also goes into the regressor
c     set.
c
c     Constantijn Panis,  2 February 2000
c     Last change: zero referenced interaction fixed (aML 2.54)
 
      implicit none
      integer levelmax,mdllvl,outlevel,nsb,nbdim,nb(nbdim,2),
     *   branch(levelmax),branchnum,meno,nold,maxxnode,maxref,
     *   vdimmax,maxmeno,
     *   moel(maxmeno),splref(maxref),numspl,beginspl(numspl),
     *   nodepos(numspl),numnodes(numspl),spltype(numspl),
     *   namepntr(levelmax),nxold(levelmax),modeltype,vdim,oldvdim,
     *   xpntr(2:levelmax),nb2,fullxpntr(2:levelmax,nb2),xdim,nmapmoel,
     *   mapmoel(*),outnx,vpos,ref,pos,status,
     *   nnssbb,ib2,oripos,refmeno,iobs,id,struc,ipt,jpt,trm
      double precision originstats(5,numspl)
      real x(xdim),v(vdimmax),xnode(maxxnode),origin,refvar
      logical detail,check99999,intercept,kick
      character splname(numspl)*12,oldname(nold)*8,string*60,
     *   idstring*60
c     Make sure xnode(0) will never be accessed
      data pos/1/
 
      oripos=moel(meno+1)
      refmeno=meno+2
c     Determine the (first) variable to be transformed.  (There may be
c     more if outlevel>mdllvl.)
      call evalexpr(1,maxxnode,xdim,oripos,nold,levelmax,outlevel,
     *   nbdim,nb,branch,xpntr,nxold,namepntr,xnode,x,origin,
     *   oldname,struc,iobs,id,ipt,jpt,status)
      if(status.gt.0)goto 820
 
c     Which spline enters?
      if(moel(meno+2).eq.1)then
c        Directly referenced:
         ref=moel(meno+3)
      elseif(moel(meno+2).eq.2)then
c        Indirect reference.
         call evalexpr(1,maxxnode,xdim,moel(meno+3),nold,levelmax,
     *      mdllvl,nbdim,nb,branch,xpntr,nxold,namepntr,xnode,x,refvar,
     *      oldname,struc,iobs,id,ipt,jpt,status)
         if(status.gt.0)stop
         if(refvar.eq.0.)then
c           This spline does not enter. Check that the (first) variable
c           to be transformed is +/-99999.
            if(check99999.and.abs(origin).ne.99999.)goto 810
c           This term and everything interacted with it drops out.
            call dropblock(vdim,oldvdim,nmapmoel,mapmoel,
     *         meno,maxmeno,moel,ipt,jpt)
            goto 999
         elseif(int(refvar).ne.refvar)then
            goto 830
         endif
         call ref2def(int(refvar),maxref,splref,ref,status)
         if(status.gt.0)goto 800
      else
         call bug(ipt,jpt)
      endif
      meno=meno+4
 
      intercept = (spltype(ref).eq.2.or.spltype(ref).eq.4)
      kick = (spltype(ref).ge.3)
 
c     Number of output variables:
      outnx=numnodes(ref)+1
      if(intercept)outnx=outnx+1
c     Number of subbranches (replications), if applicable:
      nnssbb=1
      if(outlevel.eq.mdllvl+1)nnssbb=nsb
c     Check data scratch space:
      if(vdim+outnx*nnssbb.gt.vdimmax)call
     *   maxexceed('vdimmax',vdim+outnx*nnssbb,vdimmax,ipt,jpt)
 
c     Upon output the regressor spline is just like a regressor set:
      mapmoel(nmapmoel+1) = 3
      mapmoel(nmapmoel+2) = beginspl(ref)
      mapmoel(nmapmoel+3) = outnx
      nmapmoel=nmapmoel+3
 
c     Position in xnode where nodes start:
      pos=nodepos(ref)
 
c     Place the spline transformation (incl intercept, if any) in
c     the scratch data:
      vpos=vdim+1
      if(outlevel.le.mdllvl.or.nnssbb.eq.1)then
c        One input value; possibly multiple output values:
         if(check99999.and.abs(origin).eq.99999.)goto 810
         if(kick.and.origin.lt.0.)then
c           This term and everything interacted with it drops out.
            call dropblock(vdim,oldvdim,nmapmoel,mapmoel,
     *         meno,maxmeno,moel,ipt,jpt)
            goto 999
         endif
         if(intercept)then
            v(vpos)=1.
            vpos=vpos+1
         endif
         if(detail)call figstats(originstats(1,ref),dble(origin))
         call spltx(origin,v(vpos),xnode(pos),numnodes(ref),0,ipt,jpt)
         if(nnssbb.gt.1)call putrealsrow(v(vdim+1),outnx,
     *      v(vdim+1+outnx),nnssbb-1,outnx)
      else
c        Multiple input and output values (outlevel=mdllvl+1);
c        only possible in hazard models where the origin is a time-
c        varying variable or expression.  Loop over subbranches.
c        Be careful if mdllvl=1, so that xpntr varies.  Also see
c        figure_rset.
         if(intercept)then
            call putrealrow(1.,v(vpos),nnssbb,outnx)
            vpos=vpos+1
         endif
         ib2=branch(2)
         branchnum=0
c        Warning: this routine will place all time-varying variables
c        in V, even if the upper event took place before the highest
c        timemark.  This can be slightly wasteful.
  200    call nextbranch(mdllvl,mdllvl+1,branch,nbdim,nb,branchnum)
         if(branchnum.eq.0)goto 300
c        Figure out the value of the transformand variable:
         if(mdllvl.eq.1)ib2=branch(2)
         call evalexpr(1,maxxnode,xdim,oripos,nold,levelmax,outlevel,
     *      nbdim,nb,branch,fullxpntr(2,ib2),nxold,namepntr,xnode,x,
     *      origin,oldname,struc,iobs,id,ipt,jpt,status)
         if(status.gt.0)stop
         if(check99999.and.abs(origin).eq.99999.)goto 810
         if(kick.and.origin.lt.0.)then
c           Eliminate spline for this interval:
            if(intercept)v(vpos-1)=0.
            call putreal(0.,v(vpos),numnodes(ref)+1)
         else
            if(detail)call figstats(originstats(1,ref),dble(origin))
            call spltx(origin,v(vpos),xnode(pos),numnodes(ref),0,
     *         ipt,jpt)
         endif
         vpos=vpos+outnx
         goto 200
      endif
  300 vdim=vdim+outnx*nnssbb
      goto 999
 
  800 call norefmatch(1,struc,id,iobs,mdllvl,branch,ipt,jpt,maxxnode,
     *   xdim,levelmax,nbdim,nb,moel(meno+3),xpntr,nold,nxold,
     *   namepntr,refvar,xnode,x,oldname,'regressor spline','spline')
c     Spline entered and var=99999 or spline does not enter and var!=99999
  810 string=idstring(struc,id,iobs,mdllvl,branch)
      if(moel(refmeno).eq.1)then
c        Directly referenced spline; var=99999
         if(ipt.gt.0)then
            call writeaia(8,72,1,'ERROR: spline '''//
     *         splname(ref)(1:trm(splname(ref)))//''' enters a '//
     *         'model as regressor spline. The variable to be '//
     *         'transformed is ',int(origin),', which is most '//
     *         'likely an error ('//string(1:trm(string))//').')
            write(8,811)
         endif
         if(jpt.gt.0)then
            call writeaia(6,72,1,'ERROR: spline '''//
     *         splname(ref)(1:trm(splname(ref)))//''' enters a '//
     *         'model as regressor spline. The variable to be '//
     *         'transformed is ',int(origin),', which is most '//
     *         'likely an error ('//string(1:trm(string))//').')
            write(6,811)
         endif
      else
c        Indirectly referenced spline:
         if(ipt.gt.0)then
            call wrapstring(8,0,72,0,0,'ERROR: a regressor spline '//
     *         'is indirectly referenced; its reference value is ('//
     *         string(1:trm(string))//'):  ')
            call writeexpr(111,8,maxxnode,xdim,levelmax,nbdim,
     *         -moel(refmeno+1),nb,branch,xpntr,1,nold,nxold,namepntr,
     *         -1,refvar,xnode,x,oldname,struc)
         endif
         if(jpt.gt.0)then
            call wrapstring(6,0,72,0,0,'ERROR: a regressor spline '//
     *         'is indirectly referenced; its reference value is ('//
     *         string(1:trm(string))//'):  ')
            call writeexpr(111,6,maxxnode,xdim,levelmax,nbdim,
     *         -moel(refmeno+1),nb,branch,xpntr,1,nold,nxold,namepntr,
     *         -1,refvar,xnode,x,oldname,struc)
         endif
         if(abs(origin).ne.99999.)then
            if(ipt.gt.0)then
            call wrapstring(8,0,72,0,0,'This implies that this '//
     *         'spline does not enter in this outcome.  The '//
     *         'corresponding variable to be transformed must then '//
     *         'be 99999, but instead it is:  ')
               call writeexpr(111,8,maxxnode,xdim,levelmax,nbdim,
     *            -oripos,nb,branch,xpntr,1,nold,nxold,namepntr,
     *            -1,origin,xnode,x,oldname,struc)
            endif
            if(jpt.gt.0)then
            call wrapstring(6,0,72,0,0,'This implies that this '//
     *         'spline does not enter in this outcome.  The '//
     *         'corresponding variable to be transformed must then '//
     *         'be 99999, but instead it is:  ')
               call writeexpr(111,6,maxxnode,xdim,levelmax,nbdim,
     *            -oripos,nb,branch,xpntr,1,nold,nxold,namepntr,
     *            -1,origin,xnode,x,oldname,struc)
            endif
            if(ipt.gt.0)write(8,811)
            if(jpt.gt.0)write(6,811)
         else
            if(ipt.gt.0)call writeaia(8,72,1,'The variable to be '//
     *         'transformed is ',int(origin),', which should only '//
     *         'occur if the spline does not enter the equation '//
     *         '(namely in combination with a reference variable '//
     *         'equal to 0).')
            if(jpt.gt.0)call writeaia(6,72,1,'The variable to be '//
     *         'transformed is ',int(origin),', which should only '//
     *         'occur if the spline does not enter the equation '//
     *         '(namely in combination with a reference variable '//
     *         'equal to 0).')
            if(ipt.gt.0)write(8,811)
            if(jpt.gt.0)write(6,811)
         endif
      endif
  811 format('Note:  specify "option check99999=no;" in the ',
     *   'control file to'/'by-pass this check.')
      stop
  820 if(modeltype.ne.1.or.status.ne.3)stop
      if(ipt.gt.0)call writea2ia(8,72,1,'ERROR: a hazard model '//
     *   'has outcomes at level ',mdllvl,
     *   '.  A level ',outlevel,' variable enters the equation '//
     *   'through a regressor spline, which is allowed, but only '//
     *   'if you specify TIMEMARKS in the hazard model '//
     *   'statement to delimit the intervals over which the '//
     *   'variable is constant.')
      if(ipt.gt.0)call writea2ia(6,72,1,'ERROR: a hazard model '//
     *   'has outcomes at level ',mdllvl,
     *   '.  A level ',outlevel,' variable enters the equation '//
     *   'through a regressor spline, which is allowed, but only '//
     *   'if you specify TIMEMARKS in the hazard model '//
     *   'statement to delimit the intervals over which the '//
     *   'variable is constant.')
      stop
  830 call norefmatch(2,struc,id,iobs,mdllvl,branch,ipt,jpt,maxxnode,
     *   xdim,levelmax,nbdim,nb,moel(meno+3),xpntr,nold,nxold,
     *   namepntr,refvar,xnode,x,oldname,'regressor spline','spline')
  999 return
      end
