      subroutine getdatainfo(maxcc,ccmax,rr,ii,cc,ipt,jpt,
     *   r2aversion,levelmax,datadim,datanobs,xdimmax,nbdimmax,nold,
     *   nbmax,nb2max,datastructure,datalevel,nxold,namepntr,
     *   oldname,line,dsn,datadate)
 
c     Subroutine to determine the data set name and to read in data
c     set information.  This routine is the first routine to read
c     the control file.
c
c     The maximum dimensions are:
c        levelmax = maximum number of levels in the data
c        datadim  = number of data structures in the data
c        datanobs = number of observations in the data
c        xdimmax  = maximum length of any data vector
c        nbdimmax = maximum dimension of nb
c        nold     = total number of input variables
c        nbmax    = maximum number of level 2 through level levelmax
c                   branches in any one observation
c
c     Constantijn Panis, 21 January 2006
c     Last change: remove license issues
 
      implicit none
      integer maxcc,ccmax
      real rr(maxcc/4)
      integer ii(maxcc/4)
      character cc(maxcc)*1
      integer ipt,jpt,il,r2aversion,levelmax,datadim,
     *   datanobs,xdimmax,nbdimmax,nold,status,i,i1,i2,trm
      integer nbmax,nb2max,datastructure,datalevel,nxold,namepntr,
     *   oldname
      character line*128,dsn*128,datadate*24,backslash*1
      logical reversed,triedfix,exist,smtxt
c     data backslash/'\'/
c     data backslash/'\\'/
      data backslash/'\\'/
      data reversed,triedfix/2*.false./
 
c     Go through the control file once just to determine the data set name.
      rewind 4
      read(4,'(a80)',end=810,err=810)line
      il=1
  100 if(.not.smtxt(4,line,il,'DSN ='))then
c        Read until semicolon is encountered and try again
         call gotochar(4,';',line,il,status)
         if(status.gt.0)goto 810
         goto 100
      endif
 
      call getstring(4,line,il,i1,i2)
      if(i2.eq.-1.or.i1.gt.i2)goto 800
      if(.not.smtxt(4,line,il,';'))goto 800
c     Try default extension .dat first:
  110 if(exist(line(i1:i2)//'.dat'))then
         dsn=line(i1:i2)//'.dat'
      elseif(exist(line(i1:i2)))then
         dsn=line(i1:i2)
      else
c        Try replacing / by \ and vice versa:
         do 120 i=i1,i2
            if(line(i:i).eq.'/')then
               line(i:i)=backslash
            elseif(line(i:i).eq.backslash)then
               line(i:i)='/'
            endif
  120    continue
         if(reversed)goto 820
         reversed=.true.
         goto 110
      endif
      rewind 4
 
c     Read information on the data set:
c     Position statement is extension in Fortran 77:
c     open(12,file=dsn,form='unformatted',position='append',err=850)
cS     open(12,file=dsn,form='unformatted',position='append',err=850)
c     open(12,file=dsn,form='unformatted',err=850)
c     call gotoend(12)
      open(12,file=dsn,form='unformatted',err=850)
      call gotoend(12)
 
      goto 210
  200 if(triedfix)goto 830
      call gotoend(12)
      triedfix=.true.
  210 continue
 
c     Position file at the beginning of system information:
      do 130 i=1,8
         backspace(12,err=830)
  130 continue
 
c     Read the data creation date, as in "Fri May 14 10:54:21 PDT 1993"
      read(12,end=200,err=200)datadate
c     Read raw2aml (data) version:
      read(12,end=830,err=830)r2aversion
      if(r2aversion.ge.1000)r2aversion=r2aversion/10
      if(r2aversion.lt.100)goto 840
c     Read (maximum) data dimensions:
      read(12,end=830,err=830)datadim,datanobs,nold,levelmax,
     *   xdimmax,nbdimmax
 
c     Allocate space for data arrays:
      datastructure=ccmax/4+2
      ccmax=4*(datastructure+datadim)
      nbmax=ccmax/4+2
      datalevel=nbmax+(levelmax-1)*(levelmax-1)
      nxold=datalevel+datadim
      namepntr=nxold+levelmax*datadim
c     The dimension of namepntr is levelmax*datadim
      ccmax=4*(namepntr+levelmax*datadim)
      oldname=ccmax+2
c     The dimension of oldname is 8*nold
      ccmax=oldname+8*nold
      if(ccmax.gt.maxcc)call needmemory(maxcc,ccmax,0,ipt,jpt)
 
      call getdatainfo_(levelmax,datadim,nold,ii(nbmax),nb2max,
     *   rr(datastructure),ii(datalevel),ii(nxold),ii(namepntr),
     *   cc(oldname),status)
      if(status.ne.0)goto 830
      rewind(12)
      goto 999
 
  800 if(ipt.gt.0)write(8,801)
      if(jpt.gt.0)write(6,801)
c 801 format(/,'ERROR in data set name specification.  The syntax is:',
c    *   /,6x,'DSN = ''\path\data.set.name'';')
c 801 format(/,'ERROR in data set name specification.  The syntax is:',
c    *   /,6x,'DSN = ''/path/data.set.name'';')
  801 format(/,'ERROR in data set name specification.  The syntax is:',
     *   /,6x,'DSN = ''/path/data.set.name'';')
      stop
  810 if(ipt.gt.0)write(8,811)
      if(jpt.gt.0)write(6,811)
c 811 format(/,'ERROR:  no data set specification found.  The syntax ',
c    *   'is:',/,6x,'DSN = ''\path\dsn'';')
c 811 format(/,'ERROR:  no data set specification found.  The syntax ',
c    *   'is:',/,6x,'DSN = ''/path/dsn'';')
  811 format(/,'ERROR:  no data set specification found.  The syntax ',
     *   'is:',/,6x,'DSN = ''/path/dsn'';')
      stop
  820 if(i2.gt.4.and.line(i2-3:i2).eq.'.dat')i2=i2-4
      if(ipt.gt.0)call not_found(8,line(i1:i2)//'.dat')
      if(jpt.gt.0)call not_found(6,line(i1:i2)//'.dat')
      stop
  830 if(ipt.gt.0)write(8,831)dsn(1:trm(dsn))
      if(jpt.gt.0)write(6,831)dsn(1:trm(dsn))
  831 format(/,'ERROR encountered in ''',a,'''.')
      stop
  840 if(ipt.gt.0)write(8,841)r2aversion*.01
      if(jpt.gt.0)write(6,841)r2aversion*.01
  841 format(/,'ERROR:  the data were created with raw2aml version',
     *   f5.2,';',/,'only version 1.00 and later is supported.')
      stop
  850 if(ipt.gt.0)call wrapstring(8,0,72,0,1,'ERROR:  '''//
     *   dsn(1:trm(dsn))//''' could not be opened.  Is it perhaps '//
     *   'currently being read by another instance of aML?')
      if(jpt.gt.0)call wrapstring(6,0,72,0,1,'ERROR:  '''//
     *   dsn(1:trm(dsn))//''' could not be opened.  Is it perhaps '//
     *   'currently being read by another instance of aML?')
      stop
  999 return
      end
c=======================================================================
      subroutine getdatainfo_(levelmax,datadim,nold,nbmax,nb2max,
     *   datastructure,datalevel,nxold,namepntr,oldname,status)
 
c     Subroutine to read in arrays with data set information.
c
c     Constantijn Panis, 6 July 1996
 
      implicit none
      integer levelmax,datadim,nold,nb2max,datalevel(datadim),
     *   nbmax(levelmax-1,2:levelmax),nxold(levelmax,datadim),
     *   namepntr(levelmax,datadim),status,i,j
      real datastructure(datadim)
      character oldname(nold)*8
 
      status=0
      call zeroint((levelmax-1)*(levelmax-1),nbmax)
 
c     Read data information insofar as necessary
      read(12,end=899,err=899)((nbmax(i,j),j=2,levelmax),
     *   i=1,levelmax-1)
      read(12,end=899,err=899)datalevel
      do 100 i=1,datadim
         datastructure(i)=real(datalevel(i))
  100 continue
      read(12,end=899,err=899)datalevel
      read(12,end=899,err=899)((nxold(i,j),i=1,datalevel(j)),j=1,
     *   datadim),((namepntr(i,j),i=1,datalevel(j)),j=1,datadim)
      read(12,end=899,err=899)oldname
c     There is additional information on numbers that were deducted from
c     each variable, means, standard deviations, minima, and maxima,
c     but that information is not read in here.
c     Record maximum number of level 2 branches in any observation.
c     Used to dimension struc(nb2max) and xpntr(nb2max).  Needs to be
c     at least 1, even if levelmax=1, because there is always at least
c     one struc (equal to zero).
      if(levelmax.gt.1)nb2max=nbmax(1,2)
      nb2max=max(nb2max,1)
      goto 999
 
  899 status=1
  999 return
      end
