      subroutine getposi(intdist,intdim,intdimc,numdist,sumintdim,
     *   distdim,thisdraw,ndraw,dimused,nested,posi1,posi2,posi4,
     *   bigposi1,bigposi2)
 
c     Subroutine to figure out the positions in arrays related to the
c     current draw of the currently integrated distribution:
c
c     intdist  [in] integrated distribution (scalar!), as counted among
c                   all distributions, not just integrated ones.
c     thisdraw [in] current draws of integrated distributions; only
c                   thisdraw(intdist) is used
c     ndraw    [in] number of draws of distributions in observation
c     distdim  [in] dimensions of defined distributions
c     dimused  [in] indicators of used dimensions in observation
c     nested   [in] integer array indicating how integrated distributions
c                   are nested.  Only relevant here is that if nested(.)=0,
c                   the distribution is non-integrated.
c     intdim  [out] dimension of integrated distribution
c     intdimc [out] compressed dimension of integrated distribution
c     posi1   [out] position of integrated distribution in
c                   one-dimensional non-expanded arrays of dimension
c                   sumintdim (such as dimused, intpntr, intpnts, ...)
c                   These arrays only refer to residuals of integrated
c                   distributions.
c     posi2   [out] idem, two-dimensional arrays of dimension sumdim2
c                   (such as cdm, ddcdm).  These arrays also refer to residuals
c                   of integrated distributions, even though they have
c                   sufficient space allocated to hold all.
c     posi4   [out] idem, four-dimensional arrays of dimension sumdim4
c                   (only dcdmdsigii).  These arrays also refer to residuals
c                   of integrated distributions, even though they have
c                   sufficient space allocated to hold all.
c     bigposi1 [out] position of integrated distribution and currently
c                   integrated draw in one-dimensional expanded arrays
c                   of dimension sumbigdim (such as cmui)
c     bigposi2 [out] idem, two-dimensional arrays of dimension
c                   sumbigintdim2=dimii (such as sigii, dsigii)
c
c     Note the existence of integer functions getposi1 and getbigposi1, which
c     simply return posi1 and bigposi1.
c
c     Constantijn Panis, 24 March 2002
c     Last change: do not use space in arrays that pertains to non-integrated
c     distributions.
 
      implicit none
      integer intdist,intdim,intdimc,numdist,sumintdim,distdim(numdist),
     *   thisdraw(numdist),ndraw(numdist),dimused(sumintdim),
     *   nested(numdist),posi1,posi2,posi4,bigposi1,bigposi2,
     *   dim,bigdim,i,intvecsum
 
      intdim=distdim(intdist)
      bigdim=ndraw(intdist)*intdim
      posi1=1
      posi2=1
      posi4=1
      bigposi1=(thisdraw(intdist)-1)*intdim+1
      bigposi2=(thisdraw(intdist)-1)*intdim*(bigdim+1)+1
      do 100 i=1,intdist-1
c        This is new 3/24/2002:
         if(nested(i).eq.0)goto 100
         dim=distdim(i)
         bigdim=ndraw(i)*dim
         posi1=posi1+dim
         posi2=posi2+dim*dim
         posi4=posi4+dim*dim*dim*dim
         bigposi1=bigposi1+bigdim
         bigposi2=bigposi2+bigdim*bigdim
  100 continue
      intdimc=intvecsum(dimused(posi1),intdim)
 
      return
      end
