      subroutine getword(unit,line,il,i1,i2,status)
 
c     Subroutine to determine begin and end of next word in 'line';
c     the beginning is the first character that is not a blank,
c     horizontal tab (HT) or comma.
c     The end will be marked by any of the following:
c     ' '  HT ',' '(' ':'  ';'  ')' '^' '-' '+' '='
c     '/' '>'  '<'  '!' '[' ']' '|' '&'
c     or, if the word is a parenthesis, (semi) colon or mathematical
c     operator(^, -, +, *, =, <, >, !=, <=, >= | &), then this will be
c     recognized and anything will mark the end.
c     Upon entering, il is the first position to be checked; upon
c     return, il is the position immediately following the word.
c     i1 and i2 are beginning and end positions of the word.
c
c     This routine is similar to nextword, except for unit and wrapping.
c
c     Constantijn Panis, 23 March 2001
c     Last change: status instead of alternate return
 
      implicit none
      integer unit,il,i1,i2,status
      character line*128,c*1,ht*1,ascii*1
 
      status=0
c     ht is the horizontal tab character
      ht=ascii(9)
 
      if(il.gt.80)goto 800
c     skip blanks, tabs and commas:
  110 c=line(il:il)
      if(c.ne.' '.and.c.ne.','.and.c.ne.ht)goto 120
      il=il+1
      if(il.gt.80)then
c        until position 80 all white characters; read next line
         if(unit.le.0)goto 800
         read(unit,'(a80)',end=800,err=800)line
         il=1
      endif
      goto 110
 
c     Position il is not a blank, tab or comma.
c     Next word starts here:
  120 i1=il
c     check whether next word is a parenthesis, (semi)colon or
c     mathematical operator:
      if(line(i1:i1+1).eq.'!='.or.line(i1:i1+1).eq.'>='.or.
     *   line(i1:i1+1).eq.'<='.or.line(i1:i1+1).eq.'||'.or.
     *   line(i1:i1+1).eq.'&&')then
         i2=i1+1
         il=i2+1
         goto 999
      elseif(c.eq.'('.or.c.eq.')'.or.c.eq.':'.or.c.eq.';'.or.c.eq.'-'
     *   .or.c.eq.'+'.or.c.eq.'*'.or.c.eq.'='.or.c.eq.'/'.or.c.eq.'>'
     *   .or.c.eq.'<'.or.c.eq.'['.or.c.eq.']'.or.c.eq.'^'.or.c.eq.'|'
     *   .or.c.eq.'&')then
         i2=i1
         il=i2+1
         goto 999
      endif
c     Next word is not a parenthesis, (semi)colon or mathematical
c     operator; scan next position for delimiter:
      il=il+1
  130 if(il.eq.81)goto 210
      c=line(il:il)
      if(c.eq.'-'.or.c.eq.'+'.or.c.eq.'*'.or.c.eq.'='.or.c.eq.'/'.or.
     *   c.eq.'<'.or.c.eq.'>'.or.c.eq.'!'.or.c.eq.'('.or.c.eq.')'.or.
     *   c.eq.';'.or.c.eq.':'.or.c.eq.'['.or.c.eq.']'.or.c.eq.' '.or.
     *   c.eq.'^'.or.c.eq.ht.or.c.eq.','.or.c.eq.'|'.or.c.eq.'&')
     *   goto 210
      il=il+1
      goto 130
 
c     Parenthesis, (semi)colon, mathematical delimiter, blank, tab or
c     comma or end of line encountered:
  210 i2=il-1
      goto 999
 
  800 status=1
  999 return
      end
