      subroutine ghquad(task,dim,z,wt)
 
c     Subroutine to compute weights and points corresponding to Gauss-
c     Hermite quadrature with n points of support, for approximating
c     a standard normal univariate distribution.
c
c     Arguments:
c     task [in]  1=compute full set;   2=compute half only (rest symmetric)
c     dim  [in]  order of Hermite polynomial
c     z   [out]  Hermite zero solutions (points)
c     wt  [out]  corresponding coefficients (weights)
c
c     Constantijn Panis, 24 November 1995
 
      implicit none
      integer task,dim,i
      double precision z(dim),wt(dim),sqrt2,pitominhalf
c     Note: pitominhalf = pi**-.5
      parameter(sqrt2=1.4142135623730950488d0,
     *   pitominhalf=.5641895835477562869480795d0)
 
      call hermit(2,dim,z,wt)
      do 100 i=1,(dim+1)/2
         z(i)=z(i)*sqrt2
         wt(i)=wt(i)*pitominhalf
         if(task.ne.1)goto 100
         z(dim-i+1)=-z(i)
         wt(dim-i+1)=wt(i)
  100 continue
 
      return
      end
