      subroutine highdraw(nest,thisnest,numdist,ne,draw,cdraw,ccdraw,
     *   ndraw,thisdraw,intdist,nested,status)
 
c     Subroutine to determine the highest draws for all integrated distributions
c     from nest+1 through thisnest, for the current observation.
c
c     nest     [in] current integration nesting level
c     thisnest [in] number of nesting levels.  If thisnest=nest, just
c                   determine whether thisdraw is ever used.  Typically,
c                   thisnest=numnest, but nesting may not go all the way
c                   for certain branches.
c     draw     [in] integrated distribution draws of all outcomes
c     intdist  [in] distributions currently being integrated out
c     thisdraw [in] current draws of intdist(1,...,nest)
c             [out] highest draws of intdist(nest+1,...,thisnest)
c     status  [out] 0 = highest draw determined (draw may be zero)
c                  -1 = there are no outcomes with draws corresponding
c                       to thisdraw(intdist(1,...,nest))
c
c     Constantijn Panis, 6 November 1999
 
      implicit none
      integer nest,thisnest,numdist,ne,draw(numdist,ne),cdraw(ne),
     *   ccdraw,ndraw(numdist),thisdraw(numdist),intdist(thisnest),
     *   nested(numdist),status,ie,i,nn,match,temp,combidraw
 
      status=0
      do 100 nn=nest+1,thisnest
         temp=0
         do 110 ie=1,ne
            do 120 i=1,nn-1
               if(draw(intdist(i),ie).ne.thisdraw(intdist(i)))goto 110
  120       continue
c           This outcome will be evaluated under this draw (except if
c           see below)
            temp=max(temp,draw(intdist(nn),ie))
  110    continue
         thisdraw(intdist(nn))=temp
  100 continue
 
c     Consider the following draw array (Misc\nested6c):
c        ( 1 1 1 1 1 1 1 0 0 )      0
c        ( 0 1 1 1 2 2 2 0 0 )      0
c        ( 1 2 3 4 5 6 7 0 0 )      8
c        ( 0 0 0 0 0 0 0 1 1 )      0
c        ( 0 0 0 0 0 0 0 1 2 )      0
c
c     If nest=1, intdist=(1 2 3), and thisdraw(1)=1, the above will
c     correctly find thisdraw=(1 2 7)
c     However, if thisdraw(1)=0, it will figure that the last two outcomes
c     are evaluated, and set thisdraw=(0 0 0).  The problem is that the
c     last two outcomes are in fact not evaluated under intdist=(1 2 3).
 
      ccdraw=combidraw(numdist,thisdraw,nested,ndraw)
      status=0
      call intmatch(ccdraw,cdraw,ne,match)
      if(match.ne.0)goto 999
c     There is no module with this intdist combination!
      status=-1
 
  999 return
      end
