      subroutine init_corr(corrrestrict,dim,dimp1)
 
c     Subroutine to initialize corrrestrict.
c     Note that only its lower triangle is ever used.
c     The value of corrrestrict p=i+j(dim+1) is such that the row i and
c     column j of the matrix can be found as follows:
c        i = mod(p,dim+1)
c        j = p/(dim+1)
c
c     Often a correlation matrix is stored with just dim*(dim+1)/2
c     elements, the fist dim being standard deviations and the last
c     dim*(dim-1)/2 being correlations.  The location of matrix element
c     (i,j) is simple for i=j (namely location i); if i!=j (correlation),
c     the location in the compressed vector is i+j*dim-j*(j+1)/2.
c
c     Instead of 1+1*dimp1, this version sets corrrestrict(1,1) to 0.
c     If there are any restrictions, this value will be reset to
c     1+1*dimp1 (see subroutine read_restriction).  This information
c     must be used, even when efficiency cannot be gained.
c
c     Constantijn Panis,  2 June 1992
 
      implicit none
      integer dim,corrrestrict(dim,dim),dimp1,i,j
 
      do 100 j=1,dim
         corrrestrict(j,j) = j + j*dimp1
         do 110 i=j+1,dim
            corrrestrict(i,j) = i + j*dimp1
  110    continue
  100 continue
 
      corrrestrict(1,1)=0
 
      return
      end
