      subroutine initdrawpoint(nest,numnest,thisnest,ne,intdist,intdim,
     *   intdimc,numdist,sumintdim,sumdim2,sumdim4,sumbigintdim,dimii,
     *   nfullplus,maxpnts,nec,allint6,posi1,posi2,posi4,bigposi1,
     *   bigposi2,nested,thisdraw,draw,cdraw,ccdraw,ndraw,distdim,
     *   distpos,distyp,indx,dimused,intpnts,intpntr,sigii,cdm,
     *   dcdmdsigii,fullbeta,z,wt,weight,ui,cmui,ifdrv,ipt,jpt,status)
 
c     Subroutine to figure out the Cholesky decomposition of the
c     covariance matrix of the current draw of intdist(*), and the
c     integration points ui.  Also initialize the draws and points
c     of distributions that are strictly more inner than intdist(nest),
c     and to set intdim, intdimc, and lots of pointers to refer to the
c     innermost distribution.
c
c     Upon output, status indicates:
c               -1 = there are no outcomes with draws corresponding
c                    to thisdraw(intdist(1,...,nest-1))
c                0 = normal termination
c                1 = Cholesky decomposition failed
c
c     Note: upon input, posi1 must point at the first position in indx
c     of the currently integrated distribution.
c
c     Constantijn Panis, 20 March 2002
c     Last change: lots of arrays only pertain to integrated distributions
 
      implicit none
      integer nest,numnest,thisnest,ne,intdist(numnest),intdim,intdimc,
     *   numdist,sumintdim,sumdim2,sumdim4,sumbigintdim,dimii,nfullplus,
     *   maxpnts,nec,allint6,posi1,posi2,posi4,bigposi1,bigposi2,
     *   nested(numdist,2),thisdraw(numdist),draw(numdist,ne),cdraw(ne),
     *   ccdraw,ndraw(numdist),distdim(numdist),
     *   distpos(numdist),distyp(numdist),indx(sumintdim),
     *   dimused(sumintdim),intpnts(sumintdim),intpntr(sumintdim),
     *   ipt,jpt,status,nn,countint,getposi1
      double precision sigii(dimii),cdm(sumdim2),dcdmdsigii(sumdim4),
     *   fullbeta(nfullplus),z(maxpnts),wt(maxpnts),weight(numnest),
     *   ui(sumintdim),cmui(sumbigintdim)
      logical ifdrv
 
      status=0
      if(countint(1,indx(posi1),intdim).eq.intdim)then
c        If this is the first heterogeity point of this distribution,
c        figure out the Cholesky decomposition of the covariance matrix:
         call getposi(intdist(nest),intdim,intdimc,numdist,sumintdim,
     *      distdim,thisdraw,ndraw,dimused,nested,posi1,posi2,posi4,
     *      bigposi1,bigposi2)
         if(mod(distyp(intdist(nest)),10).ne.4)then
            call compcdm(1,intdist(nest),intdim,intdimc,posi1,posi2,
     *         posi4,bigposi2,sumintdim,sumdim2,sumdim4,dimii,thisdraw,
     *         ndraw,distdim,dimused,ipt,jpt,status,sigii,0.d0,cdm,
     *         0.d0,dcdmdsigii,ifdrv)
            if(status.gt.0)goto 999
         endif
      endif
      call map_u(1,nfullplus,numdist,sumintdim,sumdim2,sumbigintdim,
     *   intdist(nest),intdim,intdimc,nec,allint6,posi1,posi2,bigposi1,
     *   maxpnts,intpnts,intpntr,dimused,indx,distyp,distpos,fullbeta,
     *   0.d0,z,wt,weight(nest),0.d0,cdm,0.d0,ui,0.d0,cmui)
 
cc     Initialize all distributions except those in intdist(1) through
cc     intdist(nest):
c      do 100 i=1,numdist
c         call intmatch(i,intdist,nest,match)
c         if(match.ne.0)goto 100
c         posi1=getposi1(i,nested,distdim)
c         thisdraw(i)=0
c         call zeroint(distdim(i),indx(posi1))
c  100 continue
 
c     Determine integrated distributions
      thisnest=numnest
      do 105 nn=nest+1,numnest
         if(nn.le.thisnest)then
            call intmatch(intdist(nn-1),nested(1,2),numdist,intdist(nn))
c           Check whether this integration branch has only nn-1 levels.
c           See, for example, Misc\nested6a when nest=1 and intdist(1)=2.
            if(intdist(nn).eq.0)thisnest=nn-1
         else
            intdist(nn)=0
         endif
  105 continue
 
c     Find highest draws for all integrated distributions from nest+1
c     through thisnest, and determine ccdraw.  Also determine whether
c     ccdraw is ever used.
      call highdraw(nest,thisnest,numdist,ne,draw,cdraw,ccdraw,ndraw,
     *   thisdraw,intdist,nested,status)
      if(status.ne.0)goto 999
 
c     Initialize all more inner distributions' points, Cholesky decomposed
c     covariance matrices, and ui's.
      do 110 nn=nest+1,thisnest
         call getposi(intdist(nn),intdim,intdimc,numdist,sumintdim,
     *      distdim,thisdraw,ndraw,dimused,nested,posi1,posi2,posi4,
     *      bigposi1,bigposi2)
         if(thisdraw(intdist(nn)).eq.0)then
            call putint(0,indx(posi1),intdim)
         else
            call putint(1,indx(posi1),intdim)
c           If the distribution is normal, compute the Cholesky
c           decomposition of the covariance matrix of this draw.
            if(mod(distyp(intdist(nn)),10).ne.4)then
               call compcdm(1,intdist(nn),intdim,intdimc,posi1,posi2,
     *            posi4,bigposi2,sumintdim,sumdim2,sumdim4,dimii,
     *            thisdraw,ndraw,distdim,dimused,ipt,jpt,status,sigii,
     *            0.d0,cdm,0.d0,dcdmdsigii,ifdrv)
               if(status.gt.0)goto 999
            endif
         endif
         call map_u(1,nfullplus,numdist,sumintdim,sumdim2,sumbigintdim,
     *      intdist(nn),intdim,intdimc,nec,allint6,posi1,posi2,
     *      bigposi1,maxpnts,intpnts,intpntr,dimused,indx,distyp,
     *      distpos,fullbeta,0.d0,z,wt,weight(nn),0.d0,cdm,0.d0,ui,
     *      0.d0,cmui)
  110 continue
 
c     Start integrating the innermost integrated distribution.
      nest=thisnest
      posi1=getposi1(intdist(nest),nested,distdim)
      intdim=distdim(intdist(nest))
 
  999 return
      end
 
