      subroutine inprod(n,jsa,jsb,a,b,prod)
 
c     Subroutine to compute the innerproduct of two vectors.
c     If the two vectors were simple vectors, double precision function
c     innerp could be used.  Subroutine inprod is useful when
c     the innerproduct of row vectors (stored in a matrix) needs
c     to be taken.  Arguments:
c     n     [in]  number of elements over which innerproduct is taken
c     jsa   [in]  distance between storage locations of subsequent
c                 elements of first vector
c     jsb   [in]  idem for second vector
c     a     [in]  array in which first vector is stored
c     b     [in]  array in which second vector is stored
c     prod  [out] innerproduct of a and b
c
c     Examples of usage:
c     (1) the innerproduct of row i of matrix x(k1,k2) and row j of
c         matrix y(k3,k2):
c         call inprod(k2,k1,k3,x(i,1),y(j,1),prod)
c     (2) the innerproduct of row i of matrix x(k1,k2) and column j
c         of matrix y(k2,k3):
c         call inprod(k2,k1,1,x(i,1),y(1,j),prod)
c     (3) the innerproduct of column i of matrix x(k1,k2) and column j
c         of matrix y(k1,k3):
c         call inprod(k1,1,1,x(1,i),y(1,j),prod)
c         This is equivalent to:
c         prod = innerp(x(i,1),y(1,j),k1)
c     Note that prod=zero is returned if n is zero or less.
c
c     Constantijn Panis,  30 November 1990
 
      implicit none
      double precision a(*),b(*),prod,temp
      integer n,jsa,jsb,i,j
 
      if(n.le.0)then
         prod = 0.d0
         goto 999
      endif
      temp = 0.d0
      j=1
      do 100 i=1,jsa*n,jsa
         temp = temp + a(i)*b(j)
         j = j + jsb
  100 continue
      prod = temp
 
  999 return
      end
 
