      subroutine maxlnl(maxcc,ccmax,nfull,nfree,nchol,ipt,jpt,fullbeta,
     *   newfullbeta,chol,dfreebeta,fp,fp2,fpt,fpt2,sp,spt,sp2,sp3,
     *   searchdir,convcrit,lnl,iffull,maxiter,numsearch,output,stprng,
     *   fulltype,fullname,converged,locdim,dd,rr,ii,ll,cc)
 
c     Subroutine to search over parameters to maximize the log-likelihood.
c
c     Constantijn Panis, 16 May 2004
c     Last change: More care with non-definite (numerical) Hessian
 
      implicit none
      integer maxcc,ccmax
      double precision dd(maxcc/8)
      real rr(maxcc/4)
      integer ii(maxcc/4),locdim(120)
      logical ll(maxcc/4)
      character cc(maxcc)*1
      integer nfull,nfree,nchol,ipt,jpt
      double precision fullbeta(nfull),newfullbeta(nfull),chol(nchol),
     *   dfreebeta(nfree),fp(nfree),fp2(nfree),fpt(nfree),fpt2(nfree),
     *   sp(nfree,nfree),spt(nfree,nfree),sp2(nfree,nfree),
     *   sp3(nfree,nfree),searchdir(nfree),convcrit(4,2),lnl,
     *   eigenvalues(5),stepsize,oldlnl,dpna,innerp
      integer iffull(nfull),maxiter,numsearch,output(10),stprng(3),
     *   fulltype(nfull),iter,status,task,poseig
      character fullname(nfull)*8
      logical converged,isna,isnan,isinf
 
      converged=.false.
      stepsize=1.d0
      oldlnl=0.d0
      poseig=0
 
      if(maxiter.eq.0.and.output(3).eq.1)then
c        Gradient check only, no searching:
         if(nchol.gt.0)call prepchol(nfull,fulltype,ipt,jpt,nchol,
     *      chol,fullbeta,fullname)
         call gradcheck(maxcc,ccmax,nfull,nfree,fullbeta,fulltype,
     *      iffull,lnl,nchol,chol,dfreebeta,fp,fp2,fpt,fpt2,sp,spt,
     *      fullname,locdim,dd,rr,ii,ll,cc,ipt,jpt)
         if(ipt.gt.0)write(8,101)lnl
         if(jpt.gt.0)write(6,101)lnl
  101    format(/,70('='),//,'Log Likelihood:',f23.8,//,70('='))
         goto 999
      elseif(maxiter.eq.0)then
c        Zero iterations, no gradient check: user error
         goto 800
      endif
 
c     Optimization loop begins here
      do 200 iter=1,maxiter
 
c        First compute (update) Cholesky decompositions of covariance
c        matrices that are estimated using Cholesky decompositions:
         if(nchol.gt.0)call prepchol(nfull,fulltype,ipt,jpt,nchol,
     *      chol,fullbeta,fullname)
 
c        Now compute the function value and first and second derivatives
         if(iter.eq.1)then
            if(ipt.gt.0)write(8,201)
            if(jpt.gt.0)write(6,201)
  201       format(/,70('='),/,'=',22x,'RESULTS OF OPTIMIZATION',
     *         23x,'=',/,70('='),/)
            if(output(3).eq.1)then
               call gradcheck(maxcc,ccmax,nfull,nfree,fullbeta,fulltype,
     *            iffull,lnl,nchol,chol,dfreebeta,fp,fp2,fpt,fpt2,
     *            sp,spt,fullname,locdim,dd,rr,ii,ll,cc,ipt,jpt)
            else
               task=111+2*numsearch
               call amlfunc(task,maxcc,ccmax,nfull,nfree,fullbeta,
     *            fulltype,iffull,lnl,nchol,chol,dfreebeta,fp,fp2,
     *            fpt,fpt2,sp,spt,locdim,dd,rr,ii,ll,cc,ipt,jpt,1)
            endif
         else
            task=11+2*numsearch
            call amlfunc(task,maxcc,ccmax,nfull,nfree,fullbeta,fulltype,
     *         iffull,0.d0,nchol,chol,dfreebeta,fp,fp2,fpt,fpt2,sp,spt,
     *         locdim,dd,rr,ii,ll,cc,ipt,jpt,1)
         endif
         if(isnan(lnl))stop
 
c        If all is well,
c        the Hessian is negative definite, so that -H is positive definite and
c        we can use definv(1,...).  If the Hessian was calculated with the BHHH
c        method, it will be definite (or semi-definite if some parameter is not
c        identified.  Definv handles semi-definite beautifully.)  However, if th
c        Hessian was computed numerically, it sometimes is not quite definite.
c        This happens when some parameter is not or poorly identified, and perha
c        also when we are far from the optimum.  In such cases, switch to BHHH
c        search for one iteration (and do not permit convergence).
c        Determine whether Hessian is negative definite on the basis of eigenval
c        (If all are -, matrix is negative definite; if some are zero, semi-defi
c        if mixed signs, non-definite.)  Use sp2 as scratch array.
  210    call copy8(spt,sp2,nfree*nfree)
         call eigen(1,sp2,0.d0,nfree)
c        Eigenvalues are in descending order.  They are all supposedly negative,
c        so that the largest eigenvalues are the closest to zero.
         call copy8(sp2,eigenvalues,min(5,nfree))
 
c                                        -1          -1
c        Compute the search direction, -H  * F = (-H)  * F
c        where H is the Hessian matrix (second derivatives).
         call opposite(spt,sp2,nfree*nfree)
         if(eigenvalues(1).gt.0.d0.and.numsearch.eq.1)then
c           Hessian was numerically computed.  Largest eigenvalue is
c           positive, i.e., the Hessian is not negative definite.
c           Proceed but do not allow convergence.
            call nrminv(1,sp2,sp3,nfree,0.d0,0.d0,status)
            if(status.eq.0)then
               call copy8(sp3,sp2,nfree*nfree)
            else
               numsearch=2
               if(ipt.gt.1)write(8,211)
               if(jpt.gt.1)write(6,211)
  211          format(/,'WARNING: the likelihood surface is not ',
     *            'concave.  In addition, the matrix',/,'of second ',
     *            'derivatives is singular, i.e., the model is not ',
     *            'identified (or',/,'very far from convergence).  ',
     *            'For this iteration only, aML switches to',/,
     *            'the BHHH approximation of second derivatives.')
               call amlfunc(11,maxcc,ccmax,nfull,nfree,fullbeta,
     *            fulltype,iffull,0.d0,nchol,chol,dfreebeta,fp,fp2,fpt,
     *            fpt2,sp,spt,locdim,dd,rr,ii,ll,cc,ipt,jpt,1)
               goto 210
            endif
         else
c           Hessian was computed by BHHH approximation which guarantees
c           negative (semi-)definite except perhaps due to rounding
c           error.  Definv deals well with singular matrices and appears
c           to deal well with ever-so-slightly non-definite matrices.
c           Hessian may also be computed numerically and was confirmed
c           negative definite.
            call definv(1,sp2,nfree,status)
            if(status.ne.0)call bug(ipt,jpt)
         endif
c        sp2 contains the inverse of spt.
c        searchdir is the search direction in transformed parameters:
         call m2mult(000,nfree,nfree,1,sp2,fpt,searchdir,0)
 
c        Compute and test convergence criteria.
c        convcrit(1,*) = weighted gradient norm (untransformed)
c        convcrit(2,*) = relative function improvement
c        convcrit(3,*) = gradient norm (untransformed)
c        convcrit(4,*) = relative step size (maximum relative parameter change)
 
c        Compute weighted gradient norm of untransformed parameters.
c        Potential issue if the Hessian was computed numerically and is
c        not negative definite.
         call copy8(sp,sp2,nfree*nfree)
         call eigen(1,sp2,0.d0,nfree)
         call copy8(sp2,eigenvalues,min(5,nfree))
         call opposite(sp,sp2,nfree*nfree)
         if(eigenvalues(1).le.0.d0)then
            call definv(1,sp2,nfree,status)
            if(status.ne.0)call bug(ipt,jpt)
         else
            call nrminv(1,sp2,sp3,nfree,0.d0,0.d0,status)
            if(status.eq.0)call copy8(sp3,sp2,nfree*nfree)
         endif
         call quadratic(1,1,nfree,fp,sp2,convcrit(1,2),0)
         if(status.eq.0.and.convcrit(1,2).ge.0.d0)then
            convcrit(1,2) = dsqrt(convcrit(1,2))
         else
c           Hessian is both non-definite and singular.
            convcrit(1,2) = dpna()
         endif
c        Compute relative function improvement:
         if(iter.gt.1.and.oldlnl.ne.0.d0)then
            convcrit(2,2)=(lnl-oldlnl)/dabs(oldlnl)
         else
c           Not Applicable:
            convcrit(2,2)=dpna()
         endif
c        Compute gradient norm:
         convcrit(3,2) = dsqrt(innerp(fp,fp,nfree))
c        Compute relative step size:
         call fignewbeta(2,nfull,nfree,fulltype,iffull,fullbeta,
     *      searchdir,1.d0,0.d0,convcrit(4,2),nchol,chol,status)
         if(status.ne.0)then
c           Problem in fignewbeta (parameter overflow)
            converged=.false.
         elseif(eigenvalues(1).gt.0.d0.and.numsearch.eq.1)then
c           Numerically computed Hessian was non-definite; do not allow converge
            converged=.false.
         elseif(numsearch.eq.2)then
c           Temporary switch from numerical to BHHH; do not allow convergence
            converged=.false.
            numsearch=1
         else
            converged = (
     *(.not.isna(convcrit(1,2)).and.convcrit(1,2).le.convcrit(1,1)).or.
     *(.not.isna(convcrit(2,2)).and.convcrit(2,2).le.convcrit(2,1)).or.
     *                              convcrit(3,2).le.convcrit(3,1).or.
     *(.not.isinf(convcrit(4,2)).and.convcrit(4,2).le.convcrit(4,1)))
         endif
 
c        Print out iteration results
         call writeiter(nfull,nfree,fulltype,iffull,output,ipt,jpt,
     *      iter,fullbeta,newfullbeta,fp,eigenvalues,nchol,chol,
     *      searchdir,oldlnl,lnl,convcrit(1,2),fullname)
 
c        Place current function value into "previous" function value.
         oldlnl=lnl
 
         if(converged)goto 999
 
c        If iter=maxiter there is no point in any more searching, since
c        newly found parameters will not be used to find correct standard
c        deviations.   Note the special case where maxiter=1: the user
c        only wants to compute correct standard errors for some
c        parameter configuration.
         if(iter.eq.maxiter)goto 200
 
c        Start stepsize algorithm, searching for improved function value
         call searchmax(maxcc,ccmax,nfull,nfree,stprng,fulltype,iffull,
     *      ipt,jpt,fullbeta,newfullbeta,searchdir,nchol,chol,dfreebeta,
     *      fp,fp2,fpt,fpt2,oldlnl,lnl,stepsize,locdim,
     *      dd,rr,ii,ll,cc,status)
         if(status.ne.0)goto 310
 
  200 continue
 
c     Failure to converge after maxiter iterations
      if(ipt.gt.0)then
         call writeaia(8,72,1,'ERROR:  convergence not achieved after ',
     *      maxiter,' iterations.')
         write(8,'(1x)')
      endif
      if(jpt.gt.0)then
         call writeaia(6,72,1,'ERROR:  convergence not achieved after ',
     *      maxiter,' iterations.')
         write(6,'(1x)')
      endif
 
      converged=.false.
      goto 999
 
c     Failure to improve likelihood:
  310 if(ipt.gt.0)write(8,311)
      if(jpt.gt.0)write(6,311)
  311 format(/,'ERROR:  failure to improve likelihood function;',/,
     *   'optimum may not have been reached.')
      convcrit(2,2)=0.d0
      converged=.false.
      goto 999
 
  800 if(ipt.gt.0)write(8,801)
      if(jpt.gt.0)write(6,801)
  801 format(/,'ERROR:  zero iterations specified.')
      stop
  999 return
      end
