      subroutine onebin(neb,draw,cdraw,p5,dddp5,
     *   epsb,lnlg,v5,ifdrv,ipt,jpt,status)
 
c     Subroutine to compute the conditional (on heterogeneity) integrand
c     of all binomial modules.
c
c     Upon output, status may be
c        unchanged with normal execution
c       -1 likelihood underflow
c        3 with illegitimate probability
c
c     Constantijn Panis, 10 December 2002
c     Last change: less underflow detection
 
      implicit none
      integer neb,draw(neb),cdraw,ie,ipt,jpt,status,n,y
      double precision p5(neb),dddp5(neb),epsb(neb),lnlg,theta,dtheta,
     *   pnorm1,dnorm1
      real v5(3*neb)
      logical ifdrv
 
      status=0
c     Compute the (log of the) integrand theta**Y * (1-theta)**(N-Y)
      do 100 ie=1,neb
c        Do we need to evaluate this (conditional) likelihood module?
         if(cdraw.ne.draw(ie))goto 100
c        Theta is the binomial probability:
         if(v5(neb+ie).eq.1.)then
c           Logistic transformation:
            theta = 1.d0/(1.d0+dexp(-p5(ie)-epsb(ie)))
         elseif(v5(neb+ie).eq.2.)then
c           Probit transformation:
            theta = pnorm1(p5(ie)+epsb(ie))
         elseif(v5(neb+ie).eq.3.)then
c           No transformation (no integrated residual allowed):
            theta = p5(ie)
         endif
         y=int(v5(ie))
         n=int(v5(ie+2*neb))
         if(theta.le.0.d0.or.theta.ge.1.d0)then
            if(theta.lt.0.d0.or.theta.gt.1.d0)goto 800
            if(theta.eq.0.d0.and.y.ne.0)goto 900
            if(theta.eq.1.d0.and.y.ne.n)goto 900
c           No contribution to likelihood or derivatives
            if(ifdrv)dddp5(ie)=0.d0
            goto 100
         else
            lnlg = lnlg + dble(y)*dlog(theta)+dble(n-y)*dlog(1.d0-theta)
         endif
         if(.not.ifdrv)goto 100
 
c        Derivatives of log-integrand wrt p5 (= wrt epsb):
         dtheta = dble(y)/theta - dble(n-y)/(1.d0-theta)
         if(v5(neb+ie).eq.1.)then
            dddp5(ie) = dtheta*theta*(1.d0-theta)
         elseif(v5(neb+ie).eq.2.)then
            dddp5(ie) = dtheta*dnorm1(p5(ie)+epsb(ie))
         elseif(v5(neb+ie).eq.3.)then
            dddp5(ie) = dtheta
         endif
  100 continue
      goto 999
 
c     Illegitimate probability.  Only possible with probability
c     without transformation.
  800 if(ipt.gt.0)write(8,801)
      if(jpt.gt.0)write(6,801)
  801 format(/,'ERROR: The function evaluation failed because of an ',
     *   'illegitimate',/,'binomial probability.  Use the ',
     *   'LOGISTIC or PROBIT',/,'probability transformation, or ',
     *   'restrict the range of the probability',/,'parameter to be ',
     *   'in (0,1).')
      status=3
      goto 999
 
c     Likelihood underflows to zero:
  900 status=-1
  999 return
      end
