      subroutine onecontin(nec,draw,cdraw,xbc,epsc,dxbc,a6,
     *   lnlg,siga6a6,dsiga6a6,siga6a6ia6,lndeta6a6,y,ifdrv)
 
c     Subroutine to compute the log-likelihood and derivatives of a
c     particular heterogeneity group for continuous models.  This routine
c     returns the log-likelihood, not the likelihood.
c
c     Derivatives wrt a6, xbc and epsc are identical (up to sign)
c     and returned in dxbc.
c
c     BUG:  replace dlog(twopi) by log2pi (not really a bug)
c
c     Note:  siga6a6 contains the inverse of the loaded and
c     accumulated covariance matrix siga6a6.
c
c     Constantijn Panis, 10 December 2002
c     Last change: no more underflow detection
 
      implicit none
      integer nec,draw(nec),cdraw,i,j
      double precision twopi,lnlg,xbc(nec),epsc(nec),
     *   dxbc(nec),a6(nec),siga6a6ia6(nec),lndeta6a6,
     *   siga6a6(nec,nec),dsiga6a6(nec,nec),temp
      parameter(twopi=6.283185307179586476925287d0)
      real y(nec)
      logical ifdrv
 
c     Do we need to evaluate this (conditional) likelihood module?
c     Note that multiple draws from an integrated distribution is
c     not supported. This would require processing continuous
c     modules in blocks, which in turn would require checking that
c     the covariance matrix of non-integrated residuals is
c     correspondingly block-diagonal.  This would get too complicated.
c     Instead, the user may achieve the same by specifying residuals
c     as non-integrated.  Subroutine figdata4 checks that this
c     condition is complied with.
c     Here check that the first outcome's draw corresponds to the
c     current draw being evaluated.  (It remains possible to draw
c     multiple draws from an integrated distribution, as long as
c     all continuous modules use the same draw.)
      if(cdraw.ne.draw(1))goto 999
 
c     compute a6 = Y-(BX+epsc):
      do 100 i=1,nec
         a6(i) = dble(y(i))-xbc(i)-epsc(i)
  100 continue
 
c     compute a6'siga6a6i:
      call m2mult(100,1,nec,nec,a6,siga6a6,siga6a6ia6,0)
 
c     compute the quadratic form a6'*siga6a6i*a6
      call m2mult(0,1,nec,1,siga6a6ia6,a6,temp,0)
 
c     L = constant / sqrt(determinant) * exp(-.5*a6'*siga6a6i*a6)
c        where constant = (2*pi)**(-.5*nec)
      lnlg=lnlg-.5d0*(nec*dlog(twopi)+lndeta6a6+temp)
 
c     compute derivatives (if desired):
      if(.not.ifdrv)goto 999
 
c     derivatives of lnL wrt a6 happen to be a6'siga6a6i:
      call copy8(siga6a6ia6,dxbc,nec)
 
c     derivatives of lnL wrt siga6a6:
c     derivative of ln(deta6a6) wrt a parameter is
c     trace(siga6a6i*derivative of siga6a6 wrt that parameter);
c     if the parameter is element (i,j) of siga6a6, this collapses to
c     element (j,i) of siga6a6i = element (i,j) of siga6a6i.
c     derivative of (a6'*siga6a6i*a6) wrt a siga6a6 parameter is
c     -a6'*siga6a6i*(deriv of siga6a6 wrt parameter)*siga6a6i*a6;
c     if the parameter is element (i,j) of siga6a6, this collapses to
c     -siga6a6ia6(i)*siga6a6ia6(j)
      do 300 i=1,nec
         do 310 j=1,i
            dsiga6a6(i,j) = .5d0*
     *         (-siga6a6(i,j) + siga6a6ia6(i)*siga6a6ia6(j))
            dsiga6a6(j,i) = dsiga6a6(i,j)
  310    continue
  300 continue
 
  999 return
      end
