      subroutine onehazard(nfullplus,neh,draw,cdraw,lnl,epsh,
     *   dxbh,lnla,lnlb,dlnla,dlnlb,dlnl,v1,ifdrv,status)
 
c     Subroutine to add heterogeneity (mixture) to hazard episodes.
c     Note:  this routine accumulates the log-likelihood, not the
c     likelihood.  Similarly, derivates are wrt the log-likelihood.
c     Upon return, status is
c        0 if normal execution
c       -1 if the likelihood underflows to zero
c
c     Note:  derivatives wrt epsh are returned as dxbh, even though
c     it is not quite the same as derivatives wrt hazard regressor
c     sets.  (They would be if the regressor sets were not varying by
c     subbranch.)  The reason for not labelling them depsh is that
c     another name must be found for rolling derivatives over to more
c     outer integrated distributions.
c
c     Constantijn Panis, 10 December 2002
c     Last change: less underflow detection
 
      implicit none
      integer nfullplus,neh,censor,draw(neh),cdraw,status,ie
      double precision lnl,epsh(neh),dxbh(neh),lnla(neh),lnlb(neh),
     *   dlnla(nfullplus,neh),dlnlb(nfullplus,neh),dlnl(nfullplus),
     *   expeps,logtmp1,logtmp2,logtmp3,tmp1,tmp2,tmp3,tmp4,tmp5,minexp
      real v1(4*neh)
      logical ifdrv
      tmp2=0.d0
      status=0
 
      do 100 ie=1,neh
c        Do we need to evaluate this (conditional) likelihood module?
         if(cdraw.ne.draw(ie))goto 100
 
         expeps=dexp(epsh(ie))
         logtmp1 = expeps*lnla(ie)
         if(logtmp1.lt.minexp())then
            if(censor.eq.0)goto 900
            tmp1=0.d0
         else
            tmp1=dexp(logtmp1)
         endif
         censor=int(v1(4*ie-3))
         if(censor.eq.0)then
            logtmp2 = expeps*lnlb(ie)
            if(logtmp2.lt.minexp())then
               tmp2=0.d0
               tmp3=tmp1
               logtmp3=logtmp1
            else
               tmp2=dexp(logtmp2)
               tmp3=tmp1-tmp2
c              Rounding error may cause tmp3 to be slightly negative:
               if(tmp3.le.0.d0)goto 900
               logtmp3=dlog(tmp3)
               if(logtmp3.lt.minexp())goto 900
            endif
         else
            tmp3=tmp1
            logtmp3=logtmp1
         endif
         lnl=lnl+logtmp3
         if(.not.ifdrv)goto 100
         if(censor.eq.1.or.tmp2.eq.0.d0)then
            call add(dlnla(1,ie),dlnl,expeps,nfullplus)
            dxbh(ie)=logtmp1
         else
            tmp4=expeps*tmp1/tmp3
            tmp5=expeps*tmp2/tmp3
            call add2(dlnla(1,ie),dlnlb(1,ie),dlnl,tmp4,-tmp5,nfullplus)
            dxbh(ie)=(tmp1*logtmp1-tmp2*logtmp2)/tmp3
         endif
  100 continue
      goto 999
 
c     Likelihood underflows to zero
  900 status=-1
  999 return
      end
