      subroutine oneprobit(nep,draw,cdraw,bdc,lnlg,tauminxb,
     *   ddtau,ddxb,sigma44,ddsigma44,cmu4,eps4,ifdrv,ibnnid4,
     *   ipt,jpt,status)
 
c     Subroutine to compute (ordered) probit probabilities for one
c     particular heterogeneity (mixture) group.
c
c     Upon return, status is
c        0 if normal execution
c       -1 if the likelihood underflows to zero
c        1 if a standard deviation was zero (message is written)
c        2 if thresholds were not ordered (message is written)
c        4 if correlation matrix was not positive definite (message written)
c
c     Note: this routine accumulates the log-likelihood, not the
c     likelihood.
c
c     Constantijn Panis, 15 December 1996
c     Last change: introduce draw numbers.
 
      implicit none
      integer nep,draw(nep),cdraw,bdc(nep),ipt,jpt,status,ie,i,
     *   blocknep
      double precision lnlg,tauminxb(2,nep),ddtau(2,nep),ddxb(nep),
     *   sigma44(nep,nep),ddsigma44(nep,nep),cmu4(nep),eps4(nep),
     *   subsigma44(9),ddsubsigma44(9),prob
      logical ifdrv,ibnnid4
 
      status=0
      if(nep.eq.1.or.ibnnid4)then
c        Non-integrated residuals are Independently But Not Necessarily
c        Identically Distributed, so that the probits are independent.
         do 100 ie=1,nep
c           Do we need to evaluate this (conditional) likelihood module?
            if(cdraw.ne.draw(ie))goto 100
            call ordprobit(prob,cmu4(ie)+eps4(ie),sigma44(ie,ie),
     *         tauminxb(1,ie),ddtau(1,ie),ddxb(ie),ddsigma44(ie,ie),
     *         ifdrv,ipt,jpt,status)
            if(prob.le.0.d0)status=-1
            if(status.ne.0)goto 999
            lnlg=lnlg+dlog(prob)
            if(.not.ifdrv)goto 100
c           Translate derivatives wrt likelihood into wrt log-likelihood:
            ddxb(ie)=ddxb(ie)/prob
            ddtau(1,ie)=ddtau(1,ie)/prob
            ddtau(2,ie)=ddtau(2,ie)/prob
            ddsigma44(ie,ie)=ddsigma44(ie,ie)/prob
  100    continue
      elseif(nep.le.3)then
c        Bi- or trivariate dependent probits.  Standard errors are on
c        diagonal of sigma44, correlations off-diagonally.
c        Do we need to evaluate this (conditional) likelihood module?
c        Figdata4 made sure that draw is the same for all nep outcomes,
c        so check only the first one.
         if(cdraw.ne.draw(1))goto 999
         call multiprobit(nep,cmu4,eps4,tauminxb,sigma44,prob,ifdrv,
     *      ddtau,ddxb,ddsigma44,ipt,jpt,status)
         if(status.ne.0)goto 999
         lnlg=lnlg+dlog(prob)
         if(.not.ifdrv)goto 999
c        Translate derivatives wrt likelihood into wrt log-likelihood:
         call mult(ddxb,1.d0/prob,ddxb,nep)
         call mult(ddtau,1.d0/prob,ddtau,2*nep)
         call mult(ddsigma44,1.d0/prob,ddsigma44,nep*nep)
      else
c        More than three correlated probits.  Only feasible if the
c        covariance structure is block-diagonal with blocks of dim<=3.
c        See subroutine blockdiagcovar.
         ie=0
         blocknep=1
  200    ie=ie+blocknep
         if(ie.gt.nep)goto 999
         blocknep=1
c        Do we need to evaluate this (conditional) likelihood module?
         if(cdraw.ne.draw(ie))goto 200
c        Is this module correlated with others?
         if(bdc(ie).ne.ie)call bug(ipt,jpt)
         i=ie
  210    i=i+1
         if(i.gt.nep)goto 220
         if(bdc(i).ne.ie)goto 220
         blocknep=blocknep+1
         goto 210
  220    if(blocknep.eq.1)then
            call ordprobit(prob,cmu4(ie)+eps4(ie),sigma44(ie,ie),
     *         tauminxb(1,ie),ddtau(1,ie),ddxb(ie),ddsigma44(ie,ie),
     *         ifdrv,ipt,jpt,status)
            if(prob.le.0.d0)status=-1
            if(status.ne.0)goto 999
            lnlg=lnlg+dlog(prob)
            if(.not.ifdrv)goto 200
c           Translate derivatives wrt likelihood into wrt log-likelihood:
            ddxb(ie)=ddxb(ie)/prob
            ddtau(1,ie)=ddtau(1,ie)/prob
            ddtau(2,ie)=ddtau(2,ie)/prob
            ddsigma44(ie,ie)=ddsigma44(ie,ie)/prob
         elseif(blocknep.le.3)then
            call msub(sigma44,nep,nep,ie,ie,
     *         subsigma44,blocknep,blocknep,1,1,blocknep,blocknep)
            call multiprobit(blocknep,cmu4(ie),eps4(ie),tauminxb(1,ie),
     *         subsigma44,prob,ifdrv,ddtau(1,ie),ddxb(ie),ddsubsigma44,
     *         ipt,jpt,status)
            if(status.ne.0)goto 999
            lnlg=lnlg+dlog(prob)
            if(.not.ifdrv)goto 200
c           Translate derivatives wrt likelihood into wrt log-likelihood:
            call mult(ddxb(ie),1.d0/prob,ddxb(ie),blocknep)
            call mult(ddtau(1,ie),1.d0/prob,ddtau(1,ie),2*blocknep)
            call mult(ddsubsigma44,1.d0/prob,ddsubsigma44,
     *         blocknep*blocknep)
            call msub(ddsubsigma44,blocknep,blocknep,1,1,
     *         ddsigma44,nep,nep,ie,ie,blocknep,blocknep)
         else
            call bug(ipt,jpt)
         endif
         goto 200
      endif
 
  999 return
      end
