      subroutine optim(maxcc,ccmax,dd,rr,ii,ll,cc,ipt,jpt,nchol,ngrid,
     *   nfull,nfreemax,maxiter,output,numsearch,iiffull,iffull,
     *   fulltype,locdim,chol,dfreebeta,fp,fp2,fpt,fpt2,sp,numsp,spt,
     *   sp2,sp3,grid,gridpos,fullbeta,newfullbeta,searchdir,
     *   convcrit,stprng,fullname,freename,gridfile)
 
c     Subroutine to optimize the likelihood function.  There may be
c     multiple rounds of estimation, in which alternative subsets
c     of parameters are free.
c
c     Also write out the (intermediate) results of estimation.
c
c     Constantijn Panis, 21 November 2002
c     Last change: fp2, fpt2, numsearch
 
      implicit none
      integer maxcc,ccmax
      double precision dd(maxcc/8)
      real rr(maxcc/4)
      integer ii(maxcc/4)
      logical ll(maxcc/4)
      character cc(maxcc)*1
      double precision convcrit(4,2)
      integer ipt,jpt,ngrid,nchol,nfull,nfreemax,maxiter,output(10),
     *   numsearch,gridpos,iiffull,iffull,fulltype,locdim(120),stprng(3)
      integer grid,chol,dfreebeta,fp,fp2,fpt,fpt2,sp,numsp,spt,sp2,sp3,
     *   fullbeta,newfullbeta,searchdir
      integer fullname,freename
      character gridfile*128
 
      call optim_(maxcc,ccmax,dd,rr,ii,ll,cc,ipt,jpt,nchol,ngrid,
     *   nfull,nfreemax,maxiter,output,numsearch,ii(gridpos),
     *   ii(gridpos+ngrid),ii(gridpos+2*ngrid),ii(gridpos+3*ngrid),
     *   ii(iiffull),ii(iffull),ii(fulltype),locdim,
     *   dd(chol),dd(dfreebeta),dd(fp),dd(fp2),dd(fpt),dd(fpt2),
     *   dd(sp),dd(numsp),dd(spt),dd(sp2),dd(sp3),
     *   dd(grid),dd(fullbeta),dd(newfullbeta),
     *   dd(searchdir),convcrit,stprng,cc(fullname),
     *   cc(freename),gridfile)
 
  999 return
      end
c==========================================================================
      subroutine optim_(maxcc,ccmax,dd,rr,ii,ll,cc,ipt,jpt,nchol,ngrid,
     *   nfull,nfreemax,maxiter,output,numsearch,gridpos,gridnum,
     *   gridpoint,gridbest,iiffull,iffull,fulltype,locdim,chol,
     *   dfreebeta,fp,fp2,fpt,fpt2,sp,numsp,spt,sp2,sp3,grid,
     *   fullbeta,newfullbeta,searchdir,convcrit,stprng,fullname,
     *   freename,gridfile)
 
      implicit none
      integer maxcc,ccmax
      double precision dd(maxcc/8)
      real rr(maxcc/4)
      integer ii(maxcc/4)
      logical ll(maxcc/4)
      character cc(maxcc)*1
      integer ipt,jpt,nchol,ngrid,nfull,nfreemax,maxiter,output(10),
     *   numsearch,gridpos(ngrid),gridnum(ngrid),gridpoint(ngrid),
     *   gridbest(ngrid),iiffull(nfull),iffull(nfull),fulltype(nfull),
     *   locdim(120),stprng(3)
      double precision chol(nchol),dfreebeta(nfreemax),fp(nfreemax),
     *   fp2(nfreemax),fpt(nfreemax),fpt2(nfreemax),
     *   sp(nfreemax*nfreemax),numsp(nfreemax*nfreemax),
     *   spt(nfreemax*nfreemax),sp2(nfreemax*nfreemax),
     *   sp3(nfreemax*nfreemax),grid(ngrid,2),fullbeta(nfull),
     *   newfullbeta(nfull),searchdir(nfreemax),convcrit(4,2)
      logical converged
      character fullname(nfull)*8,freename(nfreemax)*8,gridfile*128
c     Locally required:
      integer round,nrounds,nfree,oldnfree,intvecmax
      double precision lnl,oldlnl
 
c     Perform grid search, if applicable:
      if(ngrid.gt.0)call gridsearch(maxcc,ccmax,dd,rr,ii,ll,cc,ipt,jpt,
     *   nchol,ngrid,nfull,nfreemax,gridpos,gridnum,gridpoint,
     *   gridbest,iiffull,iffull,fulltype,locdim,chol,dfreebeta,fp,fp2,
     *   fpt,fpt2,grid,fullbeta,fullname,freename,gridfile)
 
      round=0
c     Number of rounds:
      nrounds=int(log(real(intvecmax(iiffull,nfull))+.5)/log(2.))+1
 
c     Assign iffull and figure out how many parameters are free this round.
  100 call nextround(nfull,iiffull,iffull,round,nrounds,nfree,ipt,jpt)
 
c     Maximize the likelihood:
      call maxlnl(maxcc,ccmax,nfull,nfree,nchol,ipt,jpt,fullbeta,
     *   newfullbeta,chol,dfreebeta,fp,fp2,fpt,fpt2,sp,spt,sp2,sp3,
     *   searchdir,convcrit,lnl,iffull,maxiter,numsearch,output,stprng,
     *   fulltype,fullname,converged,locdim,dd,rr,ii,ll,cc)
 
c     If this run was merely a gradient check, wrap up:
      if(maxiter.eq.0)goto 999
 
      if(.not.converged.or.round.eq.nrounds)then
c        If results with numerical standard errors or Huber-correction
c        need to be written, we need the numerical Hessian.  Compute it
c        if it has not already been courtesy of numerical search.
         if(numsearch.eq.1)then
            call copy8(sp,numsp,nfree*nfree)
c           If Huber compute BHHH Hessian:
            if(output(10).eq.1)call amlfunc(022,maxcc,ccmax,nfull,nfree,
     *         fullbeta,fulltype,iffull,0.d0,nchol,chol,dfreebeta,
     *         fp,fp2,fpt,fpt2,sp,0.d0,locdim,dd,rr,ii,ll,cc,ipt,jpt,0)
         elseif(output(9).eq.1.or.output(10).eq.1)then
            call amlfunc(024,maxcc,ccmax,nfull,nfree,fullbeta,fulltype,
     *         iffull,0.d0,nchol,chol,dfreebeta,fp,fp2,fpt,fpt2,numsp,
     *         0.d0,locdim,dd,rr,ii,ll,cc,ipt,jpt,0)
         endif
      endif
 
c     Write out the results:
      call writeresults(nfull,nfree,oldnfree,output,numsearch,round,
     *   nrounds,iffull,ipt,jpt,convcrit,lnl,oldlnl,fullbeta,
     *   sp,numsp,spt,sp2,sp3,fullname,freename,converged)
c     Optimize the next round, if any:
      if(converged.and.round.lt.nrounds)goto 100
 
  999 return
      end
