      double precision function pchi2(x,df)
 
c     Function to compute the probability that a Chi-2 distributed
c     variable exceeds a certain value:  P(chi2>x) with df degrees
c     of freedom.
c
c     Obtained from Ajay Shah in C, adapted from
c        Hill, I. D. and Pike, M. C.  Algorithm 299
c        Collected Algorithms for the CACM 1967 p. 243
c     Updated for rounding errors based on remark in ACM TOMS,
c        June 1985, page 185.
c
c     If x<0 or df<1, 1 is returned.
c
c     Constantijn Panis, 19 June 1992
 
      implicit none
      double precision bigx,lnsqrtpi,isqrtpi,x,xx,a,y,s,e,c,z,pnorm1
c     bigx = cutoff point for algorithm
c     lnsqrtpi = ln(sqrt(pi))
c     isqrtpi = 1/sqrt(pi)
      parameter(bigx=20.d0,lnsqrtpi=0.5723649429247000870717135d0,
     *   isqrtpi=0.5641895835477562869480795d0)
      integer df
      logical even
 
      if(x.le.0.d0.or.df.le.0)then
         pchi2=1.d0
         goto 999
      endif
 
      a = .5d0*x
      even = (mod(df,2).eq.0)
      if(df.gt.1)y=dexp(-a)
      if(even)then
         s=y
      else
         s=2.d0*pnorm1(-dsqrt(x))
      endif
 
      if(df.gt.2)then
         xx=.5d0*dble(df-1)
         if(even)then
            z=1.d0
         else
            z=.5d0
         endif
         if(a.gt.bigx)then
            if(even)then
               e=0.d0
            else
               e=lnsqrtpi
            endif
            c=dlog(a)
  100       e=dlog(z)+e
            s=s+dexp(c*z-a-e)
            z=z+1.d0
            if(z.le.xx)goto 100
            pchi2=s
         else
            if(even)then
               e=1.d0
            else
               e=isqrtpi/dsqrt(a)
            endif
            c=0.d0
  200       e=e*a/z
            c=c+e
            z=z+1.d0
            if(z.le.xx)goto 200
            pchi2=c*y+s
         endif
      else
         pchi2=s
      endif
 
  999 return
      end
