      subroutine prep1(task,nfullplus,neh,nihmax,numdist,dimlam,distdim,
     *   lampos1,nmapmoel,dimv,rsh3max,rsh4max,rsh5max,fullbeta,
     *   dfullbeta,gamma,xbh3,xbh4,xbh5,lam,dlam,s,ds,ssg,d1gamma,
     *   d2gamma,lnla,lnlb,d1xbh3,d2xbh3,dlnla,dlnlb,v,vpntr,xnode,
     *   mapmoel,moelpos,refspl,beginspl,nodepos,numnodes,spltype,
     *   ifdrv,ipt,jpt)
 
c     Task=1:
c     Subroutine to prepare hazard spells in ithobs: compute
c     likelihoods and derivatives for all hazard spells, for the
c     median heterogeneity group, i.e., as if there was no mixture.
c     Array lnla will contain the likelihood for the censored part
c     of each spell; lnlb for the term that is attached in case
c     of a non-censored spell.  I.e.,
c               lnlb=lnla * (S(ni+1)/S(ni))**G(ni)
c     The likelihood for each spell is then lnla if the spell
c     is censored and lnla-lnlb if it is non-censored.  Dlnla
c     and dlnlb are matrices with derivatives of lnla and lnlb
c     wrt individual parameters.
c
c     Task=2: unravel derivatives wrt Lambda into wrt fullbeta.  Note
c        that only wrt Lambda is dealt with!
c
c     Constantijn Panis, 6 July 1999
c     Last change: Introduce nl/nu.
 
      implicit none
      integer task,nfullplus,neh,nihmax,numdist,dimlam,distdim(numdist),
     *   lampos1(numdist),dimv,nmapmoel,
     *   rsh3max,rsh4max,rsh5max,ipt,jpt
      double precision fullbeta(nfullplus),dfullbeta(nfullplus),
     *   gamma(*),lnla(neh),lnlb(neh),lam(dimlam),dlam(dimlam),
     *   xbh3(nihmax,0:rsh3max),xbh4(rsh4max,neh),xbh5(rsh5max),
     *   d1xbh3(nihmax),d2xbh3(nihmax),s(0:nihmax),ds(0:nihmax,*),
     *   ssg(nihmax),d1gamma(*),d2gamma(*),dlnla(nfullplus,neh),
     *   dlnlb(nfullplus,neh)
      real v(dimv),xnode(*)
      integer mapmoel(nmapmoel),refspl(*),beginspl(*),nodepos(*),
     *   numnodes(*),spltype(*),nsb,nl,nu,censor,nds,moelpos,
     *   vpntr,vpntr1,timepos,nngam,ie,pos1,pos2
      logical ifdrv
 
      vpntr1=vpntr
      if(task.eq.2)goto 200
 
c     Prepare hazard models, before adding mixture.
c     First 3*neh positions are for censor, nsb, nl, and nu:
  100 vpntr=vpntr+4*neh
      do 110 ie=1,neh
         if(v(vpntr).ne.real(ie))call bug(ipt,jpt)
         censor=int(v(vpntr1+4*ie-4))
         nsb=int(v(vpntr1+4*ie-3))
         nl=int(v(vpntr1+4*ie-2))
         nu=int(v(vpntr1+4*ie-1))
 
         vpntr=vpntr+1
         timepos=vpntr
         vpntr=vpntr+nu+1
 
c        Loop over all model elements in this spell.  Place
c        "regular" regressor sets in xbh3(.,0), regressor sets
c        interacted with slopes in gamma(.), and regressor sets
c        that are interacted with residual components in Lambda.
         pos1=moelpos
         pos2=vpntr
         call mapmodel1(1,neh,nsb,nfullplus,numdist,dimlam,distdim,
     *      lampos1,dimv,mapmoel,nds,nngam,rsh3max,rsh4max,
     *      rsh5max,xbh3,xbh4(1,ie),xbh5,0.d0,0.d0,fullbeta,0.d0,0.d0,
     *      gamma,0.d0,0.d0,lam(ie),0.d0,0.d0,v,0,moelpos,vpntr,
     *      refspl,beginspl,numnodes,spltype,ipt,jpt)
 
c        Compute the likelihood and derivatives without mixture.
         call simhaz(nngam,nds,censor,nsb,nl,nu,refspl,nodepos,
     *      numnodes,spltype,lnla(ie),lnlb(ie),gamma,d1gamma,
     *      d2gamma,d1xbh3,d2xbh3,xbh3,s,ds,ssg,v(timepos),
     *      v(vpntr),xnode,ifdrv,ipt,jpt)
 
c        Skip nds duration spline origins:
         vpntr=vpntr+nds
 
         if(.not.ifdrv)goto 110
c        Map derivatives wrt gamma and xb into wrt fullbeta.
c        (Note usage of pos1/pos2, rather than moelpos, vpntr.)
         call mapmodel1(2,neh,nsb,nfullplus,numdist,dimlam,distdim,
     *      lampos1,dimv,mapmoel,nds,nngam,rsh3max,rsh4max,
     *      rsh5max,xbh3,0.d0,xbh5,d1xbh3,d2xbh3,fullbeta,dlnla(1,ie),
     *      dlnlb(1,ie),gamma,d1gamma,d2gamma,0.d0,0.d0,0.d0,v,
     *      censor,pos1,pos2,refspl,beginspl,numnodes,spltype,ipt,jpt)
 
  110 continue
      goto 999
 
c     Unravel wrt Lambda into fullbeta:
  200 vpntr=vpntr+4*neh
      do 210 ie=1,neh
         nsb=int(v(vpntr1+4*ie-3))
         nl=int(v(vpntr1+4*ie-2))
         nu=int(v(vpntr1+4*ie-1))
         vpntr=vpntr+nu+2
         call mapmodel1(3,neh,nsb,nfullplus,numdist,dimlam,distdim,
     *      lampos1,dimv,mapmoel,nds,nngam,rsh3max,rsh4max,
     *      rsh5max,0.d0,xbh4(1,ie),0.d0,0.d0,0.d0,fullbeta,dlnla,
     *      dlnlb,gamma,d1gamma,d2gamma,lam(ie),dlam(ie),dfullbeta,
     *      v,0,moelpos,vpntr,refspl,beginspl,numnodes,spltype,ipt,jpt)
         vpntr=vpntr+nds
  210 continue
      goto 999
 
  999 return
      end
