      subroutine prep5po(task,nfullplus,nmapmoel,nepo,dimp,numdist,
     *   dimlam,distdim,lamtyp,lampos,dimv,mapmoel,moelpos,
     *   rsnpmax,fullbeta,dfullbeta,p5,dp5,lam,dlam,v,vpntr,ipt,jpt)
 
c     Subroutine to prepare and unravel Poisson modules in ithobs.
c     Task=1: compute and accumulate locations of incidence rate,
c             log(exposure), log(outcome!).  Set up lam for
c             probability integrated residual.
c     Task 2: unravel everything into wrt fullbeta.
c
c     Conditional likelihood of one outcome is:
c
c              exp(-lambda) * lambda^y
c     P(Y=y) = ------------------------
c                      y!
c
c     ln(P(Y=y)) = -lambda + y*ln(lambda) - ln(y!)
c
c     Incidence rate lambda is always an exponential:
c        lambda = exp( ln(exposure) + Beta'X + eps)
c     where eps can be any (integrated) residua structure.
c     The conditional log-likelihood thus is:
c
c     ln(P(Y=y)) = -lambda + y*(ln(exposure)+Beta'X+eps) - ln(y!)
c                = y*ln(exposure) + y*Beta'X - ln(y!) - lambda + y*eps
c
c     This routine _could_ compute and accumulate the first three terms,
c     leaving the last two terms for onepoisson.  However, it is more
c     efficient to only accumulate -ln(y!) here and to leave the rest
c     (-lambda + y*ln(lambda)) to onepoisson.  Even that is not done
c     because -ln(y!) can be large negative and lead the remainder to
c     be large positive, causing overflow problems with there is
c     heterogeneity, so that dexp(lnlg) is computed.
c
c     In short, this routine only computes the incidence location (p5)
c     and the integrated residual load matrix.
c
c     Constantijn Panis, 12 November 2002
 
      implicit none
      integer task,nfullplus,nmapmoel,nepo,dimp,numdist,dimlam,
     *   distdim(numdist),lamtyp(numdist),lampos(numdist),dimv,
     *   mapmoel(nmapmoel),rsnpmax,ipt,jpt,moelpos,vpntr,ie
      double precision fullbeta(nfullplus),dfullbeta(nfullplus),
     *   p5(dimp,0:rsnpmax),dp5(dimp),lam(dimlam),dlam(dimlam)
      real v(dimv)
 
c     The first positions in v are for:
c        nepo Poisson count outcomes
c        nepo Poisson exposure values
      vpntr=vpntr+2*nepo
 
      if(task.eq.2)goto 200
 
c     Compute and accumulate -ln(y!),
c     set up regressor sets and load matrices for the probability.
  100 do 110 ie=1,nepo
         if(v(vpntr).ne.real(ie))call bug(ipt,jpt)
         vpntr=vpntr+1
c        Compute p5 (probability location) and the load matrix on
c        integrated residuals.
         call prepfunc(1,ie,dimp,nepo,nmapmoel,dimv,mapmoel,moelpos,
     *      vpntr,nfullplus,rsnpmax,numdist,distdim,dimlam,lamtyp,
     *      lampos,fullbeta,dfullbeta,p5,dp5,lam,dlam,0.d0,v,ipt,jpt)
  110 continue
      goto 999
 
c     Unravel everything into wrt fullbeta.
  200 do 210 ie=1,nepo
         if(v(vpntr).ne.real(ie))call bug(ipt,jpt)
         vpntr=vpntr+1
c        Unravel wrt the probability location and the load
c        matrix for integrated residuals in the probability:
         call prepfunc(2,ie,dimp,nepo,nmapmoel,dimv,mapmoel,moelpos,
     *      vpntr,nfullplus,rsnpmax,numdist,distdim,dimlam,lamtyp,
     *      lampos,fullbeta,dfullbeta,p5,dp5,lam,dlam,
     *      0.d0,v,ipt,jpt)
  210 continue
      goto 999
 
  999 return
      end
 
