      subroutine prepfunc(task,ie,nrowxb,nrowlam,dimmm,dimv,mapmoel,
     *   moelpos,vpntr,nfullplus,rsmax,numdist,distdim,dimlam,lamtyp,
     *   lampos,fullbeta,dfullbeta,xb,dxb,lam,dlam,stda4,v,ipt,jpt)
 
c     Subroutine to (task=1) prepare pieces of a function evaluation and
c     (task=2) unravel derivatives wrt those pieces into wrt original
c     parameters.  Specifically:
c        (1) compute (sums of) regressor sets (interacted with other
c            regressor sets or parameters).
c        (2) compute load matrices on all distributions.
c
c     Arguments:
c        task [in] 1=prepare function;
c                  2=unravel derivatives
c        ie   [in] outcome number (1<=ie<=nrowlam)
c        nrowxb [in] number of rows of xb
c        nrowlam [in] number of rows of lam
c        Note:  the reason for distinguishing nrowxb and nrowlam is
c               in probit/logit mixtures.  XB is used for both logits
c               and probits, but they do not share lambda.
c        rsmax [in] maximum number of interactions in the regressor set
c              specifications.
c        xb [out] (sums of) regressor sets (interacted with other
c              regressor sets or parameters).  The zero-th column
c              contains end result; column 1 through rsmax contain
c              individual pieces, yet to be added or interacted.  These
c              include pieces that are interacted with residuals.
c        dxb [in] derivatives wrt xb (stored in zero-th column; column 1
c              is scratch space.)
c        lam [out] load matrices on all distributions.
c        nfullplus [in] number of model parameters plus space for
c              inverse matrices
c        fullbeta [in] model parameters.
c        v [in] data vector.
c        vpntr [in] position in v of first relevant data value.
c        mapmoel [in] integer vector with coded function elements
c
c     This routine is used for probit, logit, continuous, and (negative)
c     binomial models.  Hazard models use mapmodel1.
c
c     Constantijn Panis, 6 September 2002
c     Last change: variables allowed
 
      implicit none
      integer task,ie,nrowxb,nrowlam,dimmm,dimv,ipt,jpt,mapmoel(dimmm),
     *   moelpos,vpntr,nfullplus,rsmax,numdist,distdim(numdist),
     *   dimlam,lamtyp(numdist),lampos(numdist),
     *   termtype,numterm,firstrs,rs,draw,pos,nx,col,dist,dim
      double precision fullbeta(nfullplus),xb(nrowxb,0:rsmax),
     *   lam(dimlam),dlam(dimlam),stda4,dfullbeta(nfullplus),
     *   dxb(nrowxb,0:1),temp,sign,dprowprod,innerp84
      real v(dimv)
 
      if(task.eq.2)goto 200
 
c     First initialize big XB to zero:
  100 xb(ie,0)=0.d0
      termtype=0
      rs=0
  120 if(mapmoel(moelpos).lt.0)then
c        End of this term, beginning of new term:
         if(termtype.eq.-3)then
c           Previous terms were regressor set(s): add to grand XB:
            if(numterm.eq.1)then
               xb(ie,0)=xb(ie,0)+sign*xb(ie,firstrs)
            else
               temp=dprowprod(xb(ie,firstrs),nrowxb,numterm)
               xb(ie,0)=xb(ie,0)+sign*temp
            endif
         endif
         if(mapmoel(moelpos).eq.-99)then
c           end of model for this module
            moelpos=moelpos+1
            goto 999
         endif
c        Next is a regressor set: termtype=-3
c            integrated residual: termtype=-4
c        non-integrated residual: termtype=-7
         termtype=mapmoel(moelpos)
         numterm=mapmoel(moelpos+1)
         moelpos=moelpos+2
         sign=1.d0
         if(numterm.lt.0)then
            sign=-1.d0
            numterm=-numterm
         endif
         if(termtype.eq.-4.or.termtype.eq.-7)numterm=numterm-1
c        Firstrs is the first column that is relevant for the current
c        model element.
         firstrs=rs+1
         goto 120
      elseif(mapmoel(moelpos).eq.2)then
c        Parameter: place in xb as if it were a regressor set:
         pos=mapmoel(moelpos+1)
         moelpos=moelpos+2
         rs=rs+1
         xb(ie,rs)=fullbeta(pos)
      elseif(mapmoel(moelpos).eq.3)then
c        Regressor set: place in xb:
         pos=mapmoel(moelpos+1)
         nx=mapmoel(moelpos+2)
         moelpos=moelpos+3
         rs=rs+1
         xb(ie,rs)=innerp84(fullbeta(pos),v(vpntr),nx)
         vpntr=vpntr+nx
      elseif(mapmoel(moelpos).eq.4.or.mapmoel(moelpos).eq.7)then
c        Integrated or non-integrated residual; fill out load matrix Lambda (a
c        function of ones, parameters, and regressor sets).  Each load matrix
c        Lambda has dimension (nrowlam,bigdim), where bigdim=distdim*ndraw
c        is the dimension of the expanded distribution.
c        Suppose dim=2, both residuals are drawn at level 3, this observation
c        has 2 level 3 branches with 2 and 4 level 4 branches (outcomes).  Only
c        second residual enters this modeltype in this observation:
c           Lambda=( 0 1   0 0 )
c                  ( 0 1   0 0 )
c                  ( 0 0   0 1 )
c                  ( 0 0   0 1 )
c                  ( 0 0   0 1 )
c                  ( 0 0   0 1 )
c
c        The above matrix is sparse, i.e., has at most one nonzero element
c        per row.  This is flagged by lamtyp=4.  In such case, only store the
c        relevant column and the element in consecutive positions of lam().  Spa
c        was determined in figure_res.  However, this determination is not fool-
c        proof.
         dist=mapmoel(moelpos+1)
         if(dist.eq.9999)then
c           Special case, only possible as non-integrated residual
c           in probit models: implicit IID N(0,1).
            stda4=1.d0
            moelpos=moelpos+2
            goto 120
         endif
         dim=mapmoel(moelpos+2)
         draw=mapmoel(moelpos+3)
         moelpos=moelpos+4
c        Cut out if Lambda is identity matrix:
         if(lamtyp(dist).eq.1)goto 120
         col=distdim(dist)*(draw-1)+dim
         if(numterm.eq.0)then
c           Plain residual, not interacted:
            temp=1.d0
         elseif(numterm.eq.1)then
c           Residual is multiplied by one term:
            temp=xb(ie,firstrs)
         else
c           Residual is multiplied by multiple terms:
            temp=dprowprod(xb(ie,firstrs),nrowxb,numterm)
         endif
         if(lamtyp(dist).eq.0)then
c           Generic matrix
            pos=lampos(dist)+nrowlam*(col-1)+ie
            lam(pos)=lam(pos)+sign*temp
         elseif(lamtyp(dist).eq.4)then
c           Matrix is probably sparse.  Store the column
c           and only the possibly-nonzero element in lam.
            pos=lampos(dist)+ie
c           Detect if matrix is in fact not sparse:
            if(lam(pos).ne.0.d0)goto 800
            lam(pos)=dble(col)
            lam(pos+nrowlam)=lam(pos+nrowlam)+sign*temp
         else
            call bug(ipt,jpt)
         endif
      elseif(mapmoel(moelpos).eq.9)then
c        Variable: place in xb as if it were a regressor set:
         moelpos=moelpos+1
         rs=rs+1
         xb(ie,rs)=dble(v(vpntr))
         vpntr=vpntr+1
      else
         call bug(ipt,jpt)
      endif
      goto 120
 
c     Map derivatives wrt XB and Lambda back into wrt parameters:
  200 termtype=0
      rs=0
  220 if(mapmoel(moelpos).lt.0)then
c        End of this term, beginning of new term:
         if(mapmoel(moelpos).eq.-99)then
c           end of model for this module
            moelpos=moelpos+1
            goto 999
         endif
c        Next is a regressor set: termtype=-3
c            integrated residual: termtype=-4
c        non-integrated residual: termtype=-7
         termtype=mapmoel(moelpos)
         numterm=mapmoel(moelpos+1)
         sign=1.d0
         if(numterm.lt.0)then
            sign=-1.d0
            numterm=-numterm
         endif
         if(termtype.eq.-4.or.termtype.eq.-7)numterm=numterm-1
         moelpos=moelpos+2
         if(termtype.eq.-3)then
c           Regressor set(s): copy dxb into dxb(ie,1)
            firstrs=rs+1
            dxb(ie,1)=sign*dxb(ie,0)
         elseif(termtype.eq.-4.or.termtype.eq.-7)then
            if(numterm.eq.0)goto 220
c           Non-integrated residual: place derivatives wrt lam
c           in wrt xb, if any.  Figure out which dimension applies:
            firstrs=rs+1
            pos=moelpos
  260       if(mapmoel(pos).eq.2)then
               pos=pos+2
               goto 260
            elseif(mapmoel(pos).eq.3)then
               pos=pos+3
               goto 260
            elseif(mapmoel(pos).eq.4.or.mapmoel(pos).eq.7)then
               dist=mapmoel(pos+1)
c              implicit IID(0,1) residuals may not be loaded:
               if(dist.eq.9999)call bug(ipt,jpt)
               dim=mapmoel(pos+2)
               draw=mapmoel(pos+3)
               if(lamtyp(dist).eq.0)then
c                 Generic
                  col=distdim(dist)*(draw-1)+dim
                  pos=lampos(dist)+nrowlam*(col-1)+ie
                  dxb(ie,1)=sign*dlam(pos)
               elseif(lamtyp(dist).eq.4)then
c                 Sparse
                  col=distdim(dist)*(draw-1)+dim
                  pos=lampos(dist)+ie
                  if(int(dlam(pos)).ne.col)call bug(ipt,jpt)
                  pos=pos+nrowlam
                  dxb(ie,1)=sign*dlam(pos)
               elseif(lamtyp(dist).eq.1)then
c                 Identity
                  call bug(ipt,jpt)
               endif
            elseif(mapmoel(pos).eq.9)then
               pos=pos+1
               goto 260
            else
               call bug(ipt,jpt)
            endif
         endif
         goto 220
      elseif(mapmoel(moelpos).eq.2)then
c        Parameter: unravel dxb
         pos=mapmoel(moelpos+1)
         moelpos=moelpos+2
         rs=rs+1
         call compxb(4,numterm,rs-firstrs+1,0,1,nrowxb,0.d0,0.d0,
     *      0.d0,xb(ie,firstrs),0.d0,dfullbeta(pos),dxb(ie,1),0.)
      elseif(mapmoel(moelpos).eq.3)then
c        Regressor set
         pos=mapmoel(moelpos+1)
         nx=mapmoel(moelpos+2)
         moelpos=moelpos+3
         rs=rs+1
         call compxb(2,numterm,rs-firstrs+1,nx,1,nrowxb,0.d0,0.d0,
     *      0.d0,xb(ie,firstrs),dfullbeta(pos),0.d0,dxb(ie,1),v(vpntr))
         vpntr=vpntr+nx
      elseif(mapmoel(moelpos).eq.4.or.mapmoel(moelpos).eq.7)then
c        Integrated residual: handled through map_u
c        Non-integrated residual: handled through ithobs.
         if(mapmoel(moelpos+1).eq.9999)then
c           IID N(0,1)
            moelpos=moelpos+2
         else
            moelpos=moelpos+4
         endif
      elseif(mapmoel(moelpos).eq.9)then
c        Variable (no derivatives involved)
         moelpos=moelpos+1
         rs=rs+1
         vpntr=vpntr+1
      else
         call bug(ipt,jpt)
      endif
      goto 220
 
  800 if(ipt.gt.0)write(8,801)
      if(jpt.gt.0)write(6,801)
  801 format(/,'ERROR.  aML always tries to gain efficiency by ',
     *   'recognizing special cases',/,'of model specifications.  In ',
     *   'your current model, it was overly ambitious',/,'and ',
     *   'incorrectly determined that there was only one residual of ',
     *   'a certain',/,'distribution per outcome.  Please specify ',
     *   '"option sparsity=no" in your',/,'control file, so that aML ',
     *   'does not attempt to gain efficiency, and run',/,'your model ',
     *   'again.')
      stop
  999 return
      end
