      subroutine quadratic(task,n1,n2,a,b,c,factor)
 
c     Subroutine to compute a quadratic form:
c
c     task = 0: compute A*B*A' = C    A(n1,n2)   B(n2,n2)   C(n1,n1)
c         else: compute A'*B*A = C    A(n2,n1)   B(n2,n2)   C(n1,n1)
c
c     It is assumed that B is symmetric.  If this is not the case,
c     instead use m3mult(10,n1,n2,n2,n1,a,b,a,c,factor).
c
c     Integer factor determines:
c        factor = 0     C = A*B*A'
c              else     C = C + factor*A*B*A'
c
c     If factor!=0, it is assumed that upon input, C is already
c     symmetric, so that upon output, C is still symmetric.  This
c     routine forces symmetry.
c
c     Note some rules for differentiation.   If C = A*B*A', then
c        dA = dC*A*B' + dC'*A*B = 2*dC*A*B if dC and B are symmetric.
c        dB = A'*dC*A
c
c     This routine is very obscure.  See m3mult (which computes the
c     product of three arbitrary matrices) for details.  The current
c     routine is a special case of m3mult, taking advantage of the
c     fact that C is symmetric.
c
c     Constantijn Panis, 4 March 2002
c     Last change: factor instead of add.
 
      implicit none
      integer task,n1,n2,factor,i,j,k,l,ia,ib,ic,id,ja,jb,jc,jd,
     *   sa,sb,sc,sd,jsa,jsb,jsc,jsd
      double precision a(n1*n2),b(n2*n2),c(n1*n1),abij
 
      if(task.eq.0)then
         ja=n1
         jsa=1
         jc=1
         jsc=n1
      else
         ja=1
         jsa=n2
         jc=n2
         jsc=1
      endif
      jb=1
      jsb=n2
      jd=n1
      jsd=1
 
      if(factor.eq.0)call zerodp(n1*n1,c)
 
      sa=1
      sd=1
      do 110 i=1,n1
         sb=1
         sc=1
         do 120 j=1,n2
            abij = 0.d0
            ia=sa
            ib=sb
            ic=sc
            id=sd
            do 130 k=1,n2
               abij = abij + a(ia)*b(ib)
               ia=ia+ja
               ib=ib+jb
  130       continue
            if(factor.ne.0.and.factor.ne.1)abij=dble(factor)*abij
            do 140 l=1,i
               c(id) = c(id) + abij*a(ic)
               id=id+jd
               ic=ic+jc
  140       continue
            sb=sb+jsb
            sc=sc+jsc
  120    continue
         sa=sa+jsa
         sd=sd+jsd
  110 continue
      call reflect(c,n1)
 
      return
      end
