      subroutine read_model(levelmax,struc,mdllvl,nfull,
     *   numrset,numspl,numdist,numpar,distdim,maxmeno,moel,meno,
     *   maxxnode,xnodepos,numfreq,maxnumfreq,freqloc,nold,prof,
     *   nxold,namepntr,parpos,pardim,modeltype,resdone,intresdone,
     *   dursplinedone,xnode,rsetname,splname,parname,oldname,resname,
     *   lhs,mname,endstring,line,il,ipt,jpt)
 
c     Subroutine to read in a model specification.  The following
c     elements are supported:
c        PAR
c        REGSET
c        REGSPLINE
c        INTRES
c        DURSPLINE
c        RES
c        varname
c
c     Upon entry, something like MODEL= is assumed to have been read.
c
c     Model elements are stored in moel();
c        0 = end of model specification
c       -3 = the main model element that follows is a regressor set
c            (including regressor spline, variable)
c       -4 = the main model element that follows is an integrated residual
c       -5 = the main model element that follows is a duration spline
c       -7 = the main model element that follows is a residual
c        2 = next factor is a parameter
c        3 = next factor is a regressor set
c        4 = next factor is an integrated residual
c        5 = next factor is a duration spline
c        7 = next factor is a non-integrated residual
c        8 = next factor is a regressor spline
c        9 = next factor is a variable
c
c     resdone [in/out]
c                -1  no non-integrated residual allowed
c                 0  no non-integrated residual specified yet
c                 1  one or more explicit non-integrated residuals specified
c                 2  implicit IID N(0,1) was explicitly specified
c                99  non-integrated residuals OK; do not change
c                    this value (saves variables)
c     intresdone [in/out]  Same as resdone for integrated residuals,
c                    except value 2 not applicable
c     dursplinedone [in/out] Same as resdone for duration splines,
c                    except value 2 not applicable
c
c     Constantijn Panis, 2 December 2002
c     Last change: error message
 
      implicit none
      integer levelmax,struc,mdllvl,nfull,numrset,numspl,
     *   numdist,numpar,distdim(numdist),maxmeno,moel(maxmeno),meno,
     *   maxxnode,xnodepos,numfreq,maxnumfreq,freqloc(maxnumfreq),nold,
     *   prof,nxold(levelmax),namepntr(levelmax),parpos(numpar),
     *   pardim(numpar),modeltype,termtype,numterm,firstmeno,resdone,
     *   intresdone,dursplinedone,il,ipt,jpt,sign,level,varpos,i1,i2,
     *   status,unit,col,trm
      real xnode(maxxnode)
      logical smtxt
      character line*128,rsetname(numrset)*12,splname(numspl)*12,
     *   parname(numpar)*12,oldname(nold)*8,resname(*)*12,
     *   lhs*(*),mname*(*),endstring*(*),string*60,idstring*60,tmp*32
 
      unit=0
      if(mdllvl.eq.0)goto 820
      termtype=0
      numterm=0
      sign=1
      firstmeno=meno+1
      meno=meno+2
  310 if(smtxt(4,line,il,'+'))then
         if(smtxt(4,line,il,'-'))then
c           Allow +-:
            il=il-1
            goto 310
         endif
         if(termtype.eq.0)then
c           First model element.  "+" is inconsequential.
            continue
         else
c           Record what the previous term was:
            moel(firstmeno)=termtype
            moel(firstmeno+1)=sign*numterm
            termtype=0
            numterm=0
            sign=1
            firstmeno=meno+1
            meno=meno+2
         endif
      elseif(smtxt(4,line,il,'-'))then
         if(termtype.eq.0)then
c           First model element:
            sign=-1
         else
c           Record what the previous term was:
            moel(firstmeno)=termtype
            moel(firstmeno+1)=sign*numterm
            termtype=0
            numterm=0
            sign=-1
            firstmeno=meno+1
            meno=meno+2
         endif
      elseif(smtxt(4,line,il,'*'))then
         if(termtype.eq.0)goto 800
c        Part of an interaction:
         if(numterm.eq.0)goto 800
         if(termtype.eq.-4.or.termtype.eq.-5.or.termtype.eq.-7)goto 800
         numterm=-numterm
         if(smtxt(4,line,il,'-'))sign=-sign
      elseif(smtxt(4,line,il,endstring))then
c        Record what the previous term was:
         moel(firstmeno)=termtype
         moel(firstmeno+1)=sign*numterm
c        Finish off with a -99:
         meno=meno+1
         moel(meno)=-99
         goto 999
      else
c        Only allowed for first model element:
         if(termtype.ne.0)goto 800
      endif
 
      if(smtxt(4,line,il,'PARameter'))goto 330
      if(smtxt(4,line,il,'REGressor SET'))goto 340
      if(smtxt(4,line,il,'REGressor SPLINE'))goto 350
      if(smtxt(4,line,il,'INTegrated RESidual'))goto 360
      if(smtxt(4,line,il,'RESidual'))goto 370
      if(smtxt(4,line,il,'DURation SPLINE'))goto 380
c     Variable name?
      goto 400
 
c     PARAMETER
  330 call read_par(levelmax,nfull,struc,maxmeno,moel,meno,numpar,
     *   pardim,parpos,nold,nxold,namepntr,maxxnode,xnodepos,mdllvl,
     *   numfreq,maxnumfreq,freqloc,xnode,parname,oldname,
     *   line,il,ipt,jpt)
      termtype=-3
      numterm=-numterm+1
      goto 310
 
c     REGRESSOR SET
  340 call read_rset(levelmax,struc,maxmeno,moel,meno,numrset,nxold,
     *   namepntr,maxxnode,xnodepos,nold,mdllvl,numfreq,maxnumfreq,
     *   freqloc,xnode,rsetname,oldname,line,il,ipt,jpt)
      termtype=-3
      numterm=-numterm+1
      goto 310
 
c     REGRESSOR SPLINE
  350 call read_rspl(levelmax,numspl,maxmeno,moel,meno,nxold,namepntr,
     *   nold,maxxnode,xnodepos,mdllvl,maxnumfreq,numfreq,freqloc,
     *   struc,xnode,oldname,splname,line,il,ipt,jpt)
      termtype=-3
      numterm=-numterm+1
      goto 310
 
c     INTEGRATED RESIDUAL
  360 call read_res(levelmax,maxmeno,meno,moel,numdist,distdim,
     *   struc,nxold,namepntr,maxxnode,xnodepos,nold,mdllvl,
     *   modeltype,numterm,maxnumfreq,numfreq,freqloc,xnode,
     *   resdone,intresdone,prof,oldname,resname,1,line,il,ipt,jpt)
      termtype=-4
      numterm=-numterm+1
      goto 310
 
c     RESIDUAL
  370 call read_res(levelmax,maxmeno,meno,moel,numdist,distdim,
     *   struc,nxold,namepntr,maxxnode,xnodepos,nold,mdllvl,
     *   modeltype,numterm,maxnumfreq,numfreq,freqloc,xnode,
     *   resdone,intresdone,prof,oldname,resname,0,line,il,ipt,jpt)
      termtype=-7
      numterm=-numterm+1
      goto 310
 
c     DURATION SPLINE
  380 call read_dspl(levelmax,mdllvl,numspl,maxmeno,moel,meno,nxold,
     *   namepntr,nold,xnodepos,maxxnode,maxnumfreq,numfreq,freqloc,
     *   struc,xnode,dursplinedone,oldname,splname,line,il,ipt,jpt)
      termtype=-5
      numterm=-numterm+1
      goto 310
 
c     Variable name
  400 call getname(4,line,il,i1,i2)
      if(i2.lt.i1)goto 800
      if(i2-i1.gt.7)then
         il=i1
         goto 800
      endif
      call findvar(levelmax,mdllvl,level,line(i1:i2),oldname,
     *   namepntr,nxold,varpos,line,il,struc,0,0,status)
c     status 0 = no error
c            1 = variable not found (user may not have intended a variable)
c            2 = variable found but not at the mandatory -mdllvl
c            3 = variable found, at level=mdllvl+1
c            4 = variable found, at level>mdllvl+1
      if(status.ne.0)then
         if(status.eq.1)goto 800
         call findvar(levelmax,mdllvl,level,line(i1:i2),oldname,
     *      namepntr,nxold,varpos,line,il,struc,ipt,jpt,status)
         stop
      endif
      varpos=level*(nold+1)+varpos
c     This is a variable:
      meno=meno+1
      moel(meno)=9
      meno=meno+1
      moel(meno)=varpos
      call addvarfreq(maxnumfreq,numfreq,freqloc,varpos)
      termtype=-3
      numterm=-numterm+1
      goto 310
 
  800 tmp=mname
      if(ipt.eq.0.and.jpt.eq.0)then
         stop
      elseif(unit.eq.0.and.ipt.gt.0)then
         unit=8
      else
         unit=6
      endif
      call caretline(unit,line,il)
      write(unit,801)tmp(1:trm(tmp)),lhs,endstring
  801 format('ERROR encountered while reading a ',a,' model',/,
     *   'specification.  The syntax is:',//,
     *   3x,a,'building_block [{+|-} building_block] [...]',a,/)
      col=1
      call wrapstring(unit,col,72,1,0,'where "building_block" may be '//
     *   'a regressor set (REGSET), parameter (PAR), data variable,')
      if(resdone.ne.-1)call wrapstring(unit,il,72,1,0,
     *   ' residual (RES),')
      if(intresdone.ne.-1)call wrapstring(unit,col,72,1,0,
     *   ' integrated residual (INTRES),')
      if(dursplinedone.ne.-1)call wrapstring(unit,col,72,1,0,
     *   ' duration spline (DURSPLINE),')
      call wrapstring(unit,col,72,0,1,
     *   ' or regressor spline (REGSPLINE).')
      write(unit,802)
  802 format(/,'Many building blocks may be interacted with others.  ',
     *   'For example:',/,3x,'PAR ... * REGSET ...',/,3x,
     *   'varname * REGSET ...')
      if(termtype.eq.-4)tmp='integrated residual'
      if(termtype.eq.-5)tmp='duration spline'
      if(termtype.eq.-7)tmp='residual'
      if(termtype.eq.-4.or.termtype.eq.-5.or.termtype.eq.-7)call
     *   wrapstring(unit,0,72,0,1,'In such interactions, '//
     *   tmp(1:trm(tmp))//'s must be listed last.  The '//
     *   'problem with your specification is probably that it '//
     *   'listed a '//tmp(1:trm(tmp))//' before another '//
     *   'building block, or that it interacted blocks that may '//
     *   'not be interacted, such as two '//tmp(1:trm(tmp))//'s.')
      if(jpt.eq.0.or.unit.eq.6)stop
      goto 800
  820 tmp=mname
      string=idstring(struc,0,0,0,0)
      if(ipt.gt.0)then
         call caretline(8,line,il)
         call wrapstring(8,0,72,0,1,'ERROR:  the outcome must '//
     *      'precede a model''s right-hand-side specification.  '//
     *      'This is violated in a '//tmp(1:trm(tmp))//' model'//
     *      string(1:trm(string)))
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         call wrapstring(6,0,72,0,1,'ERROR:  the outcome must '//
     *      'precede a model''s right-hand-side specification.  '//
     *      'This is violated in a '//tmp(1:trm(tmp))//' model'//
     *      string(1:trm(string)))
      endif
      stop
  999 return
      end
 
