      subroutine read_model1(levelmax,nfull,maxmodel,mdllvl,
     *   datadim,strctre,datas,datastructure,maxxnode,xnodepos,meno,
     *   maxmeno,moel,moelpntr,numrset,numpar,numspl,nummodel,numdist,
     *   distdim,numfreq,maxnumfreq,nfreq,freqloc,nxold,namepntr,nold,
     *   prof,modeltype,minlnl,pardim,parpos,xnode,rsetname,parname,
     *   splname,resname,oldname,line,il,ipt,jpt)
 
c     Subroutine to read the hazard model specification for a certain
c     data structure.  The syntax is:
c     HAZARD MODEL; [DATA STRUCTURE = n;]
c        [(KEEP | DROP IF ....)];
c        [NUMERATOR;]  [DENOMINATOR;]
c        CENSOR = varname;  DURATION = [durvar1 durvar2];
c        [TIMEMARKS = varname;]
c        MODEL = DURSPLINE ... + REGSET ...  +
c           REGSPLINE ... + INTRES ...
c           ;
c
c     The censor indicator variable indicates whether the spell is
c     censored. The duration of the spell is given by two duration
c     variables which delimit the event window. For censored spells,
c     durvar1=durvar2. Censor and duration variables must be at the
c     same level. If there are time-varying covariates, the TIMEMARKS
c     variable (at one level down) indicates the time points between
c     which the covariates are constant. If duration variables are not
c     specified, the event is assumed to happen between the last two
c     timemarks. Even when duration variables are not specified, a
c     DURATION=; statement must be there.
c
c     By default the likelihood function contains the modules related to
c     any model in the numerator.  If DENOMINATOR; is specified, the
c     module enters in the denominator only; if both NUMERATOR; and
c     DENOMINATOR; are specified, the module is replicated and enters
c     both ways.  The latter is useful in models with initial conditions
c     problems: the probability of making it into the sample does not
c     cancel out of the likelihood function if there is heterogeneity.
c     The default translates into minlnl=0; denominator only minlnl=1;
c     numerator and denominator minlnl=2.
c
c     Regressor splines are practically the same as regressor sets as
c     far as the model specification and ithobs go.  At this stage
c     they are still distinguished from regressor sets.
c
c     Regressor sets, splines and integrated residual can be either
c     directly or indirectly referenced.  Direct referencing is done
c     by specifying a (regressor set, spline or integrated residual)
c     name; indirect referencing is done by specifying REFVAR=varname,
c     where varname is the name of an integer-valued variable (at model
c     level or higher).  Varname may also be a linear combination of
c     variables.
c
c     Interactions may be specified with parameters and regressor sets
c     preceding splines and integrated residuals:
c        PARAMETER(REF=...) * DURATION SPLINE(...)
c        REGRESSOR SET(REF=...) * PARAMETER(REF=...) * SPLINE(...)
c        REGRESSOR SET(REF=...) * INTEGRATED RESIDUAL(...)
c
c     Duration splines may not be interacted with integrated residuals.
c
c     PARAMETER may be abbreviated to PAR;
c     REGRESSOR SET may be abbreviated to REGSET;
c     DURATION SPLINE may be abbreviated to DURSPLINE;
c     REGRESSOR SPLINE may be abbreviated to REGSPLINE;
c     INTEGRATED RESIDUAL may be abbreviated to INTRES.
c
c     Hazard models are model type 1.
c
c     Model elements are stored in moel();
c        0 = end of model specification
c       -3 = what follows is a regressor set
c       -4 = what follows is a residual
c       -5 = what follows is a duration spline
c        2 = next factor is a parameter
c        3 = next factor is a regressor set
c        4 = next factor is a residual
c        5 = next factor is a duration spline
c        8 = next factor is a regressor spline
c
c     Constantijn Panis, 7 February 2000
c     Last change: duration statement required.
 
      implicit none
      integer levelmax,nfull,maxmodel,datadim,
     *   strctre(maxmodel),datas(maxmodel),mdllvl(maxmodel),
     *   maxxnode,xnodepos,meno,maxmeno,moel(maxmeno),
     *   moelpntr(maxmodel),numrset,numpar,numspl,nummodel,numdist,
     *   distdim(numdist),nold,prof,
     *   numfreq,maxnumfreq,nfreq(maxmodel),freqloc(maxnumfreq),
     *   nxold(levelmax,datadim),namepntr(levelmax,datadim),
     *   modeltype(maxmodel),minlnl(maxmodel),pardim(numpar),
     *   parpos(numpar),struc,ds,level,timevar,status,il,i1,i2,ipt,jpt
      real xnode(maxxnode),datastructure(datadim)
      logical smtxt
      character line*128,rsetname(numrset)*12,parname(numpar)*12,
     *   splname(numspl)*12,resname(*)*12,oldname(*)*8
 
      nummodel=nummodel+1
      if(nummodel.gt.maxmodel)call bug(ipt,jpt)
      modeltype(nummodel)=1
      moelpntr(nummodel)=meno+1
 
c     Record beginning position of frequency tables; number follows below.
c     For noncensored spell, keep track of lower and upper duration
c     variables and their difference; for censored spells, keep track of
c     duration:
      if(numfreq+4.le.maxnumfreq)then
         numfreq=numfreq+1
         nfreq(nummodel)=numfreq
         call putint(-1,freqloc(numfreq),4)
         numfreq=numfreq+3
      else
         numfreq=maxnumfreq
         nfreq(nummodel)=0
      endif
 
c     [DATA STRUCTURE = n;]
      call getstruc(datadim,struc,ds,il,ipt,jpt,datas(nummodel),
     *   strctre(nummodel),datastructure,line)
 
c     [{KEEP | DROP} IF condition;]
      call getkeepdrop(levelmax,xnodepos,moel(moelpntr(nummodel)),nold,
     *   maxxnode,nxold(1,ds),namepntr(1,ds),numfreq,maxnumfreq,freqloc,
     *   ipt,jpt,il,struc,xnode,oldname,line)
 
c     [NUMERATOR;]  [DENOMINATOR;]
      call getnumdenom(minlnl(nummodel),line,il)
 
      if(.not.smtxt(4,line,il,'CENSOR='))goto 800
      moel(moelpntr(nummodel)+1)=xnodepos+1
      call readexpr(1,levelmax,levelmax,mdllvl(nummodel),1,nold,
     *   maxxnode,xnodepos,xnode,nxold(1,ds),namepntr(1,ds),0,0,0,
     *   oldname,line,il,ipt,jpt,struc,';')
 
      if(.not.smtxt(4,line,il,'DURATION='))goto 800
c     DURATION=; implies that the last timemarks indicate the event window.
      if(smtxt(4,line,il,';'))goto 110
      moel(moelpntr(nummodel)+2)=xnodepos+1
      if(mdllvl(nummodel).eq.0)then
c        The censor variable may have been CENSOR=0, so that mdllvl
c        is not yet established.
         call readexpr(1,levelmax,levelmax,mdllvl(nummodel),2,nold,
     *      maxxnode,xnodepos,xnode,nxold(1,ds),namepntr(1,ds),0,0,0,
     *      oldname,line,il,ipt,jpt,struc,';')
         if(mdllvl(nummodel).eq.0)then
            if(levelmax.gt.1)goto 810
            mdllvl(nummodel)=1
         endif
      else
         call readexpr(1,levelmax,-mdllvl(nummodel),level,2,nold,
     *      maxxnode,xnodepos,xnode,nxold(1,ds),namepntr(1,ds),0,0,0,
     *      oldname,line,il,ipt,jpt,struc,';')
      endif
 
  110 if(smtxt(4,line,il,'TIMEmarks='))then
         if(smtxt(4,line,il,';'))goto 120
c        Time marks must be a variable name, not an expression
         call getname(4,line,il,i1,i2)
         if(i2.lt.i1)goto 800
         call findvar(levelmax,-(mdllvl(nummodel)+1),level,line(i1:i2),
     *      oldname,namepntr(1,ds),nxold(1,ds),timevar,line,il,struc,
     *      ipt,jpt,status)
         if(status.gt.0)stop
         moel(moelpntr(nummodel)+3)=timevar
         if(.not.smtxt(4,line,il,';'))goto 800
      endif
 
  120 if(.not.smtxt(4,line,il,'MODEL='))goto 800
 
c     moel(moelpntr) contains keep/drop information;
c     moel(moelpntr+1) contains censor information;
c     moel(moelpntr+2) contains duration variable information;
c     moel(moelpntr+3) contains timemark information;
 
      meno=moelpntr(nummodel)+3
      call read_model(levelmax,struc,mdllvl(nummodel),nfull,
     *   numrset,numspl,numdist,numpar,distdim,maxmeno,moel,meno,
     *   maxxnode,xnodepos,numfreq,maxnumfreq,freqloc,nold,prof,
     *   nxold(1,ds),namepntr(1,ds),parpos,pardim,modeltype(nummodel),
     *   -1,99,99,xnode,rsetname,splname,parname,oldname,resname,
     *   'MODEL = ','hazard',';',line,il,ipt,jpt)
 
c     Frequency tables will be computed for selected variables.  Array
c     nfreq stores at which position in freqloc the locations of data
c     structure m start and how many such variables there are.
      numfreq=min(numfreq,maxnumfreq)
      if(nfreq(nummodel).gt.0)nfreq(nummodel) = nfreq(nummodel) +
     *   (numfreq-nfreq(nummodel)+1)*(maxnumfreq+1)
      goto 999
 
 
  800 if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,801)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         write(6,801)
      endif
  801 format('ERROR encountered in a hazard model specification.  ',
     *   'The syntax is:',//,
     *   'HAZARD MODEL;',/,3x,'[DATA STRUCTURE = n;]',/,
     *   3x,'[{KEEP | DROP} IF ...;]',/,
     *   3x,'[NUMERATOR;]  [DENOMINATOR;]',/,
     *   3x,'CENSOR = varname;  DURATION = [durvar1 durvar2];',/,
     *   3x,'[TIMEMARKS = varname;]',/,3x,'MODEL = DURSPLINE ... + ',
     *   'REGSET ... + REGSPLINE ... + INTRES ... ;',//,
     *   'Please check that the CENSOR, DURATION, TIMEMARKS and MODEL ',
     *   'statements',/,'appear in the correct order.')
      stop
  810 if(ipt.gt.0)call wrapstring(8,0,72,0,1,'ERROR:  the CENSOR '//
     *   'and DURATION specifications in a hazard model do not '//
     *   'involve data variables, so that aML does not know at '//
     *   'what level (how often) the equation contributes to the '//
     *   'likelihood.')
      if(jpt.gt.0)call wrapstring(6,0,72,0,1,'ERROR:  the CENSOR '//
     *   'and DURATION specifications in a hazard model do not '//
     *   'involve data variables, so that aML does not know at '//
     *   'what level (how often) the equation contributes to the '//
     *   'likelihood.')
      stop
  999 return
      end
