      subroutine read_model4(levelmax,nfull,maxmodel,mdllvl,
     *   datadim,numdist,numrset,numspl,numpar,nold,prof,maxmeno,meno,
     *   moel,moelpntr,distdim,mtype,strctre,datastructure,datas,
     *   numfreq,maxnumfreq,nfreq,freqloc,nxold,namepntr,modeltype,
     *   minlnl,xnodepos,maxxnode,parpos,pardim,fulltype,xnode,nummodel,
     *   iid4,rsetname,splname,parname,resname,oldname,mname,line,il,
     *   ipt,jpt)
 
c     Subroutine to read the model specification for a certain data
c     structure.  The syntax is:
c
c     PROBIT MODEL; [DATA STRUCTURE = n;]
c        [({KEEP | DROP} IF ...)];
c        [NUMERATOR;]  [DENOMINATOR;]
c        OUTCOME = varname;
c        [THRESHOLD=...;]  /* default is fixed zero threshold */
c        MODEL = REGSET ... + REGSPLINE ... + INTRES ... + RES ...;
c
c     or:
c
c     ORDERED PROBIT MODEL; [DATA STRUCTURE = n;]
c        [({KEEP | DROP} IF ...)];
c        [NUMERATOR;]  [DENOMINATOR;]
c        OUTCOMES = varname varname;  THRESHOLDS = ...;
c        MODEL = REGSET ... + REGSPLINE ... + INTRES ... + RES ...;
c
c     or (thresholds-in-data):
c
c     ORDERED PROBIT MODEL; [DATA STRUCTURE = n;]
c        [({KEEP | DROP} IF ...)];
c        [NUMERATOR;]  [DENOMINATOR;]
c        THRESHOLD VARS = varname (-Inf=...) varname (Inf=...);
c        MODEL = REGSET ... + REGSPLINE ... + INTRES ... + RES ...;
c
c     The same syntax applies to LOGIT models (with PROBIT replaced
c     by LOGIT or LOGISTIC).
c
c     By default the likelihood function contains the modules related to
c     any model in the numerator.  If DENOMINATOR; is specified, the
c     module enters in the denominator only; if both NUMERATOR; and
c     DENOMINATOR; are specified, the module is replicated and enters
c     both ways.
c     The default translates into minlnl=0; denominator only minlnl=1;
c     numerator and denominator minlnl=2.
c
c     Upon input, mtype is:
c     mtype = 41  LOGIT MODEL
c             42  ORDERED LOGIT MODEL
c             43  PROBIT MODEL
c             44  ORDERED PROBIT MODEL
c
c     OUTCOME and THRESHOLD statements must precede the MODEL statement.
c     The information contained in them may take several forms.
c     moel(moelpntr) points at position in xnode where keep/drop
c     condition is stored, if any.
c     (1) ordered probit/logit: outcomes are two categorical variables,
c         thresholds is parameter set:
c         moel(+1)    -61
c         moel(+2)    position in xnode for two outcome variables
c         moel(+3++)  threshold vector reference
c     (2) ordered probit/logit: thresholds in data
c         moel(+1) = -62
c         moel(+2) = variable/expression for first threshold;
c                    -Infinity; second threshold; +Infinity
c     (3) simple probit/logit: outcome is binary variable,
c         threshold is parameter
c         moel(+1)    -63
c         moel(+2)    position in xnode for outcome variable
c         moel(+3++)  threshold parameter reference
c     (4) simple probit/logit: outcome is binary variable,
c         threshold is assumed fixed at zero
c         moel(+1)    -64
c         moel(+2)    position in xnode for outcome variable
c
c     Subsequent moel(.) elements specify the model:
c        0=end of model specification
c       -3=what follows is a regressor set
c       -4=what follows is an integrated residual
c       -7=what follows is a residual
c        2=next factor is a parameter
c        3=next factor is a regressor set
c        4=next factor is a residual
c        7=next factor is an integrated residual
c        8=next factor is a regressor spline
c
c     Constantijn Panis, 4 September 2002
c     Last change: eliminate read_mdl4
 
      implicit none
      integer levelmax,nfull,maxmodel,datadim,numdist,numrset,
     *   numspl,numpar,nold,prof,maxmeno,meno,moel(maxmeno),
     *   moelpntr(maxmodel),distdim(numdist),strctre(maxmodel),
     *   datas(maxmodel),numfreq,maxnumfreq,freqloc(maxnumfreq),
     *   nfreq(maxmodel),mdllvl(maxmodel),
     *   nxold(levelmax,datadim),namepntr(levelmax,datadim),
     *   modeltype(maxmodel),mtype,minlnl(maxmodel),parpos(numpar),
     *   pardim(numpar),fulltype(nfull),xnodepos,maxxnode,
     *   nummodel,struc,ds,il,ipt,jpt,resdone,trm
      real xnode(maxxnode),datastructure(datadim)
      logical iid4,smtxt
      character line*128,rsetname(numrset)*12,splname(numspl)*12,
     *   parname(numpar)*12,resname(*)*12,oldname(nold)*8,mname*(*),
     *   tmp*32
 
      nummodel=nummodel+1
      if(nummodel.gt.maxmodel)call bug(ipt,jpt)
      modeltype(nummodel)=mtype
      moelpntr(nummodel)=meno+1
 
c     Record beginning position of summary stats; number follows below.
      if((mtype.eq.41.or.mtype.eq.43).and.numfreq.lt.maxnumfreq)then
         numfreq=numfreq+1
         nfreq(nummodel)=numfreq
         freqloc(numfreq)=-1
      elseif((mtype.eq.42.or.mtype.eq.44).and.numfreq+2.le.maxnumfreq)
     *   then
         numfreq=numfreq+1
         nfreq(nummodel)=numfreq
         freqloc(numfreq)=-1
         numfreq=numfreq+1
         freqloc(numfreq)=-1
      else
         numfreq=maxnumfreq
         nfreq(nummodel)=0
      endif
 
c     [DATA STRUCTURE = n;]
      call getstruc(datadim,struc,ds,il,ipt,jpt,datas(nummodel),
     *   strctre(nummodel),datastructure,line)
 
c     [{KEEP | DROP} IF condition;]
      call getkeepdrop(levelmax,xnodepos,moel(moelpntr(nummodel)),nold,
     *   maxxnode,nxold(1,ds),namepntr(1,ds),numfreq,maxnumfreq,freqloc,
     *   ipt,jpt,il,struc,xnode,oldname,line)
 
c     [NUMERATOR;]  [DENOMINATOR;]
      call getnumdenom(minlnl(nummodel),line,il)
 
      if(smtxt(4,line,il,'THRESHOLD VARS ='))then
c        THRESHOLD VARS = varname(-inf=x) varname(inf=x);
         if(modeltype(nummodel).ne.42.and.modeltype(nummodel).ne.44)
     *      goto 820
c        Ordered model with thresholds given by two variables:
c           moel(1) = -62
c           moel(2) = variable/expression for first threshold;
c                     -Infinity; second threshold; +Infinity
         meno=moelpntr(nummodel)+1
         moel(meno)=-62
         meno=meno+1
         moel(meno)=xnodepos+1
         call read_tauvars(levelmax,nxold(1,ds),namepntr(1,ds),nold,
     *      mdllvl(nummodel),numfreq,maxnumfreq,freqloc,xnodepos,
     *      maxxnode,xnode,il,struc,oldname,line,ipt,jpt)
         if(.not.smtxt(4,line,il,'MODEL='))goto 830
      elseif(smtxt(4,line,il,'OUTCOMEs='))then
c        moel(moelpntr+1) is -61, -63, or -64, to be determined
         meno=moelpntr(nummodel)+2
         moel(meno)=xnodepos+1
         if(modeltype(nummodel).eq.41.or.modeltype(nummodel).eq.43)then
c           Simple probits/logits have one outcome variable.  Note
c           that this outcome is already tabulated, above.
            call readexpr(1,levelmax,levelmax,mdllvl(nummodel),1,nold,
     *         maxxnode,xnodepos,xnode,nxold(1,ds),namepntr(1,ds),0,0,0,
     *         oldname,line,il,ipt,jpt,struc,';')
         else
c           Ordered probits/logits have two outcome variables:
            call readexpr(1,levelmax,levelmax,mdllvl(nummodel),2,nold,
     *         maxxnode,xnodepos,xnode,nxold(1,ds),namepntr(1,ds),
     *         numfreq,maxnumfreq,freqloc,oldname,line,il,ipt,jpt,
     *         struc,';')
         endif
c        Read the (optional) threshold statement:
         call read_tau(levelmax,nfull,maxmeno,moel,meno,
     *      moelpntr(nummodel),modeltype(nummodel),nxold(1,ds),
     *      namepntr(1,ds),nold,mdllvl(nummodel),numpar,numfreq,
     *      maxnumfreq,freqloc,xnodepos,maxxnode,pardim,fulltype,
     *      parpos,xnode,il,struc,oldname,parname,mname,line,ipt,jpt)
         if(.not.smtxt(4,line,il,'MODEL='))goto 840
      else
         goto 800
      endif
 
c     Outcomes may be given as, for example, OUTCOME=0; but only if
c     there is only one level in the data.  Otherwise, a variable
c     is required.
      if(mdllvl(nummodel).eq.0)then
         if(levelmax.gt.1)goto 810
         mdllvl(nummodel)=1
      endif
 
c     MODEL=....;
c     Residuals are only allowed in probits:
      if(modeltype(nummodel).eq.43.or.modeltype(nummodel).eq.44)then
         resdone=0
      else
         resdone=-1
      endif
      call read_model(levelmax,struc,mdllvl(nummodel),nfull,
     *   numrset,numspl,numdist,numpar,distdim,maxmeno,moel,meno,
     *   maxxnode,xnodepos,numfreq,maxnumfreq,freqloc,nold,prof,
     *   nxold(1,ds),namepntr(1,ds),parpos,pardim,modeltype(nummodel),
     *   resdone,99,-1,xnode,rsetname,splname,parname,oldname,resname,
     *   'MODEL = ',mname,';',line,il,ipt,jpt)
 
      if(resdone.eq.0)then
c        Residual need not be specified; default is IID N(0,1) for
c        probits and IID Sech2 for logits.  Record this for probits,
c        not for logits.  Note that moel(meno) was set to -99 to
c        denote the end of the model specification, so overwrite that.
         moel(meno)=-7
         moel(meno+1)=1
         moel(meno+2)=7
c        Residual is IID N(0,1) or IID Sech2; see figure_res.
         moel(meno+3)=4
         meno=meno+4
c        Finish off with a -99:
         moel(meno)=-99
         resdone=2
      endif
c     Are probit residuals IID N(0,1)?   (Logit residuals always
c     are IID, though not normal.)  Iid4 is only used for probits.
      iid4=iid4.and.(resdone.eq.2)
 
c     Frequency tables will be computed for selected variables.  Array
c     nfreq stores at which position in freqloc the locations of data
c     structure m start and how many such variables there are.
      numfreq=min(numfreq,maxnumfreq)
      if(nfreq(nummodel).gt.0)nfreq(nummodel) = nfreq(nummodel) +
     *   (numfreq-nfreq(nummodel)+1)*(maxnumfreq+1)
 
      goto 999
 
  800 tmp=mname
      if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,801)tmp(1:trm(tmp))
         if(modeltype(nummodel).eq.41)write(8,802)'LOGIT',';'
         if(modeltype(nummodel).eq.42)write(8,803)
     *      'LOGIT',';','LOGIT',';'
         if(modeltype(nummodel).eq.43)write(8,802)'PROBIT',' + RES ...;'
         if(modeltype(nummodel).eq.44)write(8,803)
     *      'PROBIT',' + RES ...;','PROBIT',' + RES ...;'
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         write(6,801)tmp(1:trm(tmp))
         if(modeltype(nummodel).eq.41)write(6,802)'LOGIT',';'
         if(modeltype(nummodel).eq.42)write(6,803)
     *      'LOGIT',';','LOGIT',';'
         if(modeltype(nummodel).eq.43)write(6,802)'PROBIT',' + RES ...;'
         if(modeltype(nummodel).eq.44)write(6,803)
     *      'PROBIT',' + RES ...;','PROBIT',' + RES ...;'
      endif
  801 format('ERROR encountered while reading a ',a,' model ',
     *   'specification.  The syntax is:')
  802 format(/,a,' MODEL; [DATA STRUCTURE = n;]',/,
     *   3x,'[(KEEP|DROP IF ...);]',/,
     *   3x,'[NUMERATOR;]  [DENOMINATOR;]',/,
     *   3x,'OUTCOME = ...;',/,3x,'[THRESHOLD = ...;]',/,
     *   3x,'MODEL = REGSET ... + INTRES ...',a)
  803 format(/,'ORDERED ',a,' MODEL; [DATA STRUCTURE = n;]',/,
     *   3x,'[(KEEP|DROP IF ...);]',/,
     *   3x,'[NUMERATOR;]  [DENOMINATOR;]',/,
     *   3x,'OUTCOMES = ...;',/,3x,'THRESHOLDS = ...;',/,
     *   3x,'MODEL = REGSET ... + INTRES ...',a,//,
     *   'or, if thresholds are known data variables:',//,
     *   'ORDERED ',a,' MODEL; [DATA STRUCTURE = n;]',/,
     *   3x,'[(KEEP|DROP IF ...);]',/,
     *   3x,'[NUMERATOR;]  [DENOMINATOR;]',/,
     *   3x,'THRESHOLD VARS = ...;',/,
     *   3x,'MODEL = REGSET ... + INTRES ...',a)
      stop
  810 tmp=mname
      if(ipt.gt.0)call wrapstring(8,0,72,0,1,'ERROR:  the outcome in '//
     *   'a '//tmp(1:trm(tmp))//' model specification does not '//
     *   'involve a data variable, so that aML does not know at what '//
     *   'level (how often) the equation contributes to the '//
     *   'likelihood.')
      if(jpt.gt.0)call wrapstring(6,0,72,0,1,'ERROR:  the outcome in '//
     *   'a '//tmp(1:trm(tmp))//' model specification does not '//
     *   'involve a data variable, so that aML does not know at what '//
     *   'level (how often) the equation contributes to the '//
     *   'likelihood.')
      stop
  820 if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,821)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         write(6,821)
      endif
  821 format('ERROR:  threshold variables are only allowed for ',
     *   'ordered logit',/,'and probit models.')
      stop
  830 if(ipt.gt.0)then
         call caretline(8,line,il)
         if(modeltype(nummodel).eq.42)write(8,831)'logit','LOGIT',';'
         if(modeltype(nummodel).eq.44)write(8,831)'probit','PROBIT',
     *      ' + RES ...;'
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         if(modeltype(nummodel).eq.42)write(6,831)'logit','LOGIT',';'
         if(modeltype(nummodel).eq.44)write(6,831)'probit','PROBIT',
     *      ' + RES ...;'
      endif
  831 format('ERROR:  expect MODEL statement here.  The syntax for ',
     *   'ordered ',a,/,' models with known thresholds in data is:',/,
     *   'ORDERED ',a,' MODEL;',/,3x,'[DATA STRUCTURE = n;]',/,
     *   3x,'[{KEEP|DROP} IF ...;]',/,
     *   3x,'[NUMERATOR;]  [DENOMINATOR;]',/,
     *   3x,'THRESHOLD VARS = ...;',/,
     *   3x,'MODEL = REGSET ... + INTRES ...',a)
      stop
  840 if(ipt.gt.0)then
         call caretline(8,line,il)
         if(modeltype(nummodel).eq.41)write(8,841)
     *      'LOGIT','OUTCOME','THRESHOLD',';'
         if(modeltype(nummodel).eq.42)write(8,841)
     *      'ORDERED LOGIT','OUTCOMES','THRESHOLDS',';'
         if(modeltype(nummodel).eq.43)write(8,841)
     *      'PROBIT','OUTCOME','THRESHOLD',' + RES ...;'
         if(modeltype(nummodel).eq.44)write(8,841)
     *      'ORDERED PROBIT','OUTCOMES','THRESHOLDS',' + RES ...;'
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         if(modeltype(nummodel).eq.41)write(6,841)
     *      'LOGIT','OUTCOME','THRESHOLD',';'
         if(modeltype(nummodel).eq.42)write(6,841)
     *      'ORDERED LOGIT','OUTCOMES','THRESHOLDS',';'
         if(modeltype(nummodel).eq.43)write(6,841)
     *      'PROBIT','OUTCOME','THRESHOLD',' + RES ...;'
         if(modeltype(nummodel).eq.44)write(6,841)
     *      'ORDERED PROBIT','OUTCOMES','THRESHOLDS',' + RES ...;'
      endif
  841 format('ERROR:  expect MODEL statement here.  The syntax is:',/,
     *   3x,a,' MODEL;',/,3x,'[DATA STRUCTURE = n;]',/,
     *   6x,'[{KEEP|DROP} IF ...;]',/,
     *   6x,'[NUMERATOR;]  [DENOMINATOR;]',/,
     *   6x,a,' = ...;',/,
     *   6x,'[',a,' = ...;]',/,
     *   6x,'MODEL = REGSET ... + INTRES ...',a)
      stop
  999 return
      end
