      subroutine read_model6(levelmax,nfull,nummodel,maxmodel,
     *   datadim,mdllvl,strctre,numrset,numspl,numdist,numpar,
     *   datastructure,datas,distdim,maxmeno,moel,moelpntr,meno,
     *   numfreq,maxnumfreq,nfreq,freqloc,nxold,namepntr,nold,prof,
     *   maxxnode,xnodepos,modeltype,minlnl,pardim,parpos,xnode,
     *   rsetname,splname,parname,oldname,resname,
     *   line,il,ipt,jpt)
 
c     Subroutine to read the continuous model specification for a certain
c     data structure.  The syntax is:
c
c     CONTINUOUS MODEL; [DATA STRUCTURE = n;]
c        [([KEEP | DROP] IF ...)];
c        [NUMERATOR;]  [DENOMINATOR;]
c        OUTCOME = varname;
c        MODEL = REGSET ... + REGSPLINE ... + INTRES ... + RES ...;
c
c     By default the likelihood function contains the modules related to
c     any model in the numerator.  If DENOMINATOR; is specified, the
c     module enters in the denominator only; if both NUMERATOR; and
c     DENOMINATOR; are specified, the module is replicated and enters
c     both ways.  The latter is useful in models with initial conditions
c     problems: the probability of making it into the sample does not
c     cancel out of the likelihood function if there is heterogeneity.
c     The default translates into minlnl=0; denominator only minlnl=1;
c     numerator and denominator minlnl=2.
c
c     Interactions of regressor sets, regressor splines, residuals and
c     integrated residuals with (other) regressor sets and parameters are
c     allowed.  In such cases, the main model element (regressor set,
c     regressor spline, residual) must be listed **after** the regressor
c     set or parameter with which it is interacted.
c
c     REGRESSOR SPLINE may be abbreviated to REGSPLINE.
c
c     The product of regressor sets allows for regressors to be "loaded"
c     by some factor.  For example, by tenure on the job, where tenure
c     itself may be a spline transformation.  Regressor sets may also
c     be multiplied by a parameter in the usual manner.
c
c     RESIDUAL(DRAW=_level n,..) indicates when new "draws" are taken from a
c     certain distribution.  It is crucial in determining the covariance
c     matrix of residuals of each point of observation.  Integer n must
c     be less than or equal to the outcome variable level, mdllvl.
c     Alternatively, DRAW=varname is allowed, where varname is an
c     integer variable (of any variability) that indicates which
c     points must get the same draw.
c
c     Continuous models are model type 6.
c
c     Model elements are stored in moel();
c        0 = end of model specification
c       -3 = the model element that follows is a regressor set
c       -4 = the model element that follows is an integrated residual
c       -7 = the model element that follows is a residual
c        2 = next factor is a parameter
c        3 = next factor is a regressor set
c        4 = next factor is an integrated residual
c        7 = next factor is a non-integrated residual
c        8 = next factor is a regressor spline
c
c     Constantijn Panis, 22 July 1996
c     Latest change:  any level outcomes
 
      implicit none
      integer levelmax,nfull,nummodel,maxmodel,datadim,
     *   strctre(maxmodel),mdllvl(maxmodel),numrset,numspl,numdist,
     *   numpar,
     *   datas(maxmodel),distdim(numdist),moelpntr(maxmodel),
     *   maxmeno,moel(maxmeno),meno,nold,prof,
     *   numfreq,maxnumfreq,nfreq(maxmodel),freqloc(maxnumfreq),
     *   nxold(levelmax,datadim),namepntr(levelmax,datadim),maxxnode,
     *   xnodepos,modeltype(maxmodel),minlnl(maxmodel),
     *   pardim(numpar),parpos(numpar),struc,ds,intresdone,
     *   resdone,il,ipt,jpt
      real xnode(maxxnode),datastructure(datadim)
      logical smtxt
      character line*128,rsetname(numrset)*12,splname(numspl)*12,
     *   parname(numpar)*12,oldname(nold)*8,resname(*)*12
 
      nummodel=nummodel+1
      if(nummodel.gt.maxmodel)call bug(ipt,jpt)
      modeltype(nummodel)=6
      moelpntr(nummodel)=meno+1
 
c     Record beginning position of summary stats; number follows below.
      if(numfreq.lt.maxnumfreq)then
         numfreq=numfreq+1
         nfreq(nummodel)=numfreq
         freqloc(numfreq)=-1
      else
         nfreq(nummodel)=0
      endif
 
c     [DATA STRUCTURE = n;]
      call getstruc(datadim,struc,ds,il,ipt,jpt,datas(nummodel),
     *   strctre(nummodel),datastructure,line)
 
c     [{KEEP | DROP} IF condition;]
      call getkeepdrop(levelmax,xnodepos,moel(moelpntr(nummodel)),nold,
     *   maxxnode,nxold(1,ds),namepntr(1,ds),numfreq,maxnumfreq,freqloc,
     *   ipt,jpt,il,struc,xnode,oldname,line)
 
c     [NUMERATOR;]  [DENOMINATOR;]
      call getnumdenom(minlnl(nummodel),line,il)
 
      if(.not.smtxt(4,line,il,'OUTCOME ='))goto 800
c     OUTCOME = varname;   (where varname may be an expression)
      meno=moelpntr(nummodel)+1
      moel(meno)=xnodepos+1
      call readexpr(1,levelmax,levelmax,mdllvl(nummodel),1,nold,
     *   maxxnode,xnodepos,xnode,nxold(1,ds),namepntr(1,ds),0,0,0,
     *   oldname,line,il,ipt,jpt,struc,';')
 
      if(.not.smtxt(4,line,il,'MODEL ='))goto 800
      intresdone=0
      resdone=0
      call read_model(levelmax,struc,mdllvl(nummodel),nfull,
     *   numrset,numspl,numdist,numpar,distdim,maxmeno,moel,meno,
     *   maxxnode,xnodepos,numfreq,maxnumfreq,freqloc,nold,prof,
     *   nxold(1,ds),namepntr(1,ds),parpos,pardim,modeltype(nummodel),
     *   resdone,intresdone,-1,xnode,rsetname,splname,parname,oldname,
     *   resname,'MODEL = ','continuous',';',line,il,ipt,jpt)
 
c     Make sure a residual has been specified:
      if(resdone.eq.0)goto 850
 
c     Frequency tables will be computed for selected variables.  Array
c     nfreq stores at which position in freqloc the locations of data
c     structure m start and how many such variables there are.
      numfreq=min(numfreq,maxnumfreq)
      if(nfreq(nummodel).gt.0)nfreq(nummodel) = nfreq(nummodel) +
     *   (numfreq-nfreq(nummodel)+1)*(maxnumfreq+1)
      goto 999
 
  800 if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,801)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         write(6,801)
      endif
  801 format('ERROR encountered while reading a continuous model',/,
     *   'specification.  The syntax is:',/,
     *   'CONTINUOUS MODEL;',/,3x,'[DATA STRUCTURE = n;]',/,
     *   3x,'[(KEEP|DROP IF ...);]',/,
     *   3x,'[NUMERATOR;]  [DENOMINATOR;]',/,
     *   3x,'OUTCOME = {varname | LOG(varname)};',/,
     *   3x,'MODEL = REGSET ... + REGSPLINE ... + ',
     *   'INTRES ... + RES ...;')
      stop
  810 if(ipt.gt.0)then
         call caretline(8,line,il)
         call writeaia(8,72,1,'ERROR: structure ',struc,
     *      ' not found in the data.')
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         call writeaia(6,72,1,'ERROR: structure ',struc,
     *      ' not found in the data.')
      endif
      stop
  850 if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,851)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         write(6,851)
      endif
  851 format('ERROR: a continuous model specification is incomplete.  ',
     *   'At a minimum,',/,'there must be a RESIDUAL model element.')
      stop
  999 return
      end
 
