      subroutine read_rset(levelmax,struc,maxmeno,moel,meno,numrset,
     *   nxold,namepntr,maxxnode,xnodepos,nold,maxlevel,numfreq,
     *   maxnumfreq,freqloc,xnode,rsetname,oldname,line,il,ipt,jpt)
 
c     Subroutine to read in a regressor set reference as part of a
c     model specification.
c
c     There are alternative ways of referencing a regressor set:
c     Direct referencing:
c        REGRESSOR SET regsetname
c        REGRESSOR SET (REF = regsetname)
c        REGRESSOR SET (REFERENCE = regsetname)
c     Indirect referencing:
c        REGRESSOR SET (REFVAR = varname)
c     where varname may also be an integer expression.
c
c     Reference information is stored in moel.
c     moel(1)  = 3  This is a regressor set
c     moel(2)  = 1 reference is direct
c                2 reference is indirect
c     moel(3)  = definition rank number if moel(2)=1
c                expression position in xnode if moel(2)=2
c
c     Constantijn Panis, 11 January 1998
c     Last change: new REFVAR machinery
 
      implicit none
      integer levelmax,struc,maxmeno,moel(maxmeno),meno,numrset,
     *   nxold(levelmax),namepntr(levelmax),maxxnode,xnodepos,nold,
     *   maxlevel,level,numfreq,maxnumfreq,freqloc(maxnumfreq),
     *   il,i1,i2,ref,ipt,jpt,status,trm
      real xnode(maxxnode)
      logical paren,smtxt
      character rsetname(numrset)*12,oldname(*)*8,line*128,
     *   string*60,idstring*60
 
      if(meno+3.gt.maxmeno)call
     *   maxexceed('maxmeno',meno+3,maxmeno,ipt,jpt)
 
c     This is a regressor set:
      meno=meno+1
      moel(meno)=3
 
      if(smtxt(4,line,il,'(REFVAR='))then
c        Indirectly referenced
         moel(meno+1)=2
         moel(meno+2)=xnodepos+1
         meno=meno+2
         call readexpr(1,levelmax,maxlevel,level,1,nold,maxxnode,
     *      xnodepos,xnode,nxold,namepntr,numfreq,maxnumfreq,freqloc,
     *      oldname,line,il,ipt,jpt,struc,')')
      else
c        Directly referenced:
         paren=smtxt(4,line,il,'(REFerence=')
         call getword(4,line,il,i1,i2,status)
         if(status.gt.0)goto 800
         call char12match(line(i1:i2),rsetname,numrset,ref)
         if(ref.eq.0)goto 810
         moel(meno+1)=1
         moel(meno+2)=ref
         meno=meno+2
         if(paren)then
            if(.not.smtxt(4,line,il,')'))goto 800
         endif
      endif
      goto 999
 
  800 string=idstring(struc,0,0,0,0)
      if(ipt.gt.0)then
         call wrapstring(8,0,72,0,1,'ERROR in the syntax of a '//
     *      'regressor set reference in a model specification'//
     *      string(1:trm(string))//'  The syntax is:')
         write(8,801)
      endif
      if(jpt.gt.0)then
         call wrapstring(6,0,72,0,1,'ERROR in the syntax of a '//
     *      'regressor set reference in a model specification'//
     *      string(1:trm(string))//'  The syntax is:')
         write(6,801)
      endif
  801 format(6x,'REGSET regsetname',/,
     *   'or, in case the regressor set is indirectly referenced:',/,
     *   6x,'REGSET (REFVAR = varname)',/,'where varname may ',
     *   'also be an integer expression.')
      stop
  810 string=idstring(struc,0,0,0,0)
      if(ipt.gt.0)call wrapstring(8,0,72,0,1,'ERROR in a model '//
     *   'specification'//string(1:trm(string))//'  Regressor '//
     *   'set '''//line(i1:i2)//''' has not been defined.')
      if(jpt.gt.0)call wrapstring(6,0,72,0,1,'ERROR in a model '//
     *   'specification'//string(1:trm(string))//'  Regressor '//
     *   'set '''//line(i1:i2)//''' has not been defined.')
      stop
  999 return
      end
