      subroutine read_rspl(levelmax,numspl,maxmeno,moel,meno,nxold,
     *   namepntr,nold,maxxnode,xnodepos,maxlevel,maxnumfreq,numfreq,
     *   freqloc,struc,xnode,oldname,splname,line,il,ipt,jpt)
 
c     Subroutine to read a regressor spline specification.  The syntax is:
c        REGSPLINE (VAR = varname, REF = splinename)
c        REGSPLINE (VAR = varname, REFVAR = varname)
c
c     where varnames may also be expressions.
c
c     (1) VAR  for the variable that needs to be transformed.
c     (2) REF  for the selection of the functional form of the spline,
c              if directly referenced.
c     (3) REFVAR  for the selection of the functional form of the spline,
c              if indirectly referenced.
c
c     Information is stored in moel(*):
c     moel(1) = 8  this is a regressor spline
c     moel(2) =    position in xnode where origin expression starts
c     moel(3) = 1  spline definition directly specified
c               2  spline definition indirectly referenced
c     moel(4) = spline definition number if moel(4)=1
c               expression position in xnode if moel(4)=2
c
c     Constantijn Panis, 6 July 1999
c     Last change: no restriction on level of variable to be transformed.
 
      implicit none
      integer levelmax,numspl,maxmeno,moel(maxmeno),meno,
     *   namepntr(levelmax),nold,nxold(levelmax),maxxnode,xnodepos,
     *   maxlevel,level,maxnumfreq,numfreq,freqloc(maxnumfreq),
     *   struc,ref,i1,i2,il,ipt,jpt,status,trm
      real xnode(maxxnode)
      logical smtxt
      character line*128,oldname(nold)*8,splname(numspl)*12,
     *   string*60,idstring*60
 
      if(meno+4.gt.maxmeno)call
     *   maxexceed('maxmeno',meno+4,maxmeno,ipt,jpt)
 
c     This is a regressor spline:
      moel(meno+1)=8
 
c     VAR (to be transformed)
c     Note: do not place any restrictions on the level of the variable to
c     be transformed.  It typically is levelmax=mdllvl (mdllvl+1 for hazard
c     models with TIMEMARKS statement), but if this is exceeded, figure_rspl
c     will catch it.
c     Note: do not add the transformed variable to the list of variables for
c     which summary statistics are computed (freqloc); (1) origin summary
c     statistics are already computed in originstats, see figure_rspl; (2)
c     statistics would be incorrectly computed if the transformed variable is
c     at level mdllvl+1, since figdata1 does not loop over level mdllvl+1.
      if(.not.smtxt(4,line,il,'(VARiable='))goto 800
      moel(meno+2)=xnodepos+1
      call readexpr(1,levelmax,levelmax,level,1,nold,maxxnode,xnodepos,
     *   xnode,nxold,namepntr,0,0,0,oldname,line,il,ipt,jpt,struc,',')
 
c     REF=   or   REFVAR=
c     Note: reference variables must be of level<=levelmax.  This is mdllvl,
c     even for hazard models.  Figure_rspl would otherwise be very complicated,
c     and the benefits are minimal.
      if(smtxt(4,line,il,','))continue
      if(smtxt(4,line,il,'REFerence='))then
c        Directly referenced:
         call getword(4,line,il,i1,i2,status)
         if(status.gt.0)goto 800
         call char12match(line(i1:i2),splname,numspl,ref)
         if(ref.eq.0)goto 800
         moel(meno+3)=1
         moel(meno+4)=ref
         meno=meno+4
         if(.not.smtxt(4,line,il,')'))goto 800
      elseif(smtxt(4,line,il,'REFVAR='))then
c        Indirectly referenced:
         moel(meno+3)=2
         moel(meno+4)=xnodepos+1
         meno=meno+4
         call readexpr(1,levelmax,maxlevel,level,1,nold,maxxnode,
     *      xnodepos,xnode,nxold,namepntr,numfreq,maxnumfreq,freqloc,
     *      oldname,line,il,ipt,jpt,struc,')')
      else
         goto 800
      endif
      goto 999
 
  800 string=idstring(struc,0,0,0,0)
      if(ipt.gt.0)then
         call caretline(8,line,il)
         call wrapstring(8,0,72,0,1,'ERROR in the syntax of a '//
     *      'regressor spline reference in a model specification'//
     *      string(1:trm(string))//'  The syntax is:')
         write(8,801)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         call wrapstring(6,0,72,0,1,'ERROR in the syntax of a '//
     *      'regressor spline reference in a model specification'//
     *      string(1:trm(string))//'  The syntax is:')
         write(6,801)
      endif
  801 format(6x,'REGSPLINE (VAR = varname, REF = splinename)',/,
     *   'or, in case the spline is indirectly referenced:',/,
     *   6x,'REGSPLINE ','(VAR = varname, REFVAR = varname)',/,
     *   'where the REFVAR varname may also be an integer expression.')
      stop
  999 return
      end
