      subroutine readoption(convcrit,version,nx1,stprng,output,nobs,
     *   datanobs,maxiter,wghtpos,maxmeno,maxnmapmoel,maxds,vdimmax,
     *   maxfreqcat,maxdraws,maxnfreq,maxnode,maxref,maxxnode,maxmspl,
     *   useid,useobs,prof,sparsity,numsearch,detail,check99999,posdef,
     *   ieee,oldname,title,scrfile,gridfile,line,il,ipt,jpt)
 
c     Subroutine to read an optional statement.  The syntax and defaults are:
c        option title = 'This is a title';                 # title=' '
c        option version = {1|2};                           # version=2
c        option converge = [wgn|fri|gn|rpc < x [or ...]];  # wgn<0.1
c        option scratchfile = filename for scratch data;   # ' '
c        option gridfile = filename;                       # ' '
c        option [normalized] weight = varname;             # (no weights)
c        option step range = n [to n];                     # -10 to 4
c        option save step;
c        option file info level = n;                       # ipt=5
c        option screen info level = n;                     # jpt=3
c        option iterations = n;                            # maxiter=40
c        option observations = n;                          # nobs=0 (=all)
c        option variance-covariance matrix;
c        option correlation matrix;
c        option gradient;
c        option Hessian matrix;
c        option table format;
c        option starting value format;
c        option huber;
c        option numerical search;
c        option numerical standard errors;
c        option check99999 = no;                      # yes
c        option ensure positive definite = no;        # yes
c        option IEEE warnings;
c        option sparsity=no;                          # yes
c        option maximum specification space = n;      # maxmeno=1000
c        option maximum model space = n;              # maxnmapmoel=5000
c        option maximum hazard baseline space = n;    # maxds=4000
c        option maximum scratch data space = n;       # vdimmax=max(5000,2*xdimm
c        option maximum number of frequency categories = n; # maxfreqcat=20
c        option maximum number of residual draws = n; # maxdraws=100
c        option maximum number of summary statistics = n; # maxnfreq=100
c        option maximum number of hazard baseline nodes = n; # maxnode=100
c        option maximum number of reference numbers = n; # maxref=300
c        option maximum number of user-defined constants = n; # maxxnode=500
c        option maximum number of hazard model splines = n; # maxmspl=0
c
c     Undocumented options:
c        option gradientcheck;               # no
c        option proficiency | medium | expert | guru};
c        option useid {= | <=} n;
c        option useobs {= | <=} n;
c
c     Upon entering this routine, line(il:il) is positioned after the
c     word "option".  This routine is called first by scanctl and then
c     (without any real effects) by readctl_.
c
c     Constantijn Panis,  27 November 2002
c     Last change: version
 
      implicit none
      double precision convcrit(4)
      integer version,nx1,stprng(3),output(10),ipt,jpt,nobs,datanobs,
     *   maxiter,wghtpos,maxmeno,maxnmapmoel,maxds,vdimmax,maxfreqcat,
     *   maxdraws,maxnfreq,maxnode,maxref,maxxnode,posdef,
     *   maxmspl,useid,useobs,prof,sparsity,numsearch,
     *   i,il,i1,i2,flag,status,intlen
      logical detail,check99999,ieee,equal,smtxt
      character oldname(nx1)*8,title*80,scrfile*128,gridfile*128,
     *   line*128
 
c     Read in the option:
      if(smtxt(4,line,il,';'))goto 999
      if(smtxt(4,line,il,'TITLE ='))goto 110
      if(smtxt(4,line,il,'VERSION ='))goto 115
      if(smtxt(4,line,il,'CONVERGE'))goto 120
      if(smtxt(4,line,il,'SCRATCHFILE ='))goto 140
      if(smtxt(4,line,il,'GRIDFILE ='))goto 150
      if(smtxt(4,line,il,'NORMalized WEIGHT ='))goto 160
      if(smtxt(4,line,il,'WEIGHT ='))goto 170
      if(smtxt(4,line,il,'STEP RANGE ='))goto 180
      if(smtxt(4,line,il,'SAVE STEP'))goto 190
      if(smtxt(4,line,il,'FILE INFO LEVEL ='))goto 200
      if(smtxt(4,line,il,'SCREEN INFO LEVEL ='))goto 210
      if(smtxt(4,line,il,'ITERations ='))goto 220
      if(smtxt(4,line,il,'OBServations ='))goto 230
      if(smtxt(4,line,il,'HESSIAN matrix'))goto 250
      if(smtxt(4,line,il,'VARiance-COVariance MATrix'))goto 260
      if(smtxt(4,line,il,'CORRelation MATrix'))goto 270
      if(smtxt(4,line,il,'TABLE FORMAT'))goto 280
      if(smtxt(4,line,il,'STARTING VALUE FORMAT'))goto 290
      if(smtxt(4,line,il,'HUBER'))goto 300
      if(smtxt(4,line,il,'NUMerical SEARCH'))goto 305
      if(smtxt(4,line,il,'NUMerical STanDard ERRors'))goto 310
      if(smtxt(4,line,il,'CHECK99999'))goto 330
      if(smtxt(4,line,il,'ENSure POSitive DEFinite'))goto 340
      if(smtxt(4,line,il,'IEEE warnings'))then
c        goto 800
c        goto 350
         goto 800
      endif
      if(smtxt(4,line,il,'SPARSITY'))goto 360
      if(smtxt(4,line,il,'MAXimum SPECification SPACE ='))goto 410
      if(smtxt(4,line,il,'MAXimum MODEL SPACE ='))goto 420
      if(smtxt(4,line,il,'MAXimum HAZard BASEline SPACE ='))goto 430
      if(smtxt(4,line,il,'MAXimum SCRatch DATA SPACE ='))goto 440
      if(smtxt(4,line,il,'MAXimum NUMber of'))then
         if(smtxt(4,line,il,'FREQuency CATegories ='))goto 450
         if(smtxt(4,line,il,'RESidual DRAWs ='))goto 460
         if(smtxt(4,line,il,'SUMmary STATistics ='))goto 470
         if(smtxt(4,line,il,'HAZard BASEline NODES ='))goto 480
         if(smtxt(4,line,il,'REFerence NUMbers ='))goto 490
         if(smtxt(4,line,il,'USER-DEFined CONSTants ='))goto 510
         if(smtxt(4,line,il,'HAZard MODEL SPLines ='))goto 520
         goto 800
      endif
c     Undocumented options:
      if(smtxt(4,line,il,'USEID'))goto 600
      if(smtxt(4,line,il,'USEOBS'))goto 610
      if(smtxt(4,line,il,'GRADientCHECK'))goto 620
      if(smtxt(4,line,il,'PROFiciency ='))goto 630
c     Obsolete syntax:
      if(smtxt(4,line,il,'SCRATCHDSN ='))goto 140
      goto 800
 
 
c     title = 'This is a title';                     #     ' '
  110 if(smtxt(4,line,il,';'))goto 999
      call getstring(4,line,il,i1,i2)
      if(i2.eq.-1)goto 800
      title=line(i1:i2)
      goto 700
 
c     version = {1|2};                               #     version=2
  115 if(smtxt(4,line,il,';'))goto 999
      call getint(4,version,line,il,status)
      if(status.gt.0)goto 800
      if(version.ne.1.and.version.ne.2)goto 830
      goto 700
 
c     converge = [wgn|rfi|gn|rpc < x [or ...]];      #     wgn<0.1
  120 if(.not.smtxt(4,line,il,'='))goto 800
      if(smtxt(4,line,il,';'))goto 999
c     Disable defaults:
      call put(-1.d0,convcrit,4)
  125 if(smtxt(4,line,il,'WGN<'))then
         i=1
      elseif(smtxt(4,line,il,'RFI<'))then
         i=2
      elseif(smtxt(4,line,il,'GN<'))then
         i=3
      elseif(smtxt(4,line,il,'RPC<'))then
         i=4
      else
         goto 800
      endif
      if(smtxt(4,line,il,'='))continue
      if(convcrit(i).gt.0.d0)goto 800
      call getdp(4,convcrit(i),line,il,status)
      if(status.gt.0)goto 800
      if(smtxt(4,line,il,'OR'))goto 125
      goto 700
 
c     scratchfile = filename for scratch data;       #     ' '
  140 if(smtxt(4,line,il,';'))goto 999
      call getstring(4,line,il,i1,i2)
      if(i2.eq.-1)goto 800
      scrfile=line(i1:i2)
      goto 700
 
c     gridfile = filename for grid search output;       #     ' '
  150 if(smtxt(4,line,il,';'))goto 999
      call getstring(4,line,il,i1,i2)
      if(i2.eq.-1)goto 800
      gridfile=line(i1:i2)
      goto 700
 
c     normalized weight = varname;                 #     (no weights)
  160 if(smtxt(4,line,il,';'))goto 999
      call getname(4,line,il,i1,i2)
      if(i2.lt.i1)goto 800
      if(i2-i1.gt.7)goto 800
      call char8match(line(i1:i2),oldname,nx1,wghtpos)
      if(wghtpos.eq.0)goto 810
c     Negative position numbers indicate normalized weight
      wghtpos=-wghtpos
      goto 700
 
c     weight = varname;                            #     (no weights)
  170 if(smtxt(4,line,il,';'))goto 999
      call getname(4,line,il,i1,i2)
      if(i2.lt.i1)goto 800
      if(i2-i1.gt.7)goto 800
      call char8match(line(i1:i2),oldname,nx1,wghtpos)
      if(wghtpos.eq.0)goto 810
c     Positive position numbers indicate non-normalized weight
      goto 700
 
c     step range = n [to n];                        #     -10 to 4
  180 if(smtxt(4,line,il,';'))goto 999
      call getint(4,stprng(1),line,il,status)
      if(status.gt.0)goto 800
      if(.not.smtxt(4,line,il,'TO'))then
         stprng(2)=stprng(1)
         stprng(1)=-stprng(1)
      else
         call getint(4,stprng(2),line,il,status)
         if(status.gt.0)goto 800
      endif
      if(stprng(1).gt.0.or.stprng(2).lt.0)goto 800
      goto 700
 
c     save step;                            #     no
  190 call semiyesno(stprng(3),4,line,il,ipt,jpt)
      goto 999
 
c     file info level = n;                           #     4
  200 if(smtxt(4,line,il,';'))goto 999
      call getint(4,output(1),line,il,status)
      if(status.gt.0)goto 800
      ipt=output(1)
      detail=(ipt.ge.5.or.jpt.ge.5)
      goto 700
 
c     screen info level = n;                         #     3
  210 if(smtxt(4,line,il,';'))goto 999
      call getint(4,output(2),line,il,status)
      if(status.gt.0)goto 800
      jpt=output(2)
      detail=(ipt.ge.5.or.jpt.ge.5)
      goto 700
 
c     iterations = n;                                #     40
  220 if(smtxt(4,line,il,';'))goto 999
      call getint(4,maxiter,line,il,status)
      if(status.gt.0)goto 800
      goto 700
 
c     observations = n;                              #     0 (=all)
  230 if(smtxt(4,line,il,';'))goto 999
      call getint(4,nobs,line,il,status)
      if(status.gt.0)goto 800
      if(nobs.gt.datanobs)goto 820
      goto 700
 
c     Hessian matrix;                       #     no
  250 call semiyesno(output(4),4,line,il,ipt,jpt)
      goto 999
 
c     variance-covariance matrix;           #     no
  260 call semiyesno(output(5),4,line,il,ipt,jpt)
      goto 999
 
c     correlation matrix;                   #     no
  270 call semiyesno(output(6),4,line,il,ipt,jpt)
      goto 999
 
c     table format;                         #     no
  280 call semiyesno(output(7),4,line,il,ipt,jpt)
      goto 999
 
c     starting value format;                #     no
  290 call semiyesno(output(8),4,line,il,ipt,jpt)
      goto 999
 
c     Huber;                                #     no
  300 call semiyesno(output(9),4,line,il,ipt,jpt)
      goto 999
 
c     numerical search;                     #     no
  305 call semiyesno(numsearch,4,line,il,ipt,jpt)
c     Numerical search always results in numerical standard errors:
      if(numsearch.eq.1)output(10)=1
      goto 999
 
c     numerical standard errors;            #     no
  310 call semiyesno(output(10),4,line,il,ipt,jpt)
      goto 999
 
c     check99999;
  330 call semiyesno(flag,4,line,il,ipt,jpt)
      check99999=(flag.eq.1)
      goto 999
 
c     ensure positive definite;
  340 call semiyesno(posdef,4,line,il,ipt,jpt)
      goto 999
 
c     option IEEE warnings;
c     IEEE inexact/underflow/overflow warnings may only be on a SUN.
c 350 call semiyesno(flag,4,line,il,ipt,jpt)
c     ieee=(flag.eq.1)
c     goto 999
 
c     sparsity;   # yes
  360 call semiyesno(sparsity,4,line,il,ipt,jpt)
      goto 999
 
c     maximum specification space = 1000;
  410 if(smtxt(4,line,il,';'))goto 999
      call getint(4,maxmeno,line,il,status)
      if(status.gt.0)goto 800
      if(maxmeno.lt.0)goto 830
      goto 700
 
c     maximum model space = 5000;
  420 if(smtxt(4,line,il,';'))goto 999
      call getint(4,maxnmapmoel,line,il,status)
      if(status.gt.0)goto 800
      if(maxnmapmoel.lt.0)goto 830
      goto 700
 
c     maximum hazard baseline space = 4000;
  430 if(smtxt(4,line,il,';'))goto 999
      call getint(4,maxds,line,il,status)
      if(status.gt.0)goto 800
      if(maxds.lt.0)goto 830
      goto 700
 
c     maximum scratch data space = max(5000,2*xdimmax);
  440 if(smtxt(4,line,il,';'))goto 999
      call getint(4,vdimmax,line,il,status)
      if(status.gt.0)goto 800
      if(vdimmax.lt.0)goto 830
      goto 700
 
c     maximum number of frequency categories = 20;
  450 if(smtxt(4,line,il,';'))goto 999
      call getint(4,maxfreqcat,line,il,status)
      if(status.gt.0)goto 800
      if(maxfreqcat.lt.0)goto 830
      goto 700
 
c     maximum number of residual draws = 100;
  460 if(smtxt(4,line,il,';'))goto 999
      call getint(4,maxdraws,line,il,status)
      if(status.gt.0)goto 800
      if(maxdraws.lt.0)goto 830
      goto 700
 
c     maximum number of summary statistics = 100;
  470 if(smtxt(4,line,il,';'))goto 999
      call getint(4,maxnfreq,line,il,status)
      if(status.gt.0)goto 800
      if(maxnfreq.lt.0)goto 830
      goto 700
 
c     maximum number of hazard baseline nodes = 100;
  480 if(smtxt(4,line,il,';'))goto 999
      call getint(4,maxnode,line,il,status)
      if(status.gt.0)goto 800
      if(maxnode.lt.0)goto 830
      goto 700
 
c     maximum number of reference numbers = 300;
  490 if(smtxt(4,line,il,';'))goto 999
      call getint(4,maxref,line,il,status)
      if(status.gt.0)goto 800
      if(maxref.lt.0)goto 830
      goto 700
 
c     maximum number of user-defined constants = 500;
  510 if(smtxt(4,line,il,';'))goto 999
      call getint(4,maxxnode,line,il,status)
      if(status.gt.0)goto 800
      if(maxxnode.lt.0)goto 830
      goto 700
 
c     maximum number of hazard model splines = 0;
  520 if(smtxt(4,line,il,';'))goto 999
      call getint(4,maxmspl,line,il,status)
      if(status.gt.0)goto 800
      if(maxmspl.lt.0)goto 830
      goto 700
 
c     Undocumented options:
 
c     useid = | <= n;    # Undocumented; for debug purposes only
  600 if(smtxt(4,line,il,';'))goto 999
      equal=smtxt(4,line,il,'=')
      if(.not.equal.and..not.smtxt(4,line,il,'<='))goto 800
      call getint(4,useid,line,il,status)
      if(status.gt.0)goto 800
      if(.not.equal)useid=-useid
      goto 700
 
c     useobs = | <= n;   # Undocumented; for debug purposes only
  610 if(smtxt(4,line,il,';'))goto 999
      equal=smtxt(4,line,il,'=')
      if(.not.equal.and..not.smtxt(4,line,il,'<='))goto 800
      call getint(4,useobs,line,il,status)
      if(status.gt.0)goto 800
      if(.not.equal)useobs=-useobs
      goto 700
 
c     gradientcheck;                        #     no
  620 call semiyesno(output(3),4,line,il,ipt,jpt)
      goto 999
 
c     proficiency = {novice | medium | expert | guru}; # novice
  630 if(smtxt(4,line,il,';'))goto 999
      if(smtxt(4,line,il,'NOVICE'))then
         prof=1
      elseif(smtxt(4,line,il,'MEDIUM'))then
         prof=2
      elseif(smtxt(4,line,il,'EXPERT'))then
         prof=3
      elseif(smtxt(4,line,il,'GURU'))then
         prof=4
      else
         goto 800
      endif
      goto 700
 
c     Make sure the option is followed by a semicolon.
  700 if(.not.smtxt(4,line,il,';'))goto 800
      goto 999
 
  800 if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,801)
         write(8,802)
         write(8,803)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         write(6,801)
         write(6,802)
         write(6,803)
      endif
  801 format('ERROR encountered in an OPTION statement.  The ',
     *   'correct syntax is:',//,
     *   '   option title = ''string'';',/,
     *   '   option converge = {wgn | rfi | gn | rpc} < x [or ...];',/,
     *   '   option scratchfile = ''filename'';',/,
     *   '   option gridfile = ''filename'';',/,
     *   '   option [normalized] weight = varname;',/,
     *   '   option step range = n [to n];')
  802 format('   option save step;',/,
     *   '   option file info level = n;',/,
     *   '   option screen info level = n;',/,
     *   '   option iterations = n;',/,
     *   '   option observations = n;',/,
     *   '   option Hessian matrix;',/,
     *   '   option variance-covariance matrix;',/,
     *   '   option correlation matrix;',/,
     *   '   option table format;',/,
     *   '   option starting value format;',/,
     *   '   option huber;',/,
     *   '   option numerical search;',/,
     *   '   option numerical standard errors;',/,
c    *   '   option IEEE warnings;',/,
     *   '   option ensure positive definite = {yes|no};',/,
     *   '   option check99999 = {yes|no};')
  803 format('   option maximum specification space = n;',/,
     *   '   option maximum model space = n;',/,
     *   '   option maximum hazard baseline space = n;',/,
     *   '   option maximum scratch data space = n;',/,
     *   '   option maximum number of frequency categories = n;',/,
     *   '   option maximum number of residual draws = n;',/,
     *   '   option maximum number of frequency tables = n;',/,
     *   '   option maximum number of hazard baseline nodes = n;',/,
     *   '   option maximum number of reference numbers = n;',/,
     *   '   option maximum number of regressors = n;',/,
     *   '   option maximum number of user-defined constants = n;',/,
     *   '   option maximum number of hazard model splines = n;')
      stop
  810 il=max(0,il-(i2-i1+1))
      if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,811)line(i1:i2)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         write(6,811)line(i1:i2)
      endif
  811 format(/,'ERROR: variable ''',a,''' not found among level 1 ',
     *   'variables.')
      stop
  820 il=max(0,il-intlen(nobs))
      if(ipt.gt.0)then
         call caretline(8,line,il)
         call writeaia(8,72,1,'ERROR:  the data only contain ',
     *      datanobs,' observations.')
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         call writeaia(6,72,1,'ERROR:  the data only contain ',
     *      datanobs,' observations.')
      endif
      stop
  830 if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,'(a)')'ERROR:  illegal value.'
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         write(6,'(a)')'ERROR:  illegal value.'
      endif
      stop
  999 return
      end
 
