      subroutine simhaz(nngam,nds,censor,nsb,nl,nu,refspl,nodepos,
     *   numnodes,spltype,lnl1,lnl2,gamma,d1gam,d2gam,d1xbeta,
     *   d2xbeta,xbeta,s,ds,ssg,time,tlag,xnode,ifdrv,ipt,jpt)
 
c     Subroutine to compute a very simple hazard likelihood:
c     it computes the log-likelihood and its derivatives for one
c     spell, without mixture.
c
c     For noncensored spells, time(abs(nl)) and time(nu) contain the lower
c     and upper event window marks, respectively.  If nl<0, the lower
c     event window mark was inserted, and there is thus no corresponding
c     delimiter for time-varying covariates.  For censored spells, nl=nu
c     and time(nu) is the spell duration.
c
c     This routine "splits the product" in the case of a non-censored
c     episode.  The likelihood of a non-censored episode consists
c     of the product of [S(i)/S(i-1)]**G(i) for i=1,nl and
c     1 minus a similar expression for i=nl+1,nu.  Both terms of this
c     additive form are saved in order to later add mixture.
c
c     The proportional shift of the hazard function, Beta'X is not
c     computed inside this routine, but passed in as an argument.
c     Derivatives of lnl1 and lnl2 are passed back wrt gammas and xbeta.
c
c     Constantijn Panis, 17 October 2001
c     Last change: initialize d1xbeta to zero for nsb>nu
 
      implicit none
      integer nngam,nds,censor,nsb,nl,nu,refspl(nds),nodepos(*),
     *   numnodes(*),spltype(*),ipt,jpt,i,j,ii,absnl
      double precision lnl1,lnl2,gamma(nngam),d1gam(nngam),
     *   d2gam(nngam),d1xbeta(nu),d2xbeta(nu),xbeta(nu),
     *   s(0:nu),ds(0:nu,nngam),ssg(nu),dpvecsum
      real time(0:nu),tlag(nds),xnode(*)
      logical ifdrv
 
      absnl=abs(nl)
 
c     First convert xbeta into exp(xbeta):
      call expvector(xbeta,xbeta,nsb)
 
c     Compute baseline survivor function:
      call spline(nds,nu,refspl,nodepos,numnodes,spltype,nngam,
     *   time,gamma,s,ifdrv,ds,xnode,tlag,ipt,jpt)
 
c     Compute  xbeta(i)*log(s(i)/s(i-1),  i=1,nu, accounting for
c     the possibility that a time mark was inserted
      ii=1
      do 300 i=1,nu
         ssg(i) = xbeta(ii)*(s(i)-s(i-1))
         if(i.ne.-nl)ii=ii+1
  300 continue
 
      if(censor.eq.1)then
         lnl1 = dpvecsum(ssg,nu)
         lnl2 = 0.d0
      else
         lnl1 = dpvecsum(ssg,absnl)
         lnl2 = lnl1+dpvecsum(ssg(absnl+1),nu-absnl)
      endif
 
      if(.not.ifdrv)goto 999
 
      call zerodp(nngam,d1gam)
c     Compute derivatives wrt XBetas and gammas.  Note that there may
c     may have been time marks beyond the upper event window marks, so
c     that nsb>nu.  Those derivatives are correctly returned as zero.
      if(censor.eq.1)then
c        Note that nl=nu>0 if censored
         do 410 i=1,nu
            do 420 j=1,nngam
               d1gam(j) = d1gam(j) + xbeta(i)*(ds(i,j)-ds(i-1,j))
  420       continue
            d1xbeta(i) = ssg(i)
  410    continue
c        This line fixes Oystein Kravdal's bug that led to the recall of 1.03.
         if(nu.lt.nsb)call zerodp(nsb-nu,d1xbeta(nu+1))
      else
         call zerodp(nsb,d1xbeta)
         call zerodp(nsb,d2xbeta)
         do 430 i=1,absnl
            do 440 j=1,nngam
               d1gam(j) = d1gam(j) + xbeta(i)*(ds(i,j)-ds(i-1,j))
  440       continue
            d1xbeta(i)=d1xbeta(i)+ssg(i)
  430    continue
         call copy8(d1gam,d2gam,nngam)
         call copy8(d1xbeta,d2xbeta,absnl)
c        Careful! Very tricky to access correct element of xbeta!
         ii=absnl+1
         if(nl.lt.0)ii=absnl
         do 450 i=absnl+1,nu
            do 460 j=1,nngam
               d2gam(j) = d2gam(j) + xbeta(ii)*(ds(i,j)-ds(i-1,j))
  460       continue
            d2xbeta(ii)=d2xbeta(ii)+ssg(i)
            ii=ii+1
  450    continue
      endif
 
  999 return
      end
 
