      subroutine suml2lnl(nest,thisnest,neh,nel,nep,nec,neb,nen,nepo,
     *   neml,nneml,nfullplus,allint6,numdist,sumintdim,sumdim2,sumdim4,
     *   sumbigintdim,dimii,dimlam,intdist,intdim,intdimc,posi1,posi2,
     *   posi4,bigposi2,ipt,jpt,distdim,distyp,ndraw,thisdraw,dimused,
     *   like,lnl,ddfullbeta,dfullbeta,ddxbh,dxbh,ddxbc,dxbc,ddsiga6a6,
     *   dsiga6a6,ddsigl4l6,dsigl4l6,ddtau,dtau,ddxblp,dxblp,ddsiga4a4,
     *   dsiga4a4,dda5,da5,ddp5,dp5,ddxbml,dxbml,ddlam,dlam,ddcmui,
     *   dcmui,cdm,ddcdm,dcdmdsigii,sigii,ddsigii,dsigii,ifdrv,iflam,
     *   iftau,iid4)
 
c     Subroutine to take the log of a likelihood at nest and
c     accumulate it to the log-likelihood at nest-1.
c
c     Note that derivatives of the likelihood are in dd... arrays and
c     of the accumulated log-likelihood in d... arrays.
c
c     Constantijn Panis, 11 November 2002
c     Last change: Poisson
 
      implicit none
      integer nest,thisnest,neh,nel,nep,nec,neb,nen,nepo,neml,nneml,
     *   nfullplus,allint6,numdist,sumintdim,sumdim2,sumdim4,
     *   sumbigintdim,dimii,dimlam,intdim,intdimc,posi1,posi2,posi4,
     *   bigposi2,ipt,jpt,intdist(thisnest),distdim(numdist),
     *   distyp(numdist),ndraw(numdist),thisdraw(numdist),
     *   dimused(sumintdim),status
      double precision cdm(sumdim2),ddcdm(sumdim2),
     *   like(thisnest),lnl(0:thisnest),
     *   dfullbeta(nfullplus,0:thisnest),ddfullbeta(nfullplus,thisnest),
     *   dlam(dimlam,0:thisnest-1),ddlam(dimlam,thisnest),
     *   dcmui(sumbigintdim,0:thisnest-1),ddcmui(sumbigintdim,thisnest),
     *   dsiga6a6(nec*nec,0:thisnest),ddsiga6a6(nec*nec,thisnest),
     *   sigii(dimii),dsigii(dimii,0:thisnest-1),
     *   ddsigii(dimii,thisnest-1),dcdmdsigii(sumdim4),
     *   dxbh(neh,0:thisnest),ddxbh(neh,thisnest),
     *   dxbc(nec,0:thisnest),ddxbc(nec,thisnest),
     *   dxblp(nel+nep,0:thisnest),ddxblp(nel+nep,thisnest),
     *   dsigl4l6(numdist*nep*nec,0:thisnest-1),
     *   ddsigl4l6(numdist*nep*nec,thisnest),
     *   dtau(2*(nel+nep),0:thisnest),ddtau(2*(nel+nep),thisnest),
     *   dsiga4a4(nep*nep,0:thisnest),ddsiga4a4(nep*nep,thisnest),
     *   da5(nen,0:thisnest),dda5(nen,thisnest),
     *   dp5(neb+nen+nepo,0:thisnest),ddp5(neb+nen+nepo,thisnest),
     *   dxbml(nneml,0:thisnest),ddxbml(nneml,thisnest),temp,minexp
      logical ifdrv,iflam(numdist),iftau,iid4,ifany
 
c     Convert the log-likelihood into the likelihood and compute
c     the weighted likelihood of this heterogeneity draw:
      if(like(nest).le.0.d0)then
         lnl(nest-1)=lnl(nest-1)+minexp()
c        Nothing is done with derivatives, which really should also
c        be minus infinity.  What's relevant, though, is that this
c        likelihood does not contribute more to some than to other
c        derivatives.
         goto 200
      endif
 
c     Accumulate the log-likelihood:
      lnl(nest-1)=lnl(nest-1)+dlog(like(nest))
 
      if(.not.ifdrv)goto 200
      temp=1.d0/like(nest)
 
c     Convert derivatives of the likelihood into of the log-likelihood
c     and accumulate.
c     Accumulate derivatives wrt hazard or finite mixture parameters:
      call add(ddfullbeta(1,nest),dfullbeta(1,nest-1),temp,nfullplus)
      if(neh.gt.0)call add(ddxbh(1,nest),dxbh(1,nest-1),temp,neh)
 
c     Accumulate logit/probit derivatives:
      if(nel+nep.gt.0)then
         call add(ddxblp(1,nest),dxblp(1,nest-1),temp,nep+nel)
         if(iftau)call add(ddtau(1,nest),dtau(1,nest-1),temp,
     *      2*(nel+nep))
         if(.not.iid4)call add(ddsiga4a4(1,nest),dsiga4a4(1,nest-1),
     *      temp,nep*nep)
      endif
 
c     Accumulate continuous derivatives, if needed:
      if(nec.gt.0.and.allint6.eq.1)then
         call add(ddsiga6a6(1,nest),dsiga6a6(1,nest-1),temp,nec*nec)
         call add(ddxbc(1,nest),dxbc(1,nest-1),temp,nec)
         if(nep.gt.0.and..not.iid4)then
            call add(ddsigl4l6(1,nest),dsigl4l6(1,nest-1),temp,
     *         nep*nec*numdist)
         endif
      endif
 
c     Accumulate binomial, negative binomial, and Poisson derivatives:
      if(neb+nen+nepo.gt.0)then
         call add(ddp5(1,nest),dp5(1,nest-1),temp,neb+nen+nepo)
         if(nen.gt.0)call add(dda5(1,nest),da5(1,nest-1),temp,nen)
      endif
 
c     Accumulate multinomial logit derivatives:
      if(nneml.gt.0)then
         call add(ddxbml(1,nest),dxbml(1,nest-1),temp,nneml)
      endif
 
c     Subroutine sumlnl2l unraveled derivatives wrt integrated residuals
c     eps into wrt the currently relevant (nest) Lambda and Cholesky
c     decomposed matrix.  Accumulate the former and unravel the latter
c     into wrt sigii.
c     Note that only ddlam related to levels >=nest should be
c     accumulated, but the rest is still zero.  This is much easier.
c     First accumulate derivatives wrt sigii from levels >nest;
c     for convenience do them all, as those for level nest are zero
c     in ddsigii(1,nest).
      if(nest.lt.thisnest)call add(ddsigii(1,nest),dsigii(1,nest-1),
     *   temp,dimii)
      if(ifany(iflam,numdist))
     *   call add(ddlam(1,nest),dlam(1,nest-1),temp,dimlam)
      call add(ddcmui(1,nest),dcmui(1,nest-1),temp,sumbigintdim)
      if(thisdraw(intdist(nest)).eq.0)goto 200
      if(mod(distyp(intdist(nest)),10).ne.4)then
c        Subroutine sumlnl2l unraveled finite mixture distributions
c        all the way to wrt fullbeta.
         call mult(ddcdm(posi2),temp,ddcdm(posi2),intdim*intdim)
         call compcdm(2,intdist(nest),intdim,intdimc,posi1,posi2,posi4,
     *      bigposi2,sumintdim,sumdim2,sumdim4,dimii,thisdraw,ndraw,
     *      distdim,dimused,ipt,jpt,status,sigii,dsigii(1,nest-1),
     *      cdm,ddcdm,dcdmdsigii,ifdrv)
      endif
 
c     Re-set derivatives to zero.  Control may be passed here when the
c     conditional likelihood underflows to zero, in which case derivatives
c     are not accumulated but still need to be re-set to zero.
  200 like(nest)=0
      if(.not.ifdrv)goto 999
      call zerodp(nfullplus,ddfullbeta(1,nest))
      if(neh.gt.0)call zerodp(neh,ddxbh(1,nest))
      if(nel+nep.gt.0)then
         call zerodp(nel+nep,ddxblp(1,nest))
         if(iftau)call zerodp(2*(nel+nep),ddtau(1,nest))
         if(.not.iid4)call zerodp(nep*nep,ddsiga4a4(1,nest))
      endif
      if(nec.gt.0.and.allint6.eq.1)then
         call zerodp(nec*nec,ddsiga6a6(1,nest))
         call zerodp(nec,ddxbc(1,nest))
         if(nep.gt.0.and..not.iid4)then
            call zerodp(nep*nec*numdist,ddsigl4l6(1,nest))
         endif
      endif
      if(neb+nen+nepo.gt.0)call zerodp(neb+nen+nepo,ddp5(1,nest))
      if(nen.gt.0)call zerodp(nen,dda5(1,nest))
      if(neml.gt.0)call zerodp(nneml,ddxbml(1,nest))
      if(nest.lt.thisnest)call zerodp(dimii,ddsigii(1,nest))
      if(ifany(iflam,numdist))call zerodp(dimlam,ddlam(1,nest))
      call zerodp(sumbigintdim,ddcmui(1,nest))
      if(thisdraw(intdist(nest)).eq.0)goto 999
      if(mod(distyp(intdist(nest)),10).ne.4)call
     *   zerodp(intdim*intdim,ddcdm(posi2))
 
  999 return
      end
 
 
