      subroutine timemarks(mdllvl,levelmax,maxxnode,nold,nxold,namepntr,
     *   xdim,vdimmax,vdim,nsb,nl,nu,nbdim,nb,censor,durvar,timevar,
     *   durvars,ipt,jpt,struc,iobs,id,branch,xpntr,nb2,xpntrfull,
     *   xnode,x,v,oldname)
 
c     Subroutine to place time marks in v.  If there are no time-varying
c     variables, the time marks are zero, durvar1, and durvar2.  If there
c     are time-varying variables, they follow from the timevar variable,
c     with possibly another mark inserted by durvar1. Also check that
c     the highest time mark is at least as high as durvar2.  If there is
c     no time mark at durvar2, reduce the next-highest mark to durvar2.
c     Time marks above durvar2 are irrelevant and eliminated.
c     Also check that
c        - time marks are (nonstrictly) increasing;
c        - the event window has nonzero width if the spell is censored.
c
c     Arguments:
c
c     nsb [in] Number of SubBranches (number of intervals with time-varying
c              covariates).
c     nu  [out] number of time marks (not counting zero)
c               Note that the nu-th time mark is durvar(2)
c     nl  [out] Pointer at durvar(1): the nl-th time mark (not counting
c               zero) in v is the lower bound of the event window.
c               If censor=1 then nl=nu
c                  censor=0 and event window was already in data, nl>0
c                  censor=0 and lower bound was inserted, nl<0, and the
c                           lower bound is time mark abs(nl).
c
c     Constantijn Panis, 13 November 1997
 
      implicit none
      integer mdllvl,levelmax,maxxnode,nold,nxold(levelmax),
     *   namepntr(levelmax),xdim,
     *   vdimmax,vdim,nsb,nl,nu,nbdim,nb(nbdim,2),censor,durvar,timevar,
     *   ipt,jpt,struc,iobs,id,branch(mdllvl+1),xpntr(2:levelmax),
     *   nb2,xpntrfull(2:levelmax,nb2),i,nn,match,status,trm,xpos
      real xnode(maxxnode),x(*),v(vdimmax),durvars(2)
      logical realordered
      character oldname(nold)*8,string*60,idstring*60
 
c     Make sure there is enough scratch space:
      if(vdim+nsb+2.gt.vdimmax)call
     *   maxexceed('vdimmax',vdim+nsb+2,vdimmax,ipt,jpt)
 
      if(durvar.ne.0)then
         call evalexpr(2,maxxnode,xdim,durvar,nold,levelmax,
     *      mdllvl,nbdim,nb,branch,xpntr,nxold,namepntr,xnode,x,
     *      durvars,oldname,struc,iobs,id,ipt,jpt,status)
         if(status.gt.0)stop
         if(durvars(1).lt.0.)goto 850
      endif
 
c     The first time mark is always zero (and not counted in nl, nu):
      vdim=vdim+1
      v(vdim)=0.
      if(timevar.eq.0)then
c        There are no time-varying variables, no time marks at mdllvl+1
c        (nsb=1 per figdata1.)
         if(durvar.eq.0)call bug(ipt,jpt)
         v(vdim+1)=durvars(1)
         if(censor.eq.0)then
            v(vdim+2)=durvars(2)
            if(durvars(1).ge.durvars(2))goto 820
            nl=-1
            nu=2
         else
            if(durvars(1).ne.durvars(2))goto 830
            nl=1
            nu=1
         endif
      else
c        Time marks are in data; copy them into v and adjust (truncate
c        and/or insert) below.
         if(mdllvl.eq.1)then
c           Time marks are at level 2
            do 100 i=1,nsb
               nn=xpntrfull(2,i)+timevar
               v(vdim+i)=x(nn)
  100       continue
         else
c           Time marks are at level mdllvl+1
            nn=xpos(mdllvl+1,nbdim,nb,branch,xpntr(2),nxold,timevar)
            do 110 i=1,nsb
               v(vdim+i)=x(nn)
               nn=nn+nxold(mdllvl+1)
  110       continue
         endif
         if(durvar.eq.0)then
c           Duration is implicitly given by timemarks
            nl=nsb-1
            nu=nsb
            durvars(1)=v(vdim+nl)
            durvars(2)=v(vdim+nu)
         else
c           Upper event window variable must be inside time marks range.
            if(durvars(2).gt.v(vdim+nsb))goto 840
c           Make sure durvar2 is a time mark; discard all time marks
c           beyond durvar2.
            do 210 i=nsb,0,-1
               if(durvars(2).eq.v(vdim+i))then
                  nu=i
                  goto 220
               elseif(durvars(2).gt.v(vdim+i))then
                  nu=i+1
                  v(vdim+nu)=durvars(2)
                  goto 220
               endif
  210       continue
            call bug(ipt,jpt)
  220       continue
c           Lower event window variable may insert an additional time mark.
            if(censor.eq.0)then
               if(durvars(1).ge.durvars(2))goto 820
               call realmatch(durvars(1),v(vdim),nu+1,match)
               if(match.eq.0)then
c                 Insert a time mark at durvar1 (nl<0):
                  nu=nu+1
                  do 230 i=nu-1,0,-1
                     if(v(vdim+i).gt.durvars(1))then
                        v(vdim+i+1)=v(vdim+i)
                     else
                        v(vdim+i+1)=durvars(1)
                        nl=-(i+1)
                        goto 240
                     endif
  230             continue
                  call bug(ipt,jpt)
  240             continue
               else
c                 durvar1 is nl-th time mark (nl>0)
                  nl=match-1
               endif
            else
               if(durvars(1).ne.durvars(2))goto 830
               nl=nu
            endif
         endif
      endif
 
c     Check that time marks are ordered and that event windows have
c     nonzero width.
      if(.not.realordered(v(vdim),nu+1))goto 800
      if(censor.eq.0.and.v(vdim+abs(nl)).eq.v(vdim+nu))goto 810
      vdim=vdim+nu
      goto 999
 
  800 string=idstring(struc,id,iobs,mdllvl,branch)
      if(ipt.gt.0)then
         call wrapstring(8,0,72,0,1,'ERROR: the first time mark is '//
     *      'negative or time marks are not increasing in a hazard '//
     *      'spell ('//string(1:trm(string))//'):')
         write(8,'(8f8.3)')(v(vdim+i),i=1,nu)
      endif
      if(jpt.gt.0)then
         call wrapstring(6,0,72,0,1,'ERROR: the first time mark is '//
     *      'negative or time marks are not increasing in a hazard '//
     *      'spell ('//string(1:trm(string))//'):')
         write(6,'(8f8.3)')(v(vdim+i),i=1,nu)
      endif
      stop
  810 string=idstring(struc,id,iobs,mdllvl,branch)
      if(ipt.gt.0)then
         call wrapstring(8,0,72,0,1,'ERROR: the width of the event '//
     *      'window in a noncensored hazard spell is zero ('//
     *      string(1:trm(string))//').  The time marks are:')
         write(8,'(8f8.3)')(v(vdim+i),i=1,nu)
      endif
      if(jpt.gt.0)then
         call wrapstring(6,0,72,0,1,'ERROR: the width of the event '//
     *      'window in a noncensored hazard spell is zero ('//
     *      string(1:trm(string))//').  The time marks are:')
         write(6,'(8f8.3)')(v(vdim+i),i=1,nu)
      endif
      stop
  820 string=idstring(struc,id,iobs,mdllvl,branch)
      if(ipt.gt.0)then
         call wrapstring(8,0,72,0,1,'ERROR: the event window of a '//
     *      'non-censored hazard spell has negative or zero width ('//
     *      string(1:trm(string))//'):')
         i=durvar
         call writeexpr(111,8,maxxnode,xdim,levelmax,nbdim,i,nb,branch,
     *      xpntr,1,nold,nxold,namepntr,-1,durvars(1),xnode,x,oldname,
     *      struc)
         call writeexpr(111,8,maxxnode,xdim,levelmax,nbdim,i,nb,branch,
     *      xpntr,1,nold,nxold,namepntr,-1,durvars(2),xnode,x,oldname,
     *      struc)
      endif
      if(jpt.gt.0)then
         call wrapstring(6,0,72,0,1,'ERROR: the event window of a '//
     *      'non-censored hazard spell has negative or zero width ('//
     *      string(1:trm(string))//'):')
         i=durvar
         call writeexpr(111,6,maxxnode,xdim,levelmax,nbdim,i,nb,branch,
     *      xpntr,1,nold,nxold,namepntr,-1,durvars(1),xnode,x,oldname,
     *      struc)
         call writeexpr(111,6,maxxnode,xdim,levelmax,nbdim,i,nb,branch,
     *      xpntr,1,nold,nxold,namepntr,-1,durvars(2),xnode,x,oldname,
     *      struc)
      endif
      stop
  830 string=idstring(struc,id,iobs,mdllvl,branch)
      if(ipt.gt.0)then
         call wrapstring(8,0,72,0,1,'ERROR: a hazard spell is '//
     *      'censored ('//string(1:trm(string))//'), but the '//
     *      'duration variables are not equal to each other: ')
         i=durvar
         call writeexpr(111,8,maxxnode,xdim,levelmax,nbdim,i,nb,branch,
     *      xpntr,1,nold,nxold,namepntr,-1,durvars(1),xnode,x,oldname,
     *      struc)
         call writeexpr(111,8,maxxnode,xdim,levelmax,nbdim,i,nb,branch,
     *      xpntr,1,nold,nxold,namepntr,-1,durvars(2),xnode,x,oldname,
     *      struc)
      endif
      if(jpt.gt.0)then
         call wrapstring(6,0,72,0,1,'ERROR: a hazard spell is '//
     *      'censored ('//string(1:trm(string))//'), but the '//
     *      'duration variables are not equal to each other: ')
         i=durvar
         call writeexpr(111,6,maxxnode,xdim,levelmax,nbdim,i,nb,branch,
     *      xpntr,1,nold,nxold,namepntr,-1,durvars(1),xnode,x,oldname,
     *      struc)
         call writeexpr(111,6,maxxnode,xdim,levelmax,nbdim,i,nb,branch,
     *      xpntr,1,nold,nxold,namepntr,-1,durvars(2),xnode,x,oldname,
     *      struc)
      endif
      stop
  840 string=idstring(struc,id,iobs,mdllvl,branch)
      if(ipt.gt.0)then
         write(8,841)
         call wrapstring(8,0,72,0,1,'('//string(1:trm(string))//')')
         call writea2da(8,72,1,'The duration variables are ',-8,
     *      dble(durvars(1)),' and ',-8,dble(durvars(2)),
     *      '; the time marks are')
         write(8,'(8f8.3)')(v(vdim+i),i=1,nsb)
      endif
      if(jpt.gt.0)then
         write(6,841)
         call wrapstring(6,0,72,0,1,'('//string(1:trm(string))//')')
         call writea2da(6,72,1,'The duration variables are ',-8,
     *      dble(durvars(1)),' and ',-8,dble(durvars(2)),
     *      '; the time marks are')
         write(6,'(8f8.3)')(v(vdim+i),i=1,nsb)
      endif
  841 format(/,'ERROR: the end of a hazard spell is beyond the period ',
     *   'covered by time',/,'marks, i.e., no values on time-varying ',
     *   'variables are available for',/,'the end of the spell.')
      stop
  850 string=idstring(struc,id,iobs,mdllvl,branch)
      if(ipt.gt.0)then
         call wrapstring(8,0,72,0,1,'ERROR: hazard spell duration '//
     *      'variables must be positive.  The duration variables in '//
     *      string(1:trm(string))//' are')
         i=durvar
         call writeexpr(111,8,maxxnode,xdim,levelmax,nbdim,i,nb,branch,
     *      xpntr,1,nold,nxold,namepntr,-1,durvars(1),xnode,x,oldname,
     *      struc)
         call writeexpr(111,8,maxxnode,xdim,levelmax,nbdim,i,nb,branch,
     *      xpntr,1,nold,nxold,namepntr,-1,durvars(2),xnode,x,oldname,
     *      struc)
      endif
      if(jpt.gt.0)then
         call wrapstring(6,0,72,0,1,'ERROR: hazard spell duration '//
     *      'variables must be positive.  The duration variables in '//
     *      string(1:trm(string))//' are')
         i=durvar
         call writeexpr(111,6,maxxnode,xdim,levelmax,nbdim,i,nb,branch,
     *      xpntr,1,nold,nxold,namepntr,-1,durvars(1),xnode,x,oldname,
     *      struc)
         call writeexpr(111,6,maxxnode,xdim,levelmax,nbdim,i,nb,branch,
     *      xpntr,1,nold,nxold,namepntr,-1,durvars(2),xnode,x,oldname,
     *      struc)
      endif
      stop
  999 return
      end
