      subroutine arraysizes(maxcc,datadim,levelmax,numvar,maxnb2,
     *   maxxdim,maxnbdim,nx1,xmean,xstd,xmin,xmax,x,x1copy,deduct,nx,
     *   namepntr,nbr,xbase,xpntr,nb,nblong,nbmax,branch,scratch,struc,
     *   ds,datastructure,datalevel,strucfreq,rect,double,name,initial)
 
c     Subroutine to make educated guesses about required array sizes:
c        maxnb2   = maximum number of level 2 branches
c        maxxdim  = maximum number of data variables in any one observation
c        maxnbdim = maximum dimension of nb which contains numbers of
c                   subbranches.
c
c     Also assign pointers to positions of all arrays in dd(), rr(),
c     ii(), ll(), and cc().
c
c     Constantijn Panis, 20 July 1999
 
      implicit none
      integer maxcc,datadim,levelmax,numvar,maxnb2,maxxdim,maxnbdim,nx1,
     *   xmean,xstd,xmin,xmax,x,x1copy,deduct,nx,namepntr,nbr,xbase,
     *   xpntr,nb,nblong,nbmax,branch,scratch,struc,ds,datastructure,
     *   datalevel,strucfreq,rect,double,name,initial,
     *   ccmax,nb2,xdim,nbdim,previous
      logical opened
 
c     Initial guess (without touching dimensions set by user):
      nb2=maxnb2
      xdim=maxxdim
      nbdim=maxnbdim
      if(nb2.eq.0)nb2=500
      if(xdim.eq.0)xdim=1000000
      if(nbdim.eq.0)nbdim=200
 
c     Assign pointers to the following arrays:
c     double precision xmin(numvar)
c                      xmax(numvar)
c                      xmean(numvar)
c                      xstd(numvar)
c     real x(maxxdim)
c          x1copy(2+nx1)
c          deduct(numvar)
c     integer nx(levelmax,datadim)
c             namepntr(levelmax,datadim)
c             nbr(levelmax,datadim)
c             xbase(numvar)
c             xpntr(levelmax,maxnb2)
c             nb(maxnbdim*maxnb2)
c             nblong(maxnbdim*maxnb2*2)
c             nbmax(2:levelmax,2:levelmax)
c             branch(2:levelmax)
c             scratch(maxnbdim)
c             struc(maxnb2)
c             ds(maxnb2)
c             datastructure(datadim)
c             datalevel(datadim)
c             strucfreq(datadim)
c             rect(datadim)
c             double(numvar)
c     character name(numvar)*8
c     logical initial(2:levelmax,datadim)
 
      previous=0
c     Assign pointers to arrays:
  100 if(nb2.eq.0.or.xdim.eq.0.or.nbdim.eq.0)goto 800
      ccmax=0
      call arraysize1(xmean,8,numvar,ccmax)
      call arraysize1(xstd,8,numvar,ccmax)
      call arraysize1(xmin,8,numvar,ccmax)
      call arraysize1(xmax,8,numvar,ccmax)
 
      call arraysize1(x,4,xdim,ccmax)
      call arraysize1(x1copy,4,2+nx1,ccmax)
      call arraysize1(deduct,4,numvar,ccmax)
 
      call arraysize1(nx,4,levelmax*datadim,ccmax)
      call arraysize1(namepntr,4,levelmax*datadim,ccmax)
      call arraysize1(nbr,4,levelmax*datadim,ccmax)
      call arraysize1(xbase,4,numvar,ccmax)
      call arraysize1(xpntr,4,levelmax*nb2,ccmax)
      call arraysize1(nb,4,nbdim*nb2,ccmax)
      call arraysize1(nblong,4,nbdim*nb2*2,ccmax)
      call arraysize1(nbmax,4,(levelmax-1)*(levelmax-1),ccmax)
      call arraysize1(branch,4,levelmax-1,ccmax)
      call arraysize1(scratch,4,nbdim,ccmax)
      call arraysize1(struc,4,nb2,ccmax)
      call arraysize1(ds,4,nb2,ccmax)
      call arraysize1(datastructure,4,datadim,ccmax)
      call arraysize1(datalevel,4,datadim,ccmax)
      call arraysize1(strucfreq,4,datadim,ccmax)
      call arraysize1(rect,4,datadim,ccmax)
      call arraysize1(double,4,numvar,ccmax)
 
      call arraysize1(name,1,8*numvar,ccmax)
      call arraysize1(initial,4,(levelmax-1)*datadim,ccmax)
 
c     Available space is maxcc; reduce if necessary:
      if(ccmax.gt.maxcc)then
         if(ccmax.eq.previous)goto 800
         previous=ccmax
         if(maxnb2.eq.0)nb2=(95*nb2)/100
         if(maxxdim.eq.0)xdim=(95*xdim)/100
         if(maxnbdim.eq.0)nbdim=(95*nbdim)/100
         goto 100
      endif
      maxnb2=nb2
      maxxdim=xdim
      maxnbdim=nbdim
      goto 999
 
  800 if(opened(8))then
         write(8,801)
         call writeaia(8,72,1,'   maximum number of level 2 '//
     *      'branches = ',nb2,';')
         call writeaia(8,72,1,'   maximum data space = ',xdim,';')
         call writeaia(8,72,1,'   maximum branch space = ',nbdim,';')
         write(8,802)
      endif
      write(6,801)
      call writeaia(6,72,1,'   maximum number of level 2 '//
     *   'branches = ',nb2,';')
      call writeaia(6,72,1,'   maximum data space = ',xdim,';')
      call writeaia(6,72,1,'   maximum branch space = ',nbdim,';')
      write(6,802)
  801 format(/,'ERROR: insufficient scratch space is available.  Try ',
     *   'reducing the size',/,'of allocated scratch arrays by ',
     *   'setting the following options',/,
     *   '(current values indicated):')
  802 format('If this fails, contact Technical Support ',
     *   '(support@applied-ml.com) for',/,'a complimentary ',
     *   'larger version.')
      stop
  999 return
      end
c=========================================================================
      subroutine arraysize1(array,numbytes,size,ccmax)
 
c     Subroutine to assign a pointer to an array's position
c     in cc(maxcc) or its equivalents (dd, rr, ii, ll).
c
c     Constantijn Panis, 20 July 1999
 
      implicit none
      integer array,numbytes,size,ccmax
 
c     Compute the next available position in cc(.)
      if(mod(ccmax,numbytes).eq.0)then
c        Array may start immediately following previous entry in cc(.)
         array=ccmax/numbytes+1
      else
c        Need to fill some unused bytes:
         array=ccmax/numbytes+2
      endif
 
c     Update ccmax, the number of bytes used in cc.
      ccmax=numbytes*(array+size-1)
 
      return
      end
