      subroutine checkrect(level,nbdim,nb,nbr,nx,xdim,badx,irrelvalue,
     *   x,iobs,id,struc,irrelcheck,scratch)
 
c     Subroutine to check that irrelevant variables in a rectangular
c     data structure are filled with some special number, like -99.
c     This subroutine also compresses the rectangular data and
c     the vector nb.
c
c     Note that x contains level 1+ variables.  It does not
c     contain the implicit first level 2 variable _branch2.
c
c     Constantijn Panis,  17 November 2000
c     Last change: X include X1.
 
      implicit none
      integer level,nbdim,nb(nbdim),nbr(level),nx(level),xdim,
     *   badx,irrelvalue,iobs,id,struc,pos1,pos2,pos1i,num,i,j,k,n,ll,
     *   countreal,intvecprod,scratch(nbdim),
     *   intvecmin,intvecmax,intlen,trm
      real x(xdim)
      logical irrelcheck
      character i2c*12,fmt*30
 
      if(level.lt.3)goto 999
 
c     First check that the numbers of subbranches make sense.  They
c     must be between 0 and the rectangular number, or may be equal
c     to -99.  If equal to -99, set to zero.
      if(nb(1).lt.0.or.nb(1).gt.nbr(3))goto 800
      pos2=2
      k=1
      do 100 ll=4,level
         do 110 i=1,intvecprod(nbr(3),ll-4)
c           The first nb(k) should be actual numbers of subbranches
            num=nb(k)
            pos1=pos2+(i-1)*nbr(ll-1)
            if(intvecmin(nb(pos1),num).lt.0.or.
     *         intvecmax(nb(pos1),num).gt.nbr(ll))goto 800
c           The next nbr(ll-1)-nb(k) should be 0 or -99
            do 120 j=pos1+num,pos1+nbr(ll-1)-1
               if(nb(j).eq.0)then
                  continue
               elseif(nb(j).eq.irrelvalue.or..not.irrelcheck)then
                  nb(j)=0
               else
                  goto 800
               endif
  120       continue
            k=k+1
  110    continue
         pos2=pos2+intvecprod(nbr(3),ll-3)
  100 continue
 
      if(irrelcheck)then
c        Then count number of relevant variables that are equal to irrelvalue
c        and check that irrelevant variables are always equal to irrelvalue.
c        pos1 points at position in x where relevant variables of level 3+ begin
c        pos2 is the number of variables to be checked for (ir)relevance,
         pos1=1+nx(1)+nx(2)
         k=0
         do 200 ll=3,level
            do 210 i=1,intvecprod(nbr(3),ll-3)
               k=k+1
c              Check the relevant values of the variables:
               num=nx(ll)*nb(k)
               badx = badx + countreal(real(irrelvalue),x(pos1),num)
c              Check the irrelevant values of the variables:
               if(nb(k).lt.nbr(ll))then
                  pos1i=pos1+num
                  num=nx(ll)*(nbr(ll)-nb(k))
                  if(countreal(real(irrelvalue),x(pos1i),num).ne.num)
     *               goto 810
               endif
               pos1=pos1+nx(ll)*nbr(ll)
  210       continue
  200    continue
      endif
 
c     Then compress the data vector
c     pos1 points at position in x where the next relevant variables
c          begin;
c     pos2 points at position in x where the next irrelevant variables
c          begin, which need to be replaced by relevant variables.
c     num  is the number of relevant variables folowing the position
c          defined by pos1.
      pos1=1+nx(1)+nx(2)+nx(3)*nbr(3)
      pos2=1+nx(1)+nx(2)+nx(3)*nb(1)
      k=1
      do 300 ll=4,level
         do 310 i=1,intvecprod(nbr(3),ll-3)
            k=k+1
            if(nb(k).eq.-99)goto 999
            num=nx(ll)*nb(k)
c           Copy the next relevant values at the place of the
c           irrelevant values:
            call copy4(x(pos1),x(pos2),num)
            pos1=pos1+nx(ll)*nbr(ll)
            pos2=pos2+num
  310    continue
  300 continue
 
c     Finally compress nb:
      call copyint(nb,scratch,nbdim)
      pos1=2+nbr(3)
      pos2=2+scratch(1)
      k=1
      do 400 ll=4,level-1
         do 410 i=1,intvecprod(nbr(3),ll-3)
            k=k+1
            call copyint(scratch(pos1),nb(pos2),scratch(k))
            pos2=pos2+scratch(k)
            pos1=pos1+nbr(ll)
  410    continue
  400 continue
      if(pos2.le.nbdim)nb(pos2)=-99
      goto 999
 
  800 if(struc.ne.0)then
         call writea3ia(8,72,1,'ERROR encountered in the number of '//
     *      'subbranches (data structure ',struc,', observation ',iobs,
     *      ', ID=',id,'):')
         call writea3ia(6,72,1,'ERROR encountered in the number of '//
     *      'subbranches (data structure ',struc,', observation ',iobs,
     *      ', ID=',id,'):')
      else
         call writea2ia(8,72,1,'ERROR encountered in the number of '//
     *      'subbranches (observation ',iobs,', ID=',id,'):')
         call writea2ia(6,72,1,'ERROR encountered in the number of '//
     *      'subbranches (observation ',iobs,', ID=',id,'):')
      endif
      pos2=0
      n=max(3,intlen(intvecmin(nb,nbdim)),intlen(intvecmax(nb,nbdim)))+1
      fmt='(3x,a,i2,a,i'//i2c(n,0)
      fmt=fmt(1:trm(fmt))//')'
      do 805 ll=3,level
         write(8,fmt)'Level',ll,': ',
     *      (nb(i),i=pos2+1,pos2+intvecprod(nbr(3),ll-3))
         write(6,fmt)'Level',ll,': ',
     *      (nb(i),i=pos2+1,pos2+intvecprod(nbr(3),ll-3))
         pos2=pos2+intvecprod(nbr(3),ll-3)
         fmt='(3x,a,i2,a,'//i2c(nbr(ll),0)
         fmt=fmt(1:trm(fmt))//'i'//i2c(n,0)
         fmt=fmt(1:trm(fmt))//',:,/,(12x,'//i2c(nbr(ll),0)
         fmt=fmt(1:trm(fmt))//'i'//i2c(n,0)
         fmt=fmt(1:trm(fmt))//'))'
  805 continue
      stop
  810 if(struc.eq.0)then
         call writea2ia(8,72,1,'ERROR: an irrelevant value of a level ',
     *      ll,' variable was not ',irrelvalue,'.')
         call writea2ia(6,72,1,'ERROR: an irrelevant value of a level ',
     *      ll,' variable was not ',irrelvalue,'.')
      else
         call writea3ia(8,72,1,'ERROR: an irrelevant value of a level ',
     *      ll,' variable in data structure ',struc,' was not ',
     *      irrelvalue,'.')
         call writea3ia(6,72,1,'ERROR: an irrelevant value of a level ',
     *      ll,' variable in data structure ',struc,' was not ',
     *      irrelvalue,'.')
      endif
      write(8,811)
      write(6,811)
  811 format('It is a very good idea to set irrelevant values in ',
     *   'rectangular data',/,'structures to some special value ',
     *   '(e.g., -99), so they can be tested',/,'here, but if such ',
     *   'test is not desired, specify "OPTION IRRELEVANCE',/,
     *   'CHECK = NO;" in the control file.  (Only aML gurus should ',
     *   'attempt',/,'this.)')
      call writea2ia(8,72,1,'The following variables were read '//
     *   '(observation=',iobs,', ID=',id,'):')
      call writea2ia(6,72,1,'The following variables were read '//
     *   '(observation=',iobs,', ID=',id,'):')
      write(8,'(/,a,$)')'Level 1: '
      write(6,'(/,a,$)')'Level 1: '
      do 812 j=1,nx(1)
         call writedble(8,dble(x(j)),1,-8,0,j/nx(1))
         call writedble(6,dble(x(j)),1,-8,0,j/nx(1))
  812 continue
      write(8,'(/,a,$)')'Level 2: '
      write(6,'(/,a,$)')'Level 2: '
      do 813 j=1,nx(2)
         call writedble(8,dble(x(nx(1)+j)),1,-8,0,j/nx(2))
         call writedble(6,dble(x(nx(1)+j)),1,-8,0,j/nx(2))
  813 continue
      pos1=nx(1)+nx(2)
      k=0
      do 814 ll=3,level
         do 815 i=1,intvecprod(nbr(3),ll-3)
            k=k+1
            do 816 n=1,nbr(ll)
               if(i*n.eq.1)write(8,'(/,a,i2,a,$)')'Level',ll,': '
               if(i*n.gt.1)write(8,'(a,$)')'         '
               if(i*n.eq.1)write(6,'(/,a,i2,a,$)')'Level',ll,': '
               if(i*n.gt.1)write(6,'(a,$)')'         '
               do 817 j=1,nx(ll)
                  call writedble(8,dble(x(pos1+j)),1,-8,0,0)
                  call writedble(6,dble(x(pos1+j)),1,-8,0,0)
  817          continue
               pos1=pos1+nx(ll)
               if(n.eq.1.and.nb(k).eq.0)then
                  write(8,'(a,i1,a)')'  [irrelevant; nb(',ll,')=0]'
                  write(6,'(a,i1,a)')'  [irrelevant; nb(',ll,')=0]'
               elseif(n.eq.1.and.nb(k).gt.0)then
                  write(8,'(a,i1,a,$)')'  [relevant; nb(',ll,')='
                  call writeint(8,nb(k),0,0)
                  write(8,'(a)')')]'
                  write(6,'(a,i1,a,$)')'  [relevant; nb(',ll,')='
                  call writeint(6,nb(k),0,0)
                  write(6,'(a)')')]'
               elseif(n.le.nb(k))then
                  write(8,'(a)')'  [relevant]'
                  write(6,'(a)')'  [relevant]'
               else
                  write(8,'(a)')'  [irrelevant]'
                  write(6,'(a)')'  [irrelevant]'
               endif
  816       continue
  815    continue
  814 continue
      write(8,818)
      write(6,818)
  818 format(/,'Very often this error message is the result of',/,
     *   '(1) an incorrect (too long or too short) variable list in ',
     *   'the control',/,'    file;',/,
     *   '(2) a missing value that is represented by ',
     *   'nothing, such as when',/,'    a Stata outfile statement ',
     *   'contains the comma option;',/, '(3) an incorrect ',
     *   'specification of the number of subbranches written.')
      stop
  999 return
      end
 
 
 
