      subroutine read_obs(oldid,id,nfiles,iobs,datadim,
     *   levelmax,maxnb2,nb2,maxxdim,xdim,maxnbdim,structure,ds,
     *   datalevel,struc,nx1,nx,nbr,nb,scratch,nblongdim,irrelvalue,
     *   badx,x1copy,x,rect,withid,irrelcheck,idsort,status)
 
c     Subroutine to read in an observation from ascii data files.
c
c     Modifired on 17 November 2000: Alpha compiler reads at least one
c     number even when the dimension of the to-be-read array is zero.
c     Reworked this routine to read just one array which always has
c     nonzero dimension.
c
c     Constantijn Panis, 17 November 2000
 
      implicit none
      integer oldid,id,nfiles,iobs,datadim,levelmax,maxnb2,nb2,
     *   maxxdim,xdim,maxnbdim,structure(datadim),ds(maxnb2),
     *   datalevel(datadim),struc(maxnb2),rect(datadim),
     *   nx1,nx(levelmax,datadim),scratch(maxnbdim),
     *   nbr(levelmax,datadim),nb(maxnbdim,maxnb2),nblongdim,
     *   nbdim,irrelvalue,badx,unit,dsib,ib2,level,posx,status,
     *   numcompx,dimnb,numbranch,trm
      double precision thisid,thisstruc
      real x1copy(nx1),x(maxxdim)
      logical withid,irrelcheck,idsort,realvecequal
      character line*128
 
      status=0
c     Read the next ID:
  100 call get_id(thisid,id,nfiles,iobs,unit,withid,status)
      if(status.eq.-1)goto 999
      if(status.eq.1)goto 860
      if(iobs.gt.1.and.id.lt.oldid)idsort=.false.
      oldid=id
 
c     Read in all records with this ID:
      ib2=0
      unit=21
c     The first 2 positions in x are for _id and _one:
      x(1)=real(id)
      x(2)=1.
      posx=3
 
c     nblongdim is dimension of nblong.  At least 2 for number of level 1
c     branches (always 1) and number of level 2 branches.
      nblongdim=2
 
  110 if(structure(1).eq.0)then
         read(unit,*,end=150,err=800)thisid
      else
         read(unit,*,end=150,err=800)thisid,thisstruc
      endif
      backspace unit
c     Make sure ID is integer:
      if(thisid.ne.int(thisid))goto 860
c     If IDs don't match try next file (if any)
      if(int(thisid).ne.id)goto 150
      ib2=ib2+1
      if(ib2.gt.maxnb2)goto 810
      if(structure(1).eq.0)then
         struc(ib2)=0
         ds(ib2)=1
         dsib=1
      else
         struc(ib2)=int(thisstruc)
c        Make sure structure is an integer:
         if(thisstruc.ne.int(thisstruc))goto 870
         call intmatch(int(thisstruc),structure,datadim,ds(ib2))
         if(ds(ib2).eq.0)goto 830
         dsib=ds(ib2)
      endif
 
c     Make space to insert ib2-counter before each level 2 branch.
      if(levelmax.gt.1)posx=posx+1
 
c     Read all data variables, including X1, into x, starting at x(posx).
      if(rect(dsib).eq.1)then
         call readrect(unit,id,struc(ib2),datalevel(dsib),nbr(1,dsib),
     *      nb(1,ib2),nx(1,dsib),maxnbdim,nbdim,
     *      maxxdim-posx+1,xdim,x(posx),status)
         if(status.ne.0)goto 800
         call checkrect(datalevel(dsib),nbdim,nb(1,ib2),nbr(1,dsib),
     *      nx(1,dsib),xdim,badx,irrelvalue,x(posx),
     *      iobs,id,struc(ib2),irrelcheck,scratch)
c        Number of values in compressed data vector:
         xdim=numcompx(datalevel(dsib),nx(1,dsib),nb(1,ib2))
      else
         call readcomp(unit,id,struc(ib2),datalevel(dsib),maxnbdim,
     *      nb(1,ib2),nx(1,dsib),maxxdim-posx+1,xdim,x(posx),
     *      status,line,iobs)
         if(status.ne.0)goto 800
      endif
 
c     Check that level 1 variables are the same in all levels 2 and move
c     data variables such that x contains the output data, with ib2-counters
c     (_branch2) for data integrity checks but without duplicate X1.
      if(ib2.eq.1)then
         call copy4(x(posx),x1copy,nx1)
         if(levelmax.gt.1)then
            call copy4(x(posx),x(posx-1),nx1)
            x(posx+nx1-1)=real(ib2)
         endif
         posx=posx+xdim
      else
         if(.not.realvecequal(x(posx),x1copy,nx1))goto 820
         if(levelmax.gt.1)then
            x(posx-1)=real(ib2)
            call copy4(x(posx+nx1),x(posx),xdim-nx1)
            posx=posx+xdim-nx1
         endif
      endif
 
c     Accumulate the total length of relevant elements of nb:
c     nb(1,ib2) is for number of level 3 branches, even if datalevel=2.
      nblongdim=nblongdim+1
c     Each of those branches yields another number of subbranches in nb:
      do 120 level=3,datalevel(dsib)-1
         nblongdim=nblongdim+numbranch(level,nb(1,ib2))
  120 continue
 
      goto 110
 
  150 if(unit.eq.20+nfiles)goto 200
      unit=unit+1
      goto 110
 
  200 nb2=ib2
 
c     Was there any match to this ID?  If not, try the next ID.
      if(nb2.eq.0)goto 100
 
c     Dimension of all variables (including X1):
      xdim=posx-1
      goto 999
 
  800 inquire(unit=unit,name=line)
      if(status.eq.0)then
c        0 = trouble reading id [and struc]; probably corrupted file
         call writea2ia(8,72,1,'ERROR encountered while reading '''//
     *      line(1:trm(line))//''' (observation ',iobs,', ID=',id,
     *      '), possibly because the raw data file is corrupt.')
         call writea2ia(6,72,1,'ERROR encountered while reading '''//
     *      line(1:trm(line))//''' (observation ',iobs,', ID=',id,
     *      '), possibly because the raw data file is corrupt.')
      elseif(status.eq.1)then
c        1 = ID or structure not what was expected
c        Highly unlikely if there are no data structures
         if(structure(1).eq.0)then
            status=0
            goto 800
         endif
         call writea3ia(8,72,1,'ERROR encountered while reading '''//
     *      line(1:trm(line))//''' (observation ',iobs,', ID=',id,
     *      '), probably because the data structure number (',
     *      struc(ib2),') is not on the same line as the ID, as is '//
     *      'required.')
         call writea3ia(6,72,1,'ERROR encountered while reading '''//
     *      line(1:trm(line))//''' (observation ',iobs,', ID=',id,
     *      '), probably because the data structure number (',
     *      struc(ib2),') is not on the same line as the ID, as is '//
     *      'required.')
      elseif(status.eq.2)then
c        2 = insufficient space available for nb or X
         if(rect(dsib).ne.1)nbdim=dimnb(datalevel(dsib),nb(1,ib2))
         if(nbdim.gt.maxnbdim)then
            call writeaia(8,71,1,'ERROR: maximum space allocated to '//
     *         'store numbers of subbranches in any one level 2 '//
     *         'branch (',maxnbdim,') is exceeded.')
            write(8,801)
            call writeaia(6,71,1,'ERROR: maximum space allocated to '//
     *         'store numbers of subbranches in any one level 2 '//
     *         'branch (',maxnbdim,') is exceeded.')
            write(6,801)
  801       format('Remedy: specify "option maximum branch space = ',
     *         'n;" in the control file,',/,'where is n sufficiently ',
     *         'high for your data.')
         endif
         if(xdim.gt.maxxdim-posx+1)then
            call writeaia(8,72,1,'ERROR: maximum space allocated to '//
     *         'store data values in any one observation (',maxxdim,
     *         ') is exceeded.')
            write(8,802)
            call writeaia(6,72,1,'ERROR: maximum space allocated to '//
     *         'store data values in any one observation (',maxxdim,
     *         ') is exceeded.')
            write(6,802)
  802       format('Remedy: specify "option maximum data space = n;" ',
     *         'in the control file,',/,'where is n sufficiently high ',
     *         'for your data.')
         endif
      elseif(status.eq.3.and.structure(1).eq.0)then
         call writea2ia(8,72,1,'ERROR or unexpected END-OF-FILE '//
     *      'encountered while reading '''//line(1:trm(line))//
     *      ''' (observation ',iobs,', ID=',id,').')
         write(8,803)
         call writea2ia(6,72,1,'ERROR or unexpected END-OF-FILE '//
     *      'encountered while reading '''//line(1:trm(line))//
     *      ''' (observation ',iobs,', ID=',id,').')
         write(6,803)
  803    format(/,'Common reasons for this error are:',/,'(1) The ',
     *      'data contain fewer variables than the .r2a control file',/,
     *      4x,'specifies.  In particular, the observation ID is ',
     *      'assumed to be',/,4x,'in the data and should not be ',
     *      'specified in the .r2a control file.',/,'(2) There are ',
     *      'missing values in the form of a period (".").')
         call findchar(unit)
      elseif(status.eq.3.and.structure(1).ne.0)then
         call writea3ia(8,72,1,'ERROR or unexpected END-OF-FILE '//
     *      'encountered while reading '''//line(1:trm(line))//
     *      ''' (observation ',iobs,', ID=',id,'), in particular '//
     *      'data structure ',struc(ib2),'.')
         write(8,803)
         call writea3ia(6,72,1,'ERROR or unexpected END-OF-FILE '//
     *      'encountered while reading '''//line(1:trm(line))//
     *      ''' (observation ',iobs,', ID=',id,'), in particular '//
     *      'data structure ',struc(ib2),'.')
         write(6,803)
         call findchar(unit)
      endif
      stop
  810 call writea3ia(8,72,1,'ERROR: the maximum number of level '//
     *   '2 branches (',maxnb2,') is exceeded in observation ',
     *   iobs,' (ID=',id,').')
      write(8,811)
      call writea3ia(6,72,1,'ERROR: the maximum number of level '//
     *   '2 branches (',maxnb2,') is exceeded in observation ',
     *   iobs,' (ID=',id,').')
      write(6,811)
  811 format('Remedy: specify "option maximum number of level 2 ',
     *   'branches = n;" in the',/,'control file, where is n ',
     *   'sufficiently high for your data.')
      stop
  820 call writea2ia(8,72,1,'ERROR:  level 1 variables must '//
     *   'be equal in all records of an observation.  This is '//
     *   'violated in observation ',iobs,' (ID=',id,').')
      call writea2ia(6,72,1,'ERROR:  level 1 variables must '//
     *   'be equal in all records of an observation.  This is '//
     *   'violated in observation ',iobs,' (ID=',id,').')
c     If levelmax>=3 and structure is supposed to be compressed,
c     suggest that perhaps it is rectangular:
      if(rect(dsib).ne.-1)stop
      write(8,821)
      write(6,821)
  821 format(/,'A potential cause of this error is that your raw data ',
     *   'are rectangular',/,'while you did not specify that.  The ',
     *   'raw2aml control file syntax for',/,'rectangular data is:',//,
     *   3x,'LEVEL 1 VAR = varlist;',/,
     *   3x,'LEVEL 2 VAR = varlist;  /* always one level 2 branch per ',
     *   'record */',/,3x,'LEVEL 3 VAR (NB=n3) = varlist;  /* n3 ',
     *   'level 3 branches */',/,3x,'LEVEL 4 VAR (NB=n4) = varlist;  ',
     *   '/* n4 level 4 branches */',/,3x,'et cetera...',/)
      stop
  830 call writea3ia(8,72,1,'ERROR: a record in observation ',iobs,
     *   ' (ID=',id,') has data structure = ',int(thisstruc),
     *   ';  this value is not accounted for in your control file '//
     *   'specifications.')
      call writea3ia(6,72,1,'ERROR: a record in observation ',iobs,
     *   ' (ID=',id,') has data structure = ',int(thisstruc),
     *   ';  this value is not accounted for in your control file '//
     *   'specifications.')
      stop
  860 call writeada(8,72,1,'ERROR: program attempted to read an ID '//
     *   'from an ascii data file.  IDs must always be integers, '//
     *   'but instead value ',8,thisid,' was read.  If your IDs '//
     *   'are all integers, this error is most likely caused by a '//
     *   'mismatch of the number of variables actually present in '//
     *   'the ascii file and the number listed in the raw2aml '//
     *   'control file.')
      call writeada(6,72,1,'ERROR: program attempted to read an ID '//
     *   'from an ascii data file.  IDs must always be integers, '//
     *   'but instead value ',8,thisid,' was read.  If your IDs '//
     *   'are all integers, this error is most likely caused by a '//
     *   'mismatch of the number of variables actually present in '//
     *   'the ascii file and the number listed in the raw2aml '//
     *   'control file.')
      goto 890
  870 call writeada(8,72,1,'ERROR: program attempted to read a data '//
     *   'structure number from an ascii data file.  The number read '//
     *   'is not an integer: ',8,thisstruc,'.  This error is most '//
     *   'likely the result of a mismatch of the numbers of '//
     *   'variables actually present in the ascii file and listed '//
     *   'in the raw2aml control file.')
      call writeada(6,72,1,'ERROR: program attempted to read a data '//
     *   'structure number from an ascii data file.  The number read '//
     *   'is not an integer: ',8,thisstruc,'.  This error is most '//
     *   'likely the result of a mismatch of the numbers of '//
     *   'variables actually present in the ascii file and listed '//
     *   'in the raw2aml control file.')
      goto 890
  890 call writefileline(unit,1,1)
c     Check for non-numeric data in the line:
      call findchar(unit)
      stop
 
c     Observation has been read successfully.
  999 return
      end
