      subroutine scanr2a(datadim,levelmax,numvar,maxnb2,maxxdim,
     *   maxnbdim,nx1,deviations,cross,irrelcheck,irrelvalue,maxobs,
     *   withid,il,line)
 
c     Subroutine to scan the .r2a file, read all options, open the output
c     files, and figure out:
c        datadim  = number of data structures;
c        levelmax = maximum number of levels;
c        numvar   = number of variables
c
c     Other dimensional parameters are (user-defined options):
c        maxnb2   = maximum number of level 2 branches
c        maxxdim  = maximum number of data variables in any one observation
c        maxnbdim = maximum dimension of nb which contains numbers of
c                   subbranches.
c
c     Constantijn Panis, 20 July 1999
 
      implicit none
      integer datadim,levelmax,numvar,maxnb2,maxxdim,maxnbdim,
     *   nx1,irrelvalue,maxobs,il,previl,level,prevlevel,ii,status
      logical deviations,irrelcheck,cross,withid,smtxt,opened
      character line*128
 
c     Option defaults:
c        deviations: no
c        compute means across data structures: yes
c        check irrelevant variables: yes (-99)
c        number of observations: all
c        ID file: nonexistent
      deviations=.false.
      cross=.true.
      irrelcheck=.true.
      irrelvalue=-99
      maxobs=-1
      withid=.false.
 
c     Dimensional parameters:
      datadim=0
      levelmax=0
c     _id and _one:
      numvar=2
      maxnb2=0
      maxxdim=0
      maxnbdim=0
c     Nx1 needs to be initialized in case there is no level 1 var list.
c     Nx1 is the number of level 1 variables in the data, excluding _id
c     and _one, which needs to be read in from ascii data.
      nx1=0
 
      read(4,'(a80)',end=800,err=800)line
      il=1
      prevlevel=0
 
  100 if(smtxt(4,line,il,'DATA STRUCTURE'))then
         datadim=datadim+1
         call gotochar(4,';',line,il,status)
         if(status.gt.0)goto 200
         prevlevel=1
      elseif(smtxt(4,line,il,'VARiables='))then
c        Shortcut in case there is only one level:
         if(levelmax.gt.0)goto 810
         levelmax=-1
         if(datadim.ne.0)goto 810
         datadim=1
c        Numvar counts number of output variables; nx1 is number in
c        ascii data, i.e., without _id and _one.
         call countvars(numvar,line,il)
         nx1=numvar-2
      elseif(smtxt(4,line,il,'LEVEL'))then
         if(levelmax.eq.-1)goto 810
         previl=il
         call getint(4,level,line,il,status)
         if(status.gt.0)goto 820
         if(level.lt.1)goto 820
         levelmax=max(levelmax,level)
         if(.not.smtxt(4,line,il,'VARiables'))goto 850
         if(smtxt(4,line,il,'(NB='))then
c           Data are rectangular with nb branches
            call getint(4,ii,line,il,status)
            if(status.gt.0)goto 850
            if(.not.smtxt(4,line,il,')'))goto 850
         endif
         if(.not.smtxt(4,line,il,'='))goto 850
         call countvars(numvar,line,il)
c        Account for _branch2:
         if(level.ge.2.and.prevlevel.le.1)numvar=numvar+1
         prevlevel=level
c        Numvar counts number of output variables; nx1 is number in
c        ascii data, i.e., without _id and _one.
         if(level.eq.1)nx1=numvar-2
      elseif(smtxt(4,line,il,'OPTIONs'))then
         call read_option(maxobs,maxnb2,maxxdim,maxnbdim,
     *      deviations,irrelcheck,irrelvalue,cross,line,il)
      else
         call gotochar(4,';',line,il,status)
         if(status.gt.0)goto 200
      endif
      goto 100
 
  200 if(levelmax.eq.0)goto 810
      if(levelmax.eq.-1)levelmax=1
      datadim=max(1,datadim)
      goto 999
 
  800 write(6,801)
  801 format(/,'ERROR or END-OF-FILE encountered while reading the ',
     *   'control file.',/,'It is probably empty.')
      call r2asyntax()
      stop
  810 write(6,811)
      if(opened(8))write(8,811)
  811 format(/,'ERROR encountered in the level specification(s) ',
     *   'of your data.')
      call r2asyntax()
      stop
  820 il=min(il,previl)
      call caretline(6,line,il)
      write(6,821)
      if(opened(8))then
         call caretline(8,line,il)
         write(8,821)
      endif
  821 format('ERROR: positive integer expected.')
      stop
  850 call caretline(6,line,il)
      write(6,851)
      if(opened(8))then
         call caretline(8,line,il)
         write(8,851)
      endif
  851 format(/,'Syntax error in the control file.')
      call r2asyntax()
      stop
  999 return
      end
