      subroutine writedoc1(levelmax,datadim,nvar,nfiles,struc,
     *   strucfreq,datalevel,nx,namepntr,nobs,xbase,nlen,r2aversion,
     *   nbmax,xmean,xstd,xmin,xmax,deduct,withid,name,date)
 
c     Subroutine to write out the main part of user-readable
c     documentation.  The number of observations, [a frequency table
c     of data structures,] maximum numbers of branches per observations,
c     et cetera, are written, as well as a list of all variables with
c     their means, standard deviations, minima and maxima.  The means
c     written here are the means _within_ data structures.  If summary
c     statistics across data structures are desired (which is the
c     default), crossstats will compute them and writedoc2 will write
c     them out.
c     Note that the means that are written out in the data file are
c     within data structure means if cross-means are not desired, and
c     cross-means by default.
c
c     Constantijn Panis, 13 May 1998
 
      implicit none
      integer levelmax,datadim,nvar,nfiles,struc(datadim),
     *   strucfreq(datadim),datalevel(datadim),nx(levelmax,datadim),
     *   namepntr(levelmax,datadim),nbmax(levelmax-1,2:levelmax),
     *   nobs,xbase(nvar),r2aversion,pos,i,ll,dim,n,
     *   novariance,unit,basemax,nlen,trm,intlen,intvecmax
      double precision xmean(nvar),xstd(nvar),xmin(nvar),xmax(nvar)
      real deduct(nvar)
      logical withid
      character name(nvar)*8,date*24,filename*128,fmt1*36,fmt2*36,i2c*12
 
      inquire(unit=13,name=filename)
      write(8,101)filename(1:trm(filename))
      write(6,101)filename(1:trm(filename))
  101 format('Documentation for ''',a,'''')
      call timestamp(date)
      write(8,102)date,.01*(r2aversion/10)
      write(6,102)date,.01*(r2aversion/10)
  102 format('Created on ',a24,' with raw2aml version',f5.2,'.')
      if(withid)then
         inquire(unit=20,name=filename)
         write(8,103)filename(1:trm(filename))
         write(6,103)filename(1:trm(filename))
  103    format('ID file: ''',a,'''')
      elseif(nfiles.gt.1)then
         write(8,104)
         write(6,104)
  104    format('(No separate ID file was used.)')
      endif
      inquire(unit=21,name=filename)
      if(nfiles.eq.1)then
         write(8,105)filename(1:trm(filename))
         write(6,105)filename(1:trm(filename))
  105    format('Ascii data set: ''',a,'''')
      else
         write(8,106)filename(1:trm(filename))
         write(6,106)filename(1:trm(filename))
  106    format('Ascii data sets: ''',a,'''')
         do 100 unit=22,20+nfiles
            inquire(unit=unit,name=filename)
            write(8,'(17x,a)')''''//filename(1:trm(filename))//''''
            write(6,'(17x,a)')''''//filename(1:trm(filename))//''''
  100    continue
      endif
      write(8,107)nobs
      write(6,107)nobs
  107 format(/,'Number of observations:',i8)
      if(levelmax.gt.1)then
         write(8,111)(ll,nbmax(1,ll),ll=2,levelmax)
         write(6,111)(ll,nbmax(1,ll),ll=2,levelmax)
      endif
      if(levelmax.gt.2)then
         write(8,112)((ll,i,nbmax(i,ll),ll=i+1,levelmax),i=2,levelmax-1)
         write(6,112)((ll,i,nbmax(i,ll),ll=i+1,levelmax),i=2,levelmax-1)
      endif
  111 format('Maximum number of level',i2,
     *   ' branches in any observation:',i7)
  112 format('Maximum number of level',i2,' branches in any level',
     *   i2,' branch:',i4)
 
      if(struc(1).ne.0)call writetable('STRUCTURE',struc,
     *   strucfreq,datadim)
 
c     Number of spaces allowed for summary statistic N depends on
c     number needed.
      basemax=intvecmax(xbase,nvar)
      nlen=intlen(basemax)+2
c     Format for "Variable N Mean Std Dev Min Max" is something like
c        (a,9x,a,7x,a,6x,a,10x,a,10x,a)
      fmt1='(a,'//i2c(nlen-1,0)
      fmt1=fmt1(1:trm(fmt1))//'x,a,7x,a,4x,a,8x,a,8x,a)'
 
      unit=8
c     Level 1 variables (including _id but not _one):
  200 write(unit,'(/,60(''-''),//,a)')'LEVEL 1 VARIABLES:'
      write(unit,fmt1)'Variable','N','Mean','Std Dev','Min','Max'
      call write_entry(unit,deduct(1),nobs,nlen,
     *   xmean(1),xstd(1),xmin(1),xmax(1),name(1))
      do 210 i=3,nx(1,1)+2
         call write_entry(unit,deduct(i),nobs,nlen,
     *      xmean(i),xstd(i),xmin(i),xmax(i),name(i))
  210 continue
      do 220 dim=1,datadim
         if(struc(dim).ne.0)then
            fmt2='(/,18(''-''),1x,a,1x,i'//i2c(intlen(struc(dim)),0)
            n=25-intlen(struc(dim))
            fmt2=fmt2(1:trm(fmt2))//',1x,'//i2c(n,0)
            fmt2=fmt2(1:trm(fmt2))//'(''-''))'
            write(unit,fmt2)'DATA STRUCTURE',struc(dim)
         endif
         do 230 ll=2,datalevel(dim)
            write(unit,'(1x)')
            pos=namepntr(ll,dim)
            if(ll.eq.2)pos=pos+1
            if(nx(ll,dim).eq.0)call writeaia(unit,0,1,'LEVEL ',
     *         ll,' VARIABLES:  none')
            if(nx(ll,dim).eq.0)goto 230
            call writeaia(unit,0,1,'LEVEL ',ll,' VARIABLES:')
            write(unit,fmt1)'Variable','N','Mean','Std Dev','Min','Max'
            do 240 i=pos,pos+nx(ll,dim)-1
               call write_entry(unit,deduct(i),xbase(i),nlen,
     *            xmean(i),xstd(i),xmin(i),xmax(i),name(i))
  240       continue
  230    continue
  220 continue
 
      write(unit,'(/,60(''-''))')
 
c     Are there variables without variation?  Don't count _id, _one,
c     and _branch2:
      novariance=0
      do 260 i=3,nvar
         if(name(i).eq.'_branch2')goto 260
         if(xmin(i).eq.xmax(i))novariance=novariance+1
  260 continue
      if(novariance.eq.0)then
         write(unit,261)
  261    format(/,'NOTE: there is variation in all data variables.')
      else
         write(unit,'(1x)')
         if(datadim.eq.1)call writeaia(unit,72,1,'WARNING: there are ',
     *      novariance,' data variables without any variation.')
         if(datadim.gt.1)call writeaia(unit,72,1,'NOTE: there are ',
     *      novariance,' data variables without any variation.')
      endif
 
      if(unit.eq.6)goto 999
      unit=6
      goto 200
 
  999 return
      end
