      subroutine writedoc2(nvar,xbase,nlen,xmean,xstd,xmin,xmax,
     *   double,name,varywarn)
 
c     Some variables may appear in multiple data structures.  The
c     summary statistics of these variables were computed in subroutine
c     crossstats.  Subroutine writesys wrote them to the .dat file;
c     this routine writes them in user-readable form, and warns if
c     some variables are of different variability across data structures.
c
c     Note: the original variable names are destroyed in this routine.
c
c     Constantijn Panis, 18 January 2000
 
      implicit none
      integer nvar,xbase(nvar),nlen,double(nvar),i,j,zerostd,n,trm
      double precision xmean(nvar),xstd(nvar),xmin(nvar),xmax(nvar)
      character name(nvar)*8,varywarn*8,fmt1*36,i2c*12
 
c     Are there any variable names appearing in multiple data structures?
      n=0
      do 100 i=1,nvar
         if(name(i).eq.'_branch2')goto 100
         if(double(i).eq.i)n=n+1
  100 continue
      if(n.eq.0)then
         write(8,'(/,a)')'NOTE: all variable names are unique.'
         write(6,'(/,a)')'NOTE: all variable names are unique.'
         goto 999
      endif
 
c     Format for "Variable N Mean Std Dev Min Max" is something like
c        (a,9x,a,7x,a,6x,a,10x,a,10x,a)
      fmt1='(a,'//i2c(nlen-1,0)
      fmt1=fmt1(1:trm(fmt1))//'x,a,7x,a,4x,a,8x,a,8x,a)'
 
      write(8,'(1x)')
      write(6,'(1x)')
      call writeaia(8,72,1,'NOTE: the following ',n,' variables '//
     *   'appear in multiple data structures.  Summary statistics '//
     *   'across all data structures in which they appear:')
      call writeaia(6,72,1,'NOTE: the following ',n,' variables '//
     *   'appear in multiple data structures.  Summary statistics '//
     *   'across all data structures in which they appear:')
      write(8,'(1x)')
      write(6,'(1x)')
      write(8,fmt1)'Variable','N','Mean','Std Dev','Min','Max'
      write(6,fmt1)'Variable','N','Mean','Std Dev','Min','Max'
 
      do 200 i=1,nvar
c        Is this variable the first occurrence of a double?
         if(double(i).ne.i)goto 200
         if(name(i).eq.'_branch2')goto 200
         call write_entry(8,0.,xbase(i),nlen,
     *      xmean(i),xstd(i),xmin(i),xmax(i),name(i))
         call write_entry(6,0.,xbase(i),nlen,
     *      xmean(i),xstd(i),xmin(i),xmax(i),name(i))
  200 continue
 
c     Warn if there are variables (other than _id, _one, _branch2)
c     without any variation; the array with variable names, name, is
c     destroyed here:
      j=0
      do 230 i=3,nvar
         if(xmin(i).ne.xmax(i))goto 230
         if(name(i).eq.'_branch2')goto 230
         j=j+1
         name(j)=name(i)
  230 continue
      call char8sort(name,j,.true.,zerostd)
      write(8,'(1x)')
      write(6,'(1x)')
      if(zerostd.eq.0)then
         write(8,202)
         write(6,202)
  202    format('NOTE: there is variation in all data variables.')
      else
         call writeaia(8,72,1,'WARNING: after computing summary '//
     *      'statistics across all data structures in which each '//
     *      'variable appears, there are still ',zerostd,
     *      ' data variables without any variation.')
         call writeaia(6,72,1,'WARNING: after computing summary '//
     *      'statistics across all data structures in which each '//
     *      'variable appears, there are still ',zerostd,
     *      ' data variables without any variation.')
      endif
 
      if(varywarn.ne.'(((())))')write(8,204)varywarn(1:trm(varywarn))
      if(varywarn.ne.'(((())))')write(6,204)varywarn(1:trm(varywarn))
  204 format(/,'WARNING: there is at least one variable whose name ',
     *   'appears at unequal',/,'levels in multiple data structures.  ',
     *   'One such variable is ''',a,'''.')
 
  999 return
      end
