      subroutine prepall(task,nfull,nfullplus,neh,nel,nep,nec,neb,nen,
     *   nepo,neml,nneml,nihmax,numdist,numarma,dimlam,vpntr1,vpntr4,
     *   vpntr6,vpntr5,vpntr5n,vpntr5po,vpntr7,dimv,mp1,mp4,mp6,mp5,mp7,
     *   rsh3max,rsh4max,rsh5max,rspmax,rscmax,rsnamax,rsnpmax,rsmlmax,
     *   nmapmoel,fullbeta,dfullbeta,lnlobs,lam,dlam,s,ds,
     *   ssg,xbh3,d1xbh3,d2xbh3,xbh4,xbh5,gamma,d1gamma,d2gamma,lnla,
     *   lnlb,dlnla,dlnlb,xblp,dxblp,tauminxb,dtau,xbc,dxbc,a5,da5,p5,
     *   dp5,xbml,dxbml,stda4,v,xnode,distdim,lamtyp,lampos,mapmoel,
     *   refspl,beginspl,nodepos,numnodes,spltype,ifdrv,iflam,ipt,jpt,
     *   status)
 
c     Subroutine to compute as much as possible in all models that
c     does not depend on numerical integration issues.
c
c     Throughout aML, there are seven covariance matrices and seven load matrice
c     hence lamtyp(numdist,7), lampos(numdist,7).  They are:
c        1 = hazard models
c        2 = logit models
c        3 = probit models
c        4 = continuous models
c        5 = binomial models
c        6 = negative binomial models
c        7 = Poisson
c        8 = multinomial logit
c
c     Constantijn Panis, 2 December 2002
c     Last change:  status
 
      implicit none
      integer task,nfull,nfullplus,neh,nel,nep,nec,neb,nen,nepo,neml,
     *   nneml,nihmax,numdist,numarma,dimlam,vpntr1,vpntr4,vpntr6,
     *   vpntr5,vpntr5n,vpntr5po,vpntr7,mp1,mp4,mp6,mp5,mp7,
     *   dimv,rsh3max,rsh4max,rsh5max,rspmax,rscmax,rsnamax,rsnpmax,
     *   rsmlmax,nmapmoel,ipt,jpt,status
      double precision fullbeta(nfullplus),dfullbeta(nfullplus),
     *   lnlobs,lam(dimlam),dlam(dimlam),xbh3(nihmax,0:rsh3max),
     *   xbh4(rsh4max,neh),xbh5(rsh5max),d1xbh3(nihmax),d2xbh3(nihmax),
     *   xbml(nneml,0:rsmlmax),dxbml(nneml,0:1),
     *   s(0:nihmax),ds(0:nihmax,*),ssg(nihmax),gamma(*),
     *   d1gamma(*),d2gamma(*),lnla(neh),lnlb(neh),dlnla(nfullplus,neh),
     *   dlnlb(nfullplus,neh),xblp(nel+nep,0:rspmax),dxblp(nel+nep,0:1),
     *   tauminxb(2,nel+nep),dtau(2,nel+nep),xbc(nec,0:rscmax),
     *   dxbc(nec,0:1),a5(nen,0:rsnamax),da5(nen),
     *   p5(neb+nen+nepo,0:rsnpmax),dp5(neb+nen+nepo),stda4(nep)
      real v(dimv),xnode(*)
      integer distdim(numdist),lamtyp(numdist,8),lampos(numdist,8),
     *   mapmoel(nmapmoel),refspl(*),beginspl(*),nodepos(*),numnodes(*),
     *   spltype(*),vpntr,moelpos,i
      logical ifdrv,iflam(numdist),ifany
 
c     Continuous: compute regressor sets and either accumulate them into
c     BX or place them in Lambda6, to load residual distributions.
      vpntr=vpntr6
      moelpos=mp6
      if(nec.gt.0)call prep6(task,nec,nfullplus,numarma,nmapmoel,
     *   dimv,mapmoel,moelpos,rscmax,numdist,distdim,dimlam,
     *   lamtyp(1,4),lampos(1,4),xbc,dxbc,fullbeta,dfullbeta,lam,dlam,
     *   v,vpntr,ipt,jpt)
 
c     Hazard: compute products over (S(i+1)/S(i))**exp(B'X(i))
      vpntr=vpntr1
      moelpos=mp1
      if(neh.gt.0)call prep1(task,nfullplus,neh,nihmax,numdist,
     *   dimlam,distdim,lampos(1,1),nmapmoel,dimv,
     *   rsh3max,rsh4max,rsh5max,fullbeta,dfullbeta,gamma,xbh3,xbh4,
     *   xbh5,lam,dlam,s,ds,ssg,d1gamma,d2gamma,lnla,lnlb,d1xbh3,
     *   d2xbh3,dlnla,dlnlb,v,vpntr,xnode,mapmoel,moelpos,refspl,
     *   beginspl,nodepos,numnodes,spltype,ifdrv,ipt,jpt)
 
c     Logit/probit: compute tau1-alpha'X and tau2-alpha'X for each
c     outcome and figure the load matrices on residual distributions.
      vpntr=vpntr4
      moelpos=mp4
      if(nel.gt.0)call prep4l(task,nel,nel+nep,nfull,nfullplus,nmapmoel,
     *   dimv,mapmoel,moelpos,numdist,distdim,dimlam,lamtyp(1,2),
     *   lampos(1,2),rspmax,xblp,dxblp,fullbeta,dfullbeta,lam,
     *   dlam,tauminxb,dtau,v,vpntr,ipt,jpt)
      if(nep.gt.0)call prep4p(task,nel,nep,nel+nep,nfull,nfullplus,
     *   nmapmoel,dimv,mapmoel,moelpos,numdist,distdim,dimlam,
     *   lamtyp(1,3),lampos(1,3),rspmax,xblp,dxblp,stda4,fullbeta,
     *   dfullbeta,lam,dlam,tauminxb,dtau,v,vpntr,ipt,jpt)
 
c     Binomial: compute the log of product over outcomes of
c     exposure-choose-outcome, probability locations (p5) and the
c     load matrix of probability integrated residuals (lam).
      vpntr=vpntr5
      moelpos=mp5
      if(neb.gt.0)call prep5b(task,nfullplus,nmapmoel,neb,neb+nen+nepo,
     *   numdist,dimlam,distdim,lamtyp(1,5),lampos(1,5),
     *   dimv,mapmoel,moelpos,rsnpmax,fullbeta,dfullbeta,
     *   p5,dp5,lam,dlam,lnlobs,v,vpntr,ipt,jpt)
 
c     Negative Binomial: compute the log of product over outcomes of
c     of Gamma(A+Y)/(Gamma(A)*Gamma(Y+1)), probability locations (p5)
c     and the load matrix of probability integrated residuals (lam).
c     Negative binomial outcomes start in v(vpntr5n); needed in onepoint.
      vpntr5n=vpntr
      if(nen.gt.0)call prep5n(task,nfullplus,nmapmoel,nen,neb+nen+nepo,
     *   numdist,dimlam,distdim,lamtyp(1,6),lampos(1,6),
     *   dimv,mapmoel,moelpos,rsnamax,rsnpmax,fullbeta,dfullbeta,
     *   a5,da5,p5(neb+1,0),dp5(neb+1),lam,dlam,lnlobs,v,vpntr,
     *   ipt,jpt,status)
      if(status.ne.0)goto 999
 
c     Poisson: compute locations of incidence rate, accumulate
c     log(outcome!), compute load matrix of integrated residuals.
c     Poisson outcomes start in v(vpntr5po); needed in onepoint.
      vpntr5po=vpntr
      if(nepo.gt.0)call prep5po(task,nfullplus,nmapmoel,nepo,
     *   neb+nen+nepo,numdist,dimlam,distdim,lamtyp(1,7),lampos(1,7),
     *   dimv,mapmoel,moelpos,rsnpmax,fullbeta,dfullbeta,
     *   p5(neb+nen+1,0),dp5(neb+nen+1),lam,dlam,v,vpntr,ipt,jpt)
 
c     Multinomial logit:
      vpntr=vpntr7
      moelpos=mp7
      if(neml.gt.0)call prep71(task,neml,nneml,nfullplus,nmapmoel,dimv,
     *   mapmoel,moelpos,rsmlmax,numdist,distdim,dimlam,lamtyp(1,8),
     *   lampos(1,8),xbml,dxbml,fullbeta,dfullbeta,lam,dlam,v,vpntr,
     *   ipt,jpt)
 
c     Lambdas may be sparse, in which case only column numbers and
c     possibly nonzero elements are stored.  Copy the structure of
c     sparse Lambdas into their derivatives.  Load matrices on
c     integrated distributions are never sparse.  Note that dlam and
c     ddlam are numnest deep.  However, ddlam is only used for load
c     matrices on integrated distributions, and the numnest-1 levels
c     below dlam(.,0) also only for integrated distributions.  (All
c     derivatives of non-integrated distributions are carried in
c     dsiga4a4/ddsiga4a4 and dsiga6a6/ddsiga6a6 and unraveled in compsig44
c     and compsig66 into dlam(.,0).)  We therefore only need to copy
c     sparse structures of non-integrated distributions into dlam(.,0).
c     Note that dlam and ddlam are only assigned space if any load matrix
c     element is estimated (see figure_pos).
      if(.not.(task.eq.1.and.ifdrv))goto 999
      if(.not.ifany(iflam,numdist))goto 999
      do 100 i=1,numdist
         if(lamtyp(i,3).eq.4)call
     *      copy8(lam(lampos(i,3)+1),dlam(lampos(i,3)+1),nep)
         if(lamtyp(i,4).eq.4)call
     *      copy8(lam(lampos(i,4)+1),dlam(lampos(i,4)+1),nec)
  100 continue
 
  999 return
      end
