      subroutine m2mult300(numblocka,dimblocka,ncolc,ablock,b,c,factor)
 
c     Subroutine to multiply two matrices A*B' = C.  This routine
c     is similar to m2mult, but exploits special matrix properties.
c     Matrix types can be:
c        0 = generic
c        1 = identity (Only its dimension is stored.)
c        2 = symmetric
c        3 = symmetric and block-diagonal (Only its first block, its
c            dimension, and number of blocks are stored.)
c        4 = sparse in the sense that there is at most one nonzero
c            element in any one row.  (Only its nonzero elements
c            and their column position are stored.)
c
c     In this routine, A is block-diagonal, B generic, and C generic.
c     A consists of numblocka blocks of dimension (dimblocka,dimblocka).
c     Only the first block is stored, as ablock(dimblocka,dimblocka).
c
c     Conceptual dimensions:
c
c            A(dima,dima)   B(ncolc,dima)   C(dima,ncolc)
c
c     Integer factor determines:
c        factor = 0     C = A*B
c              else     C = C + factor*A*B
c
c     Constantijn Panis, 2 December 2001
 
      implicit none
      integer numblocka,dimblocka,ncolc,factor,i,j,k,ii,kk,dima
      double precision ablock(dimblocka,dimblocka),
     *   b(ncolc,numblocka*dimblocka),c(numblocka*dimblocka,ncolc),abij
 
      dima=numblocka*dimblocka
      if(factor.eq.0)call zerodp(dima*ncolc,c)
 
      do 100 i=1,dima
         ii=mod(i-1,dimblocka)+1
         do 200 j=1,ncolc
            abij=0.d0
            do 300 k=1,dima
               if((i-1)/dimblocka.ne.(k-1)/dimblocka)goto 300
               kk=mod(k-1,dimblocka)+1
               abij=abij+ablock(ii,kk)*b(j,k)
  300       continue
            if(factor.ne.0.and.factor.ne.1)abij=dble(factor)*abij
            c(i,j)=c(i,j)+abij
  200    continue
  100 continue
 
      return
      end
