      subroutine m3mult0300(task,nrowa,numblockb,dimblockb,ncolc,
     *   a,bblock,c,d,factor)
 
c     Subroutine to multiply matrices A*B*C = D.  This routine
c     is similar to m3mult, but exploits special matrix properties.
c     Matrix types can be:
c        0 = generic
c        1 = identity (Only its dimension is stored.)
c        2 = symmetric
c        3 = symmetric and block-diagonal (Only its first block, its
c            dimension, and number of blocks are stored.)
c        4 = sparse in the sense that there is at most one nonzero
c            element in any one row.  (Only its nonzero elements
c            and their column position are stored.)
c
c     In this routine, A is generic, B is block-diagonal, C and D generic.
c     B consists of numblockc blocks of dimension (dimblockb,dimblockb).
c     Only the first block is stored, as bblock(dimblockb,dimblockb).
c
c     Conceptual dimensions:
c            A(nrowa,dimb)   B(dimb,dimb)   C(dimb,ncolc)   D(nrowa,ncolc)
c     where dimb=numblockb*dimblockb
c
c     The four matrices may be in transposed format.  Integer task
c     may be 1000,0100,0010, et cetera, depending on which matrices
c     are transposed.  The first digit refers to A; the second to B
c     (irrelevant because B is symmetric); the third to C; the fourth to D.
c     For example,
c        task = 1100 means that A and B are transposed: A'*B'*C = D
c        task =   10 means that C is transposed: A*B*C' = D
c
c     Dimensions as given above assume no transpositions; if a matrix
c     is transposed, its dimensions really are reversed.
c
c     Integer factor determines:
c        factor = 0     D = A*B*C
c              else     D = D + factor*A*B*C
c
c     Constantijn Panis, 3 December 2001
 
      implicit none
      integer task,nrowa,numblockb,dimblockb,ncolc,factor,dimb,numzero,
     *   i,j,k,l,ia,ic,id,ja,jc,jd,sa,sc,sd,jsa,jsc,jsd
      double precision a(nrowa*numblockb*dimblockb),
     *   bblock(dimblockb,dimblockb),
     *   c(numblockb*dimblockb*ncolc),d(nrowa*ncolc),abij
 
      dimb=numblockb*dimblockb
      if(factor.eq.0)call zerodp(nrowa*ncolc,d)
 
      if(mod(task/1000,10).eq.0)then
         ja=nrowa
         jsa=1
      else
         ja=1
         jsa=dimb
      endif
c     B is symmetric
      if(mod(task/10,10).eq.0)then
         jc=dimb
         jsc=1
      else
         jc=1
         jsc=ncolc
      endif
      if(mod(task,10).eq.0)then
         jd=nrowa
         jsd=1
      else
         jd=1
         jsd=ncolc
      endif
 
      sa=1
      sd=1
      do 110 i=1,nrowa
         sc=1
         do 120 j=1,dimb
            abij = 0.d0
            ic=sc
            id=sd
c           AB(i,j) is the innerproduct of A(i,.) and B(.,j).  First numzero
c           elements of B(.,j) are zero, followed by dimblockb nonzero elements.
            numzero=((j-1)/dimblockb)*dimblockb
            ia=sa+numzero*ja
            abij=0.d0
            do 130 k=1,dimblockb
               abij = abij + a(ia)*bblock(k,j-numzero)
               ia=ia+ja
  130       continue
            if(factor.ne.0.and.factor.ne.1)abij=dble(factor)*abij
            do 140 l=1,ncolc
               d(id) = d(id) + abij*c(ic)
               id=id+jd
               ic=ic+jc
  140       continue
            sc=sc+jsc
  120    continue
         sa=sa+jsa
         sd=sd+jsd
  110 continue
 
      return
      end
