      subroutine m3mult4340(numrowa,numblockb,dimblockb,numrowc,
     *   asparse,bblock,csparse,d,factor)
 
c     Subroutine to multiply three matrices A*B*C' = D.  This routine
c     is similar to m3mult, but exploits special matrix properties.
c     Matrix types can be:
c        0 = generic
c        1 = identity (Only its dimension is stored.)
c        2 = symmetric
c        3 = symmetric and block-diagonal (Only its first block, its
c            dimension, and number of blocks are stored.)
c        4 = sparse in the sense that there is at most one nonzero
c            element in any one row.  (Only its nonzero elements
c            and their column position are stored.)
c
c     In this routine, A is sparse, B is block-diagonal, C is sparse, D generic.
c     A is stored as two column vectors.  The first contains the column
c     number, the second the actual element.  In other words, A(i,.)=0
c     except A(i,as(i,1)) = as(i,2).  If as(i,1)<=0, all A(i,.)=0.
c     C is stored analogously.
c
c     B consists of numblockc blocks of dimension (dimblockb,dimblockb).
c     Only the first block is stored, as bblock(dimblockb,dimblockb).
c
c     Conceptual dimensions:
c        A(numrowa,dimb)    -- sparse
c        B(dimb,dimb)       -- symmetric and block-diagonal
c        C(numrowc,dimb)    -- sparse
c        D(numrowa,numrowc) -- generic
c
c     where dimb = numblockb*dimblockb
c
c     Integer factor determines:
c        factor = 0     D = A*B*C'
c              else     D = D + factor*A*B*C'
c
c     Constantijn Panis, 29 November 2001
 
      implicit none
      integer numrowa,numblockb,dimblockb,numrowc,
     *   factor,rowd,cold,rowb,colb,i,j
      double precision asparse(numrowa,2),csparse(numrowc,2),
     *   bblock(dimblockb,dimblockb),d(numrowa,numrowc)
 
      if(factor.eq.0)call zerodp(numrowa*numrowc,d)
      do 100 cold=1,numrowc
         colb=int(csparse(cold,1))
         if(colb.le.0)goto 100
         do 200 rowd=1,numrowa
            rowb=int(asparse(rowd,1))
            if(rowb.le.0)goto 200
            if((colb-1)/dimblockb.ne.(rowb-1)/dimblockb)goto 200
            i=mod(rowb-1,dimblockb)+1
            j=mod(colb-1,dimblockb)+1
            if(factor.eq.0.or.factor.eq.1)then
               d(rowd,cold) = d(rowd,cold) +
     *            asparse(rowd,2)*bblock(i,j)*csparse(cold,2)
            else
               d(rowd,cold) = d(rowd,cold) + dble(factor)*
     *            asparse(rowd,2)*bblock(i,j)*csparse(cold,2)
            endif
  200    continue
  100 continue
 
      return
      end
 
